@@ -23,7 +23,7 @@ class CControllerMenuPopup extends CController {
 
 	protected function checkInput() {
 		$fields = [
-			'type' => 'required|in dashboard,history,host,item,item_prototype,map_element,refresh,trigger,trigger_macro',
+			'type' => 'required|in history,host,item,item_prototype,map_element,refresh,trigger,trigger_macro,widget_actions',
 			'data' => 'array'
 		];
 
@@ -35,7 +35,7 @@ class CControllerMenuPopup extends CController {
 				$output['errors'] = $messages->toString();
 			}
 
-			$this->setResponse(new CControllerResponseData(['main_block' => CJs::encodeJson($output)]));
+			$this->setResponse(new CControllerResponseData(['main_block' => json_encode($output)]));
 		}
 
 		return $ret;
@@ -45,39 +45,6 @@ class CControllerMenuPopup extends CController {
 		return true;
 	}
 
-	/**
-	 * Prepare data for dashboard context menu popup.
-	 *
-	 * @param array  $data
-	 * @param string $data['dashboardid']
-	 *
-	 * @return mixed
-	 */
-	private static function getMenuDataDashboard(array $data) {
-		$db_dashboards = API::Dashboard()->get([
-			'output' => [],
-			'dashboardids' => $data['dashboardid'],
-		]);
-
-		if ($db_dashboards) {
-			$db_dashboard = $db_dashboards[0];
-
-			return [
-				'type' => 'dashboard',
-				'dashboardid' => $data['dashboardid'],
-				'editable' => (bool) API::Dashboard()->get([
-					'output' => [],
-					'dashboardids' => $data['dashboardid'],
-					'editable' => true
-				])
-			];
-		}
-
-		error(_('No permissions to referred object or it does not exist!'));
-
-		return null;
-	}
-
 	/**
 	 * Prepare data for history context menu popup.
 	 *
@@ -99,7 +66,8 @@ class CControllerMenuPopup extends CController {
 			return [
 				'type' => 'history',
 				'itemid' => $data['itemid'],
-				'hasLatestGraphs' => in_array($db_item['value_type'], [ITEM_VALUE_TYPE_UINT64, ITEM_VALUE_TYPE_FLOAT])
+				'hasLatestGraphs' => in_array($db_item['value_type'], [ITEM_VALUE_TYPE_UINT64, ITEM_VALUE_TYPE_FLOAT]),
+				'allowed_ui_latest_data' => CWebUser::checkAccess(CRoleHelper::UI_MONITORING_LATEST_DATA)
 			];
 		}
 
@@ -128,20 +96,31 @@ class CControllerMenuPopup extends CController {
 
 		$db_hosts = $has_goto
 			? API::Host()->get([
-				'output' => ['status'],
+				'output' => ['hostid', 'status'],
 				'selectGraphs' => API_OUTPUT_COUNT,
-				'selectScreens' => API_OUTPUT_COUNT,
+				'selectHttpTests' => API_OUTPUT_COUNT,
 				'hostids' => $data['hostid']
 			])
 			: API::Host()->get([
-				'output' => [],
+				'output' => ['hostid'],
 				'hostids' => $data['hostid']
 			]);
 
 		if ($db_hosts) {
 			$db_host = $db_hosts[0];
+			$rw_hosts = false;
 
-			$scripts = API::Script()->getScriptsByHosts([$data['hostid']])[$data['hostid']];
+			if ($has_goto && CWebUser::getType() > USER_TYPE_ZABBIX_USER) {
+				$rw_hosts = (bool) API::Host()->get([
+					'output' => [],
+					'hostids' => $db_host['hostid'],
+					'editable' => true
+				]);
+			}
+
+			$scripts = CWebUser::checkAccess(CRoleHelper::ACTIONS_EXECUTE_SCRIPTS)
+				? API::Script()->getScriptsByHosts([$data['hostid']])[$data['hostid']]
+				: [];
 
 			foreach ($scripts as &$script) {
 				$script['name'] = trimPath($script['name']);
@@ -153,15 +132,22 @@ class CControllerMenuPopup extends CController {
 			$menu_data = [
 				'type' => 'host',
 				'hostid' => $data['hostid'],
-				'hasGoTo' => (bool) $has_goto
+				'hasGoTo' => (bool) $has_goto,
+				'allowed_ui_inventory' => CWebUser::checkAccess(CRoleHelper::UI_INVENTORY_HOSTS),
+				'allowed_ui_latest_data' => CWebUser::checkAccess(CRoleHelper::UI_MONITORING_LATEST_DATA),
+				'allowed_ui_problems' => CWebUser::checkAccess(CRoleHelper::UI_MONITORING_PROBLEMS),
+				'allowed_ui_hosts' => CWebUser::checkAccess(CRoleHelper::UI_MONITORING_HOSTS),
+				'allowed_ui_conf_hosts' => CWebUser::checkAccess(CRoleHelper::UI_CONFIGURATION_HOSTS)
 			];
 
 			if ($has_goto) {
 				$menu_data['showGraphs'] = (bool) $db_host['graphs'];
-				$menu_data['showScreens'] = (bool) $db_host['screens'];
+				$menu_data['showDashboards'] = (bool) getHostDashboards($data['hostid']);
+				$menu_data['showWeb'] = (bool) $db_host['httpTests'];
+				$menu_data['isWriteable'] = $rw_hosts;
 				$menu_data['showTriggers'] = ($db_host['status'] == HOST_STATUS_MONITORED);
 				if (array_key_exists('severity_min', $data)) {
-					$menu_data['severity_min'] = $data['severity_min'];
+					$menu_data['severities'] = array_column(getSeverities($data['severity_min']), 'value');
 				}
 				if (array_key_exists('show_suppressed', $data)) {
 					$menu_data['show_suppressed'] = $data['show_suppressed'];
@@ -322,6 +308,8 @@ class CControllerMenuPopup extends CController {
 		}
 
 		CArrayHelper::sort($selement['urls'], ['name']);
+		$selement['urls'] = array_values($selement['urls']);
+
 		// Prepare urls for processing in menupopup.js.
 		$selement['urls'] = CArrayHelper::renameObjectsKeys($selement['urls'], ['name' => 'label']);
 
@@ -375,7 +363,8 @@ class CControllerMenuPopup extends CController {
 					case SYSMAP_ELEMENT_TYPE_MAP:
 						$menu_data = [
 							'type' => 'map_element_submap',
-							'sysmapid' => $selement['elements'][0]['sysmapid']
+							'sysmapid' => $selement['elements'][0]['sysmapid'],
+							'allowed_ui_maps' => CWebUser::checkAccess(CRoleHelper::UI_MONITORING_MAPS)
 						];
 						if (array_key_exists('severity_min', $data)) {
 							$menu_data['severity_min'] = $data['severity_min'];
@@ -391,10 +380,11 @@ class CControllerMenuPopup extends CController {
 					case SYSMAP_ELEMENT_TYPE_HOST_GROUP:
 						$menu_data = [
 							'type' => 'map_element_group',
-							'groupid' => $selement['elements'][0]['groupid']
+							'groupid' => $selement['elements'][0]['groupid'],
+							'allowed_ui_problems' => CWebUser::checkAccess(CRoleHelper::UI_MONITORING_PROBLEMS)
 						];
 						if (array_key_exists('severity_min', $data)) {
-							$menu_data['severity_min'] = $data['severity_min'];
+							$menu_data['severities'] = array_column(getSeverities($data['severity_min']), 'value');
 						}
 						if ($db_map['show_suppressed']) {
 							$menu_data['show_suppressed'] = true;
@@ -428,10 +418,11 @@ class CControllerMenuPopup extends CController {
 					case SYSMAP_ELEMENT_TYPE_TRIGGER:
 						$menu_data = [
 							'type' => 'map_element_trigger',
-							'triggerids' => zbx_objectValues($selement['elements'], 'triggerid')
+							'triggerids' => zbx_objectValues($selement['elements'], 'triggerid'),
+							'allowed_ui_problems' => CWebUser::checkAccess(CRoleHelper::UI_MONITORING_PROBLEMS)
 						];
 						if (array_key_exists('severity_min', $data)) {
-							$menu_data['severity_min'] = $data['severity_min'];
+							$menu_data['severities'] = array_column(getSeverities($data['severity_min']), 'value');
 						}
 						if ($db_map['show_suppressed']) {
 							$menu_data['show_suppressed'] = true;
@@ -490,8 +481,7 @@ class CControllerMenuPopup extends CController {
 	 * @param array  $data
 	 * @param string $data['triggerid']
 	 * @param string $data['eventid']                 (optional) Mandatory for Acknowledge menu.
-	 * @param array  $data['acknowledge']             (optional) Acknowledge link parameters.
-	 * @param string $data['acknowledge']['backurl']
+	 * @param bool   $data['acknowledge']             (optional) Whether to show Acknowledge menu.
 	 * @param int    $data['severity_min']            (optional)
 	 * @param bool   $data['show_suppressed']         (optional)
 	 * @param array  $data['urls']                    (optional)
@@ -560,7 +550,13 @@ class CControllerMenuPopup extends CController {
 				'triggerid' => $data['triggerid'],
 				'items' => $items,
 				'showEvents' => $show_events,
-				'configuration' => in_array(CWebUser::$data['type'], [USER_TYPE_ZABBIX_ADMIN, USER_TYPE_SUPER_ADMIN])
+				'allowed_ui_problems' => CWebUser::checkAccess(CRoleHelper::UI_MONITORING_PROBLEMS),
+				'allowed_ack' => CWebUser::checkAccess(CRoleHelper::ACTIONS_ACKNOWLEDGE_PROBLEMS)
+						|| CWebUser::checkAccess(CRoleHelper::ACTIONS_CLOSE_PROBLEMS)
+						|| CWebUser::checkAccess(CRoleHelper::ACTIONS_CHANGE_SEVERITY)
+						|| CWebUser::checkAccess(CRoleHelper::ACTIONS_ADD_PROBLEM_COMMENTS),
+				'allowed_ui_conf_hosts' => CWebUser::checkAccess(CRoleHelper::UI_CONFIGURATION_HOSTS),
+				'allowed_ui_latest_data' => CWebUser::checkAccess(CRoleHelper::UI_MONITORING_LATEST_DATA)
 			];
 
 			if ($db_trigger['url'] !== '') {
@@ -602,7 +598,7 @@ class CControllerMenuPopup extends CController {
 			}
 
 			if (array_key_exists('acknowledge', $data)) {
-				$menu_data['acknowledge'] = $data['acknowledge'];
+				$menu_data['acknowledge'] = (bool) $data['acknowledge'];
 			}
 
 			return $menu_data;
@@ -622,14 +618,53 @@ class CControllerMenuPopup extends CController {
 		return ['type' => 'trigger_macro'];
 	}
 
+	/**
+	 * Prepare data for widget actions menu popup.
+	 *
+	 * @param array  $data
+	 * @param string $data['widget_uniqueid']  Widget instance unique id.
+	 * @param string $data['currentRate']      Refresh rate for widget.
+	 * @param bool   $data['multiplier']       Multiplier or time mode.
+	 *
+	 * @return mixed
+	 */
+	private static function getMenuDataWidgetActions(array $data) {
+		$menu_data = [
+			'type' => 'widget_actions',
+			'widget_uniqueid' => $data['widget_uniqueid'],
+			'currentRate' => $data['currentRate'],
+			'multiplier' => (bool) $data['multiplier']
+		];
+
+		if ($data['widgetType'] == WIDGET_SVG_GRAPH) {
+			$menu_data['download'] = true;
+		}
+
+		if (array_key_exists('params', $data)) {
+			$menu_data['params'] = $data['params'];
+		}
+
+		if ($data['widgetType'] == WIDGET_GRAPH && array_key_exists('graphid', $data) && $data['graphid']) {
+			$menu_data['download'] = (bool) API::Graph()->get([
+				'output' => ['graphid'],
+				'graphids' => $data['graphid']
+			]);
+		}
+		elseif ($data['widgetType'] == WIDGET_GRAPH && array_key_exists('itemid', $data) && $data['itemid']) {
+			$menu_data['download'] = (bool) API::Item()->get([
+				'output' => ['itemid'],
+				'itemids' => $data['itemid'],
+				'webitems' => true
+			]);
+		}
+
+		return $menu_data;
+	}
+
 	protected function doAction() {
 		$data = $this->hasInput('data') ? $this->getInput('data') : [];
 
 		switch ($this->getInput('type')) {
-			case 'dashboard':
-				$menu_data = self::getMenuDataDashboard($data);
-				break;
-
 			case 'history':
 				$menu_data = self::getMenuDataHistory($data);
 				break;
@@ -661,6 +696,10 @@ class CControllerMenuPopup extends CController {
 			case 'trigger_macro':
 				$menu_data = self::getMenuDataTriggerMacro();
 				break;
+
+			case 'widget_actions':
+				$menu_data = self::getMenuDataWidgetActions($data);
+				break;
 		}
 
 		$output = [];
@@ -673,6 +712,6 @@ class CControllerMenuPopup extends CController {
 			$output['errors'] = $messages->toString();
 		}
 
-		$this->setResponse(new CControllerResponseData(['main_block' => CJs::encodeJson($output)]));
+		$this->setResponse(new CControllerResponseData(['main_block' => json_encode($output)]));
 	}
 }
