@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2019 Zabbix SIA
+** Copyright (C) 2001-2020 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -22,9 +22,11 @@
 class CControllerPopupTriggerExpr extends CController {
 	private $metrics = [];
 	private $param1SecCount = [];
+	private $param1Period = [];
 	private $param1Sec = [];
 	private $param1Str = [];
 	private $param2SecCount = [];
+	private $param2SecMode = [];
 	private $param3SecVal = [];
 	private $param3SecPercent = [];
 	private $paramSecIntCount = [];
@@ -66,6 +68,19 @@ class CControllerPopupTriggerExpr extends CController {
 			]
 		];
 
+		$this->param1Period = [
+			'last' => [
+				'C' => _('Last of').' (T)',
+				'T' => T_ZBX_INT,
+				'A' => true
+			],
+			'period_shift' => [
+				'C' => _('Period shift'),
+				'T' => T_ZBX_INT,
+				'A' => true
+			]
+		];
+
 		$this->param1Sec = [
 			'last' => [
 				'C' => _('Last of').' (T)',
@@ -96,6 +111,19 @@ class CControllerPopupTriggerExpr extends CController {
 			]
 		];
 
+		$this->param2SecMode = [
+			'last' => [
+				'C' => _('Last of').' (T)',
+				'T' => T_ZBX_INT,
+				'A' => true
+			],
+			'mode' => [
+				'C' => 'Mode',
+				'T' => T_ZBX_STR,
+				'A' => false
+			]
+		];
+
 		$this->param3SecVal = [
 			'last' => [
 				'C' => _('Last of').' (T)',
@@ -380,8 +408,8 @@ class CControllerPopupTriggerExpr extends CController {
 				'operators' => ['=', '<>', '>', '<', '>=', '<=']
 			],
 			'nodata' => [
-				'description' => _('nodata() - No data received during period of time T (1 - true, 0 - false)'),
-				'params' => $this->param1Sec,
+				'description' => _('nodata() - No data received during period of time T (1 - true, 0 - false), Mode (strict - ignore proxy time delay in sending data)'),
+				'params' => $this->param2SecMode,
 				'allowed_types' => $this->allowedTypesAny,
 				'operators' => ['=', '<>']
 			],
@@ -402,6 +430,42 @@ class CControllerPopupTriggerExpr extends CController {
 				'params' => $this->paramTimeleft,
 				'allowed_types' => $this->allowedTypesNumeric,
 				'operators' => ['=', '<>', '>', '<', '>=', '<=']
+			],
+			'trendavg' => [
+				'description' => _('trendavg() - Average value of a period T with exact period shift'),
+				'params' => $this->param1Period,
+				'allowed_types' => $this->allowedTypesNumeric,
+				'operators' => ['=', '<>', '>', '<', '>=', '<=']
+			],
+			'trendcount' => [
+				'description' => _('trendcount() - Number of successfully retrieved values V (which fulfill operator O) for period T with exact period shift'),
+				'params' => $this->param1Period,
+				'allowed_types' => $this->allowedTypesAny,
+				'operators' => ['=', '<>', '>', '<', '>=', '<=']
+			],
+			'trenddelta' => [
+				'description' => _('trenddelta() - Difference between MAX and MIN value of a period T with exact period shift'),
+				'params' => $this->param1Period,
+				'allowed_types' => $this->allowedTypesNumeric,
+				'operators' => ['=', '<>', '>', '<', '>=', '<=']
+			],
+			'trendmax' => [
+				'description' => _('trendmax() - Maximum value for period T with exact period shift'),
+				'params' => $this->param1Period,
+				'allowed_types' => $this->allowedTypesNumeric,
+				'operators' => ['=', '<>', '>', '<', '>=', '<=']
+			],
+			'trendmin' => [
+				'description' => _('trendmin() - Minimum value for period T with exact period shift'),
+				'params' => $this->param1Period,
+				'allowed_types' => $this->allowedTypesNumeric,
+				'operators' => ['=', '<>', '>', '<', '>=', '<=']
+			],
+			'trendsum' => [
+				'description' => _('trendsum() - Sum of values of a period T with exact period shift'),
+				'params' => $this->param1Period,
+				'allowed_types' => $this->allowedTypesNumeric,
+				'operators' => ['=', '<>', '>', '<', '>=', '<=']
 			]
 		];
 
@@ -441,7 +505,7 @@ class CControllerPopupTriggerExpr extends CController {
 
 			if ($this->hasInput('add')) {
 				$this->setResponse(
-					(new CControllerResponseData(['main_block' => CJs::encodeJson($output)]))->disableView()
+					(new CControllerResponseData(['main_block' => json_encode($output)]))->disableView()
 				);
 			}
 			else {
@@ -475,7 +539,7 @@ class CControllerPopupTriggerExpr extends CController {
 
 			if ($result) {
 				$function_macro_tokens = $result->getTokensByType(
-					CTriggerExpressionParserResult::TOKEN_TYPE_FUNCTION_MACRO
+					CTriggerExprParserResult::TOKEN_TYPE_FUNCTION_MACRO
 				);
 
 				if ($function_macro_tokens) {
@@ -501,14 +565,26 @@ class CControllerPopupTriggerExpr extends CController {
 					 */
 					$tokens = $result->getTokens();
 					foreach ($tokens as $key => $token) {
-						if ($token['type'] == CTriggerExpressionParserResult::TOKEN_TYPE_FUNCTION_MACRO) {
+						if ($token['type'] == CTriggerExprParserResult::TOKEN_TYPE_FUNCTION_MACRO) {
 							if (array_key_exists($key + 2, $tokens)
-									&& $tokens[$key + 1]['type'] == CTriggerExpressionParserResult::TOKEN_TYPE_OPERATOR
+									&& $tokens[$key + 1]['type'] == CTriggerExprParserResult::TOKEN_TYPE_OPERATOR
 									&& array_key_exists($function, $this->functions)
 									&& in_array($tokens[$key + 1]['value'],
 										$this->functions[$function]['operators'])) {
 								$operator = $tokens[$key + 1]['value'];
-								$value = $tokens[$key + 2]['value'];
+
+								$value = '';
+								$i = 2;
+
+								if (array_key_exists($key + 3, $tokens)
+										&& $tokens[$key + 2]['type'] == CTriggerExprParserResult::TOKEN_TYPE_OPERATOR) {
+									$value .= $tokens[$key + 2]['value'];
+									$i++;
+								}
+
+								$value .= ($tokens[$key + $i]['type'] == CTriggerExprParserResult::TOKEN_TYPE_STRING)
+									? $tokens[$key + $i]['data']['string']
+									: $tokens[$key + $i]['value'];
 							}
 							else {
 								break;
@@ -584,7 +660,7 @@ class CControllerPopupTriggerExpr extends CController {
 			'value' => $value,
 			'params' => $params,
 			'paramtype' => $param_type,
-			'description' => $description,
+			'item_description' => $description,
 			'functions' => $this->functions,
 			'function' => $function,
 			'operator' => $operator,
@@ -625,7 +701,7 @@ class CControllerPopupTriggerExpr extends CController {
 		// Create and validate trigger expression before inserting it into textarea field.
 		if ($this->getInput('add', false)) {
 			try {
-				if ($data['description']) {
+				if ($data['item_description']) {
 					if ($data['paramtype'] == PARAM_TYPE_COUNTS
 							&& array_key_exists('last', $data['params'])
 							&& $data['params']['last'] !== '') {
@@ -649,7 +725,7 @@ class CControllerPopupTriggerExpr extends CController {
 						$function,
 						rtrim(implode(',', $quoted_params), ','),
 						$operator,
-						$data['value']
+						CTriggerExpression::quoteString($data['value'])
 					);
 
 					// Validate trigger expression.
@@ -705,7 +781,7 @@ class CControllerPopupTriggerExpr extends CController {
 			}
 
 			$this->setResponse(
-				(new CControllerResponseData(['main_block' => CJs::encodeJson($output)]))->disableView()
+				(new CControllerResponseData(['main_block' => json_encode($output)]))->disableView()
 			);
 		}
 		else {
