@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2019 Zabbix SIA
+** Copyright (C) 2001-2020 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -40,15 +40,22 @@ class CControllerWidgetNavTreeView extends CControllerWidget {
 
 	protected function getNumberOfProblemsBySysmap(array $navtree_items = []) {
 		$response = [];
-		$sysmapids = array_keys(array_flip(zbx_objectValues($navtree_items, 'mapid')));
+		$sysmapids = [];
 
-		$sysmaps = API::Map()->get([
-			'sysmapids' => $sysmapids,
-			'preservekeys' => true,
-			'output' => ['sysmapid', 'severity_min'],
-			'selectLinks' => ['linktriggers', 'permission'],
-			'selectSelements' => ['elements', 'elementtype', 'permission']
-		]);
+		foreach ($navtree_items as $navtree_item) {
+			$sysmapids[$navtree_item['sysmapid']] = true;
+		}
+		unset($sysmapids[0]);
+
+		$sysmaps = $sysmapids
+			? API::Map()->get([
+				'output' => ['sysmapid', 'severity_min'],
+				'selectLinks' => ['linktriggers', 'permission'],
+				'selectSelements' => ['elements', 'elementtype', 'permission'],
+				'sysmapids' => array_keys($sysmapids),
+				'preservekeys' => true
+			])
+			: [];
 
 		if ($sysmaps) {
 			$triggers_per_hosts = [];
@@ -75,11 +82,11 @@ class CControllerWidgetNavTreeView extends CControllerWidget {
 			$sysmaps_resolved = array_keys($sysmaps);
 			while ($diff = array_diff($submaps_found, $sysmaps_resolved)) {
 				$submaps = API::Map()->get([
-					'sysmapids' => $diff,
-					'preservekeys' => true,
 					'output' => ['sysmapid', 'severity_min'],
 					'selectLinks' => ['linktriggers', 'permission'],
-					'selectSelements' => ['elements', 'elementtype', 'permission']
+					'selectSelements' => ['elements', 'elementtype', 'permission'],
+					'sysmapids' => $diff,
+					'preservekeys' => true
 				]);
 
 				$sysmaps_resolved = array_merge($sysmaps_resolved, $diff);
@@ -116,11 +123,13 @@ class CControllerWidgetNavTreeView extends CControllerWidget {
 								$host_groups[$element['groupid']] = true;
 							}
 							break;
+
 						case SYSMAP_ELEMENT_TYPE_TRIGGER:
 							foreach (zbx_objectValues($selement['elements'], 'triggerid') as $triggerid) {
 								$problems_per_trigger[$triggerid] = $this->problems_per_severity_tpl;
 							}
 							break;
+
 						case SYSMAP_ELEMENT_TYPE_HOST:
 							if (($element = reset($selement['elements'])) !== false) {
 								$hosts[$element['hostid']] = true;
@@ -178,9 +187,9 @@ class CControllerWidgetNavTreeView extends CControllerWidget {
 			if ($problems_per_trigger) {
 				$triggers = API::Trigger()->get([
 					'output' => [],
+					'selectGroups' => ['groupid'],
 					'triggerids' => array_keys($problems_per_trigger),
 					'skipDependent' => true,
-					'selectGroups' => ['groupid'],
 					'preservekeys' => true,
 					'monitored' => true
 				]);
@@ -202,20 +211,18 @@ class CControllerWidgetNavTreeView extends CControllerWidget {
 			}
 
 			// Count problems in each submap included in navigation tree:
-			foreach ($navtree_items as $itemid => $item_details) {
-				$maps_need_to_count_in = $item_details['children_mapids'];
-				if ($item_details['mapid']) {
-					$maps_need_to_count_in[] = $item_details['mapid'];
+			foreach ($navtree_items as $id => $navtree_item) {
+				$maps_need_to_count_in = $navtree_item['child_sysmapids'];
+				if ($navtree_item['sysmapid'] != 0) {
+					$maps_need_to_count_in[$navtree_item['sysmapid']] = true;
 				}
 
-				$maps_need_to_count_in = array_keys(array_flip($maps_need_to_count_in));
-
-				$response[$itemid] = $this->problems_per_severity_tpl;
+				$response[$id] = $this->problems_per_severity_tpl;
 				$problems_counted = [];
 
-				foreach ($maps_need_to_count_in as $mapid) {
-					if (array_key_exists($mapid, $sysmaps)) {
-						$map = $sysmaps[$mapid];
+				foreach (array_keys($maps_need_to_count_in) as $sysmapid) {
+					if (array_key_exists($sysmapid, $sysmaps)) {
+						$map = $sysmaps[$sysmapid];
 
 						// Count problems occurred in linked elements.
 						foreach ($map['selements'] as $selement) {
@@ -225,36 +232,35 @@ class CControllerWidgetNavTreeView extends CControllerWidget {
 									$triggers_per_host_groups
 								);
 
-								if (is_array($problems)) {
-									$response[$itemid] = array_map(function () {
-										return array_sum(func_get_args());
-									}, $response[$itemid], $problems);
+								if ($problems !== null) {
+									$response[$id] = self::sumArrayValues($response[$id], $problems);
 								}
 							}
 						}
 
 						// Count problems occurred in triggers which are related to links.
 						foreach ($map['links'] as $link) {
-							foreach ($link['linktriggers'] as $lt) {
-								if (!array_key_exists($lt['triggerid'], $problems_counted)) {
-									$problems_to_add = $problems_per_trigger[$lt['triggerid']];
-									$problems_counted[$lt['triggerid']] = true;
+							$uncounted_problem_triggers = array_diff_key(
+								array_flip(zbx_objectValues($link['linktriggers'], 'triggerid')),
+								$problems_counted
+							);
 
-									// Remove problems which are less important than map's min-severity.
-									if ($map['severity_min'] > 0) {
-										foreach ($problems_to_add as $sev => $probl) {
-											if ($map['severity_min'] > $sev) {
-												$problems_to_add[$sev] = 0;
-											}
+							foreach ($uncounted_problem_triggers as $triggerid => $var) {
+								$problems_to_add = $problems_per_trigger[$triggerid];
+								$problems_counted[$triggerid] = true;
+
+								// Remove problems which are less important than map's min-severity.
+								if ($map['severity_min'] > 0) {
+									foreach ($problems_to_add as $sev => $probl) {
+										if ($map['severity_min'] > $sev) {
+											$problems_to_add[$sev] = 0;
 										}
 									}
-
-									// Sum problems.
-									$response[$itemid] = array_map(function() {
-										return array_sum(func_get_args());
-									}, $problems_to_add, $response[$itemid]);
 								}
+
+								$response[$id] = self::sumArrayValues($response[$id], $problems_to_add);
 							}
+							unset($uncounted_problem_triggers);
 						}
 					}
 				}
@@ -283,46 +289,48 @@ class CControllerWidgetNavTreeView extends CControllerWidget {
 
 				if (($element = reset($selement['elements'])) !== false) {
 					if (array_key_exists($element['groupid'], $triggers_per_host_groups)) {
-						foreach ($triggers_per_host_groups[$element['groupid']] as $triggerid => $val) {
-							if (!array_key_exists($triggerid, $problems_counted)) {
-								$problems_counted[$triggerid] = true;
-								$problems = array_map(function() {
-									return array_sum(func_get_args());
-								}, $problems_per_trigger[$triggerid], $problems);
-							}
+						$uncounted_problem_triggers = array_diff_key($triggers_per_host_groups[$element['groupid']],
+							$problems_counted
+						);
+						foreach ($uncounted_problem_triggers as $triggerid => $var) {
+							$problems_counted[$triggerid] = true;
+							$problems = self::sumArrayValues($problems, $problems_per_trigger[$triggerid]);
 						}
+						unset($uncounted_problem_triggers);
 					}
 				}
 				break;
+
 			case SYSMAP_ELEMENT_TYPE_TRIGGER:
 				$problems = $this->problems_per_severity_tpl;
-
-				foreach (zbx_objectValues($selement['elements'], 'triggerid') as $triggerid) {
-					if (!array_key_exists($triggerid, $problems_counted)) {
-						$problems_counted[$triggerid] = true;
-
-						$problems = array_map(function() {
-							return array_sum(func_get_args());
-						}, $problems_per_trigger[$triggerid], $problems);
-					}
+				$uncounted_problem_triggers = array_diff_key(
+					array_flip(zbx_objectValues($selement['elements'], 'triggerid')),
+					$problems_counted
+				);
+				foreach ($uncounted_problem_triggers as $triggerid => $var) {
+					$problems_counted[$triggerid] = true;
+					$problems = self::sumArrayValues($problems, $problems_per_trigger[$triggerid]);
 				}
+				unset($uncounted_problem_triggers);
 				break;
+
 			case SYSMAP_ELEMENT_TYPE_HOST:
 				$problems = $this->problems_per_severity_tpl;
 
 				if (($element = reset($selement['elements'])) !== false) {
 					if (array_key_exists($element['hostid'], $triggers_per_hosts)) {
-						foreach ($triggers_per_hosts[$element['hostid']] as $triggerid => $val) {
-							if (!array_key_exists($triggerid, $problems_counted)) {
-								$problems_counted[$triggerid] = true;
-								$problems = array_map(function() {
-									return array_sum(func_get_args());
-								}, $problems_per_trigger[$triggerid], $problems);
-							}
+						$uncounted_problem_triggers = array_diff_key($triggers_per_hosts[$element['hostid']],
+							$problems_counted
+						);
+						foreach ($uncounted_problem_triggers as $triggerid => $var) {
+							$problems_counted[$triggerid] = true;
+							$problems = self::sumArrayValues($problems, $problems_per_trigger[$triggerid]);
 						}
+						unset($uncounted_problem_triggers);
 					}
 				}
 				break;
+
 			case SYSMAP_ELEMENT_TYPE_MAP:
 				$problems = $this->problems_per_severity_tpl;
 
@@ -350,15 +358,14 @@ class CControllerWidgetNavTreeView extends CControllerWidget {
 						if (array_key_exists($sysmapid, $sysmaps)) {
 							foreach ($sysmaps[$sysmapid]['selements'] as $submap_selement) {
 								if ($submap_selement['permission'] >= PERM_READ) {
-									$problems_in_submap = $this->getElementProblems($submap_selement, $problems_per_trigger,
-										$sysmaps, $submaps_relations, $sysmaps[$sysmapid]['severity_min'],
-										$problems_counted, $triggers_per_hosts, $triggers_per_host_groups
+									$problems_in_submap = $this->getElementProblems($submap_selement,
+										$problems_per_trigger, $sysmaps, $submaps_relations,
+										$sysmaps[$sysmapid]['severity_min'], $problems_counted, $triggers_per_hosts,
+										$triggers_per_host_groups
 									);
 
-									if (is_array($problems_in_submap)) {
-										$problems = array_map(function () {
-											return array_sum(func_get_args());
-										}, $problems, $problems_in_submap);
+									if ($problems_in_submap !== null) {
+										$problems = self::sumArrayValues($problems, $problems_in_submap);
 									}
 								}
 							}
@@ -368,17 +375,15 @@ class CControllerWidgetNavTreeView extends CControllerWidget {
 						if (array_key_exists($sysmapid, $sysmaps)) {
 							foreach ($sysmaps[$sysmapid]['links'] as $link) {
 								if ($link['permission'] >= PERM_READ) {
-									foreach ($link['linktriggers'] as $lt) {
-										if (!array_key_exists($lt['triggerid'], $problems_counted)) {
-											$problems_counted[$lt['triggerid']] = true;
-											$add_problems = $problems_per_trigger[$lt['triggerid']];
-
-											// Sum problems.
-											$problems = array_map(function() {
-												return array_sum(func_get_args());
-											}, $add_problems, $problems);
-										}
+									$uncounted_problem_triggers = array_diff_key(
+										array_flip(zbx_objectValues($link['linktriggers'], 'triggerid')),
+										$problems_counted
+									);
+									foreach ($uncounted_problem_triggers as $triggerid => $var) {
+										$problems_counted[$triggerid] = true;
+										$problems = self::sumArrayValues($problems, $problems_per_trigger[$triggerid]);
 									}
+									unset($uncounted_problem_triggers);
 								}
 							}
 						}
@@ -388,7 +393,7 @@ class CControllerWidgetNavTreeView extends CControllerWidget {
 		}
 
 		// Remove problems which are less important than $severity_min.
-		if (is_array($problems) && $severity_min > 0) {
+		if ($problems !== null && $severity_min > 0) {
 			foreach ($problems as $sev => $probl) {
 				if ($severity_min > $sev) {
 					$problems[$sev] = 0;
@@ -404,72 +409,50 @@ class CControllerWidgetNavTreeView extends CControllerWidget {
 		$error = null;
 
 		// Get list of sysmapids.
+		$sysmapids = [];
 		$navtree_items = [];
-		foreach ($fields as $field_key => $field_value) {
-			if (is_numeric($field_value)) {
-				preg_match('/^map\.parent\.(\d+)$/', $field_key, $field_details);
-				if ($field_details) {
-					$fieldid = $field_details[1];
-					$navtree_items[$fieldid] = [
-						'parent' => $field_value,
-						'mapid' => array_key_exists('mapid.'.$fieldid, $fields) ? $fields['mapid.'.$fieldid] : 0,
-						'children_mapids' => []
-					];
-				}
+		foreach ($fields['navtree'] as $id => $navtree_item) {
+			$sysmapid = array_key_exists('sysmapid', $navtree_item) ? $navtree_item['sysmapid'] : 0;
+			if ($sysmapid != 0) {
+				$sysmapids[$sysmapid] = true;
 			}
-		}
 
-		// Find and fix circular dependencies.
-		foreach ($navtree_items as $fieldid => $field_details) {
-			if ($field_details['parent'] != 0) {
-				$parent = $navtree_items[$field_details['parent']];
-				while ($parent['parent'] != 0) {
-					if ($parent['parent'] == $fieldid) {
-						$navtree_items[$fieldid]['parent'] = 0;
-						break;
-					}
-					elseif (array_key_exists($parent['parent'], $navtree_items)) {
-						$parent = $navtree_items[$parent['parent']];
-					}
-					else {
-						break;
-					}
-				}
-			}
+			$navtree_items[$id] = [
+				'parent' => $navtree_item['parent'],
+				'sysmapid' => $sysmapid,
+				'child_sysmapids' => []
+			];
 		}
 
 		// Propagate item mapids to all its parent items.
-		foreach ($navtree_items as $field_details) {
-			$parentid = $field_details['parent'];
-			if ($field_details['parent'] != 0 && array_key_exists($parentid, $navtree_items)) {
-				while (array_key_exists($parentid, $navtree_items)) {
-					if ($field_details['mapid'] != 0) {
-						$navtree_items[$parentid]['children_mapids'][] = $field_details['mapid'];
-					}
-					$parentid = $navtree_items[$parentid]['parent'];
+		foreach ($navtree_items as $navtree_item) {
+			$parent = $navtree_item['parent'];
+
+			while (array_key_exists($parent, $navtree_items)) {
+				if ($navtree_item['sysmapid'] != 0) {
+					$navtree_items[$parent]['child_sysmapids'][$navtree_item['sysmapid']] = true;
 				}
+				$parent = $navtree_items[$parent]['parent'];
 			}
 		}
 
 		// Get severity levels and colors and select list of sysmapids to count problems per maps.
 		$this->problems_per_severity_tpl = [];
-		$config = select_config();
 		$severity_config = [];
 
-		$sysmapids = array_keys(array_flip(zbx_objectValues($navtree_items, 'mapid')));
-		$maps_accessible = API::Map()->get([
-			'output' => ['sysmapid'],
-			'sysmapids' => $sysmapids,
-			'preservekeys' => true
-		]);
+		$maps_accessible = $sysmapids
+			? API::Map()->get([
+				'output' => [],
+				'sysmapids' => array_keys($sysmapids),
+				'preservekeys' => true
+			])
+			: [];
 
-		$maps_accessible = array_keys($maps_accessible);
-
-		foreach (range(TRIGGER_SEVERITY_NOT_CLASSIFIED, TRIGGER_SEVERITY_COUNT - 1) as $severity) {
+		for ($severity = TRIGGER_SEVERITY_NOT_CLASSIFIED; $severity < TRIGGER_SEVERITY_COUNT; $severity++) {
 			$this->problems_per_severity_tpl[$severity] = 0;
 			$severity_config[$severity] = [
-				'color' => $config['severity_color_'.$severity],
-				'name' => $config['severity_name_'.$severity],
+				'name' => getSeverityName($severity),
+				'style_class' => getSeverityStatusStyle($severity)
 			];
 		}
 
@@ -489,11 +472,12 @@ class CControllerWidgetNavTreeView extends CControllerWidget {
 		$this->setResponse(new CControllerResponseData([
 			'name' => $this->getInput('name', $this->getDefaultHeader()),
 			'uniqueid' => $this->getInput('uniqueid'),
+			'navtree' => $fields['navtree'],
 			'navtree_item_selected' => $navtree_item_selected,
 			'navtree_items_opened' => $navtree_items_opened,
 			'problems' => $this->getNumberOfProblemsBySysmap($navtree_items),
-			'show_unavailable' => array_key_exists('show_unavailable', $fields) ? $fields['show_unavailable'] : 0,
-			'maps_accessible' => $maps_accessible,
+			'show_unavailable' => $fields['show_unavailable'],
+			'maps_accessible' => array_keys($maps_accessible),
 			'severity_config' => $severity_config,
 			'initial_load' => $this->getInput('initial_load', 0),
 			'error' => $error,
@@ -502,4 +486,26 @@ class CControllerWidgetNavTreeView extends CControllerWidget {
 			]
 		]));
 	}
+
+	/**
+	 * Function is used to sum problems in 2 arrays.
+	 *
+	 * Example:
+	 * $a1 = [1 => 0, 2 => 5, 3 => 10];
+	 * $a2 = [1 => 1, 2 => 2, 3 => 3];
+	 * self::sumArrayValues($a1, $a2); // returns [1 => 1, 2 => 7, 3 => 13]
+	 *
+	 * @param array $a1  Array containing severity as key and number of problems as value.
+	 * @param array $a2  Array containing severity as key and number of problems as value.
+	 *
+	 * @return array  Array containing problems in both arrays summed.
+	 */
+	protected static function sumArrayValues(array $a1, array $a2) {
+		foreach ($a1 as $key => &$value) {
+			$value += $a2[$key];
+		}
+		unset($value);
+
+		return $a1;
+	}
 }
