@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2019 Zabbix SIA
+** Copyright (C) 2001-2020 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -19,8 +19,6 @@
 **/
 
 
-require_once dirname(__FILE__).'/../../include/hostgroups.inc.php';
-
 class CControllerWidgetProblemsView extends CControllerWidget {
 
 	public function __construct() {
@@ -29,15 +27,14 @@ class CControllerWidgetProblemsView extends CControllerWidget {
 		$this->setType(WIDGET_PROBLEMS);
 		$this->setValidationRules([
 			'name' => 'string',
-			'fields' => 'json'
+			'fields' => 'json',
+			'initial_load' => 'in 0,1'
 		]);
 	}
 
 	protected function doAction() {
 		$fields = $this->getForm()->getFieldsData();
 
-		$config = select_config();
-
 		$data = CScreenProblem::getData([
 			'show' => $fields['show'],
 			'groupids' => getSubGroups($fields['groupids']),
@@ -49,16 +46,16 @@ class CControllerWidgetProblemsView extends CControllerWidget {
 			'tags' => $fields['tags'],
 			'show_suppressed' => $fields['show_suppressed'],
 			'unacknowledged' => $fields['unacknowledged'],
-			'show_latest_values' => $fields['show_latest_values']
-		], $config, true, true);
+			'show_opdata' => $fields['show_opdata']
+		], true, true);
 		list($sortfield, $sortorder) = self::getSorting($fields['sort_triggers']);
-		$data = CScreenProblem::sortData($data, $config, $sortfield, $sortorder);
+		$data = CScreenProblem::sortData($data, $sortfield, $sortorder);
 
 		if (count($data['problems']) > $fields['show_lines']) {
 			$info = _n('%1$d of %3$d%2$s problem is shown', '%1$d of %3$d%2$s problems are shown',
 				min($fields['show_lines'], count($data['problems'])),
-				(count($data['problems']) > $config['search_limit']) ? '+' : '',
-				min($config['search_limit'], count($data['problems']))
+				(count($data['problems']) > CSettingsHelper::get(CSettingsHelper::SEARCH_LIMIT)) ? '+' : '',
+				min(CSettingsHelper::get(CSettingsHelper::SEARCH_LIMIT), count($data['problems']))
 			);
 		}
 		else {
@@ -69,7 +66,7 @@ class CControllerWidgetProblemsView extends CControllerWidget {
 		$data = CScreenProblem::makeData($data, [
 			'show' => $fields['show'],
 			'details' => 0,
-			'show_latest_values' => $fields['show_latest_values']
+			'show_opdata' => $fields['show_opdata']
 		], true, true);
 
 		if ($fields['show_tags']) {
@@ -84,6 +81,7 @@ class CControllerWidgetProblemsView extends CControllerWidget {
 
 		$this->setResponse(new CControllerResponseData([
 			'name' => $this->getInput('name', $this->getDefaultHeader()),
+			'initial_load' => (bool) $this->getInput('initial_load', 0),
 			'fields' => [
 				'show' => $fields['show'],
 				'show_lines' => $fields['show_lines'],
@@ -92,18 +90,7 @@ class CControllerWidgetProblemsView extends CControllerWidget {
 				'tags' => $fields['tags'],
 				'tag_name_format' => $fields['tag_name_format'],
 				'tag_priority' => $fields['tag_priority'],
-				'show_latest_values' => $fields['show_latest_values']
-			],
-			'config' => [
-				'problem_unack_style' => $config['problem_unack_style'],
-				'problem_ack_style' => $config['problem_ack_style'],
-				'blink_period' => timeUnitToSeconds($config['blink_period']),
-				'severity_name_0' => $config['severity_name_0'],
-				'severity_name_1' => $config['severity_name_1'],
-				'severity_name_2' => $config['severity_name_2'],
-				'severity_name_3' => $config['severity_name_3'],
-				'severity_name_4' => $config['severity_name_4'],
-				'severity_name_5' => $config['severity_name_5']
+				'show_opdata' => $fields['show_opdata']
 			],
 			'data' => $data,
 			'info' => $info,
@@ -111,7 +98,17 @@ class CControllerWidgetProblemsView extends CControllerWidget {
 			'sortorder' => $sortorder,
 			'user' => [
 				'debug_mode' => $this->getDebugMode()
-			]
+			],
+			'config' => [
+				'problem_ack_style' => CSettingsHelper::get(CSettingsHelper::PROBLEM_ACK_STYLE),
+				'problem_unack_style' => CSettingsHelper::get(CSettingsHelper::PROBLEM_UNACK_STYLE),
+				'blink_period' => CSettingsHelper::get(CSettingsHelper::BLINK_PERIOD)
+			],
+			'allowed_ui_problems' => $this->checkAccess(CRoleHelper::UI_MONITORING_PROBLEMS),
+			'allowed_ack' => $this->checkAccess(CRoleHelper::ACTIONS_ACKNOWLEDGE_PROBLEMS)
+					|| $this->checkAccess(CRoleHelper::ACTIONS_CLOSE_PROBLEMS)
+					|| $this->checkAccess(CRoleHelper::ACTIONS_CHANGE_SEVERITY)
+					|| $this->checkAccess(CRoleHelper::ACTIONS_ADD_PROBLEM_COMMENTS)
 		]));
 	}
 
