@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2019 Zabbix SIA
+** Copyright (C) 2001-2020 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -19,16 +19,18 @@
 **/
 
 
+/**
+ * @var CView $this
+ */
+
 // indicator of sort field
 $sort_div = (new CSpan())->addClass(($data['sortorder'] === ZBX_SORT_DOWN) ? ZBX_STYLE_ARROW_DOWN : ZBX_STYLE_ARROW_UP);
 
-$backurl = (new CUrl('zabbix.php'))
-	->setArgument('action', 'dashboard.view')
-	->getUrl();
-
-$url_details = (new CUrl('tr_events.php'))
-	->setArgument('triggerid', '')
-	->setArgument('eventid', '');
+$url_details = $data['allowed_ui_problems']
+	? (new CUrl('tr_events.php'))
+		->setArgument('triggerid', '')
+		->setArgument('eventid', '')
+	: null;
 
 $show_timeline = ($data['sortfield'] === 'clock' && $data['fields']['show_timeline']);
 $show_recovery_data = in_array($data['fields']['show'], [TRIGGERS_OPTION_RECENT_PROBLEM, TRIGGERS_OPTION_ALL]);
@@ -46,6 +48,8 @@ else {
 	$header = [$header_time];
 }
 
+$show_opdata = $data['fields']['show_opdata'];
+
 $table = (new CTableInfo())
 	->setHeader(array_merge($header, [
 		$show_recovery_data ? _('Recovery time') : null,
@@ -57,7 +61,7 @@ $table = (new CTableInfo())
 			' &bullet; ',
 			($data['sortfield'] === 'severity') ? [_('Severity'), $sort_div] : _('Severity')
 		],
-		$data['fields']['show_latest_values'] ? _('Latest values') : null,
+		($show_opdata == OPERATIONAL_DATA_SHOW_SEPARATELY) ? _('Operational data') : null,
 		_('Duration'),
 		_('Ack'),
 		_('Actions'),
@@ -94,23 +98,28 @@ foreach ($data['data']['problems'] as $eventid => $problem) {
 		$value_clock = $in_closing ? time() : $problem['clock'];
 	}
 
-	$url_details
-		->setArgument('triggerid', $problem['objectid'])
-		->setArgument('eventid', $problem['eventid']);
-
 	$cell_clock = ($problem['clock'] >= $today)
 		? zbx_date2str(TIME_FORMAT_SECONDS, $problem['clock'])
 		: zbx_date2str(DATE_TIME_FORMAT_SECONDS, $problem['clock']);
-	$cell_clock = new CCol(new CLink($cell_clock, $url_details));
 
-	$is_acknowledged = $problem['acknowledged'] == EVENT_ACKNOWLEDGED;
+	if ($url_details !== null) {
+		$url_details
+			->setArgument('triggerid', $problem['objectid'])
+			->setArgument('eventid', $problem['eventid']);
+		$cell_clock = new CCol(new CLink($cell_clock, $url_details));
+	}
+	else {
+		$cell_clock = new CCol($cell_clock);
+	}
+
+	$is_acknowledged = ($problem['acknowledged'] == EVENT_ACKNOWLEDGED);
 
 	if ($show_recovery_data) {
 		if ($problem['r_eventid'] != 0) {
 			$cell_r_clock = ($problem['r_clock'] >= $today)
 				? zbx_date2str(TIME_FORMAT_SECONDS, $problem['r_clock'])
 				: zbx_date2str(DATE_TIME_FORMAT_SECONDS, $problem['r_clock']);
-			$cell_r_clock = (new CCol(new CLink($cell_r_clock, $url_details)))
+			$cell_r_clock = (new CCol($url_details !== null ? new CLink($cell_r_clock, $url_details) : $cell_r_clock))
 				->addClass(ZBX_STYLE_NOWRAP)
 				->addClass(ZBX_STYLE_RIGHT);
 		}
@@ -140,7 +149,7 @@ foreach ($data['data']['problems'] as $eventid => $problem) {
 			$info_icons[] = makeInformationIcon(
 				array_key_exists($problem['userid'], $data['data']['users'])
 					? _s('Resolved by user "%1$s".', getUserFullname($data['data']['users'][$problem['userid']]))
-					: _('Resolved by user.')
+					: _('Resolved by inaccessible user.')
 			);
 		}
 	}
@@ -149,19 +158,63 @@ foreach ($data['data']['problems'] as $eventid => $problem) {
 		$info_icons[] = makeSuppressedProblemIcon($problem['suppression_data']);
 	}
 
-	$description = (new CCol([
+	$opdata = null;
+	if ($show_opdata != OPERATIONAL_DATA_SHOW_NONE) {
+
+		// operational data
+		if ($trigger['opdata'] === '') {
+			if ($show_opdata == OPERATIONAL_DATA_SHOW_SEPARATELY) {
+				$opdata = (new CCol(CScreenProblem::getLatestValues($trigger['items'])))->addClass('latest-values');
+			}
+		} else {
+			$opdata = CMacrosResolverHelper::resolveTriggerOpdata(
+				[
+					'triggerid' => $trigger['triggerid'],
+					'expression' => $trigger['expression'],
+					'opdata' => $trigger['opdata'],
+					'clock' => ($problem['r_eventid'] != 0) ? $problem['r_clock'] : $problem['clock'],
+					'ns' => ($problem['r_eventid'] != 0) ? $problem['r_ns'] : $problem['ns']
+				],
+				[
+					'events' => true,
+					'html' => true
+				]
+			);
+
+			if ($show_opdata == OPERATIONAL_DATA_SHOW_SEPARATELY) {
+				$opdata = (new CCol($opdata))
+					->addClass('opdata')
+					->addClass(ZBX_STYLE_WORDWRAP);
+			}
+		}
+	}
+
+	$allowed = [
+		'ui_problems' => CWebUser::checkAccess(CRoleHelper::UI_MONITORING_PROBLEMS),
+		'ack' => CWebUser::checkAccess(CRoleHelper::ACTIONS_ACKNOWLEDGE_PROBLEMS)
+				|| CWebUser::checkAccess(CRoleHelper::ACTIONS_CLOSE_PROBLEMS)
+				|| CWebUser::checkAccess(CRoleHelper::ACTIONS_CHANGE_SEVERITY)
+				|| CWebUser::checkAccess(CRoleHelper::ACTIONS_ADD_PROBLEM_COMMENTS)
+	];
+
+	$problem_link = [
 		(new CLinkAction($problem['name']))
 			->setHint(
-				make_popup_eventlist($trigger, $eventid, $backurl, $show_timeline,
-					$data['fields']['show_tags'], $data['fields']['tags'], $data['fields']['tag_name_format'],
-					$data['fields']['tag_priority']),
-				'',
-				true
+				make_popup_eventlist(['comments' => $problem['comments'], 'url' => $problem['url'],
+					'triggerid' => $trigger['triggerid']], $eventid, $allowed, $show_timeline, $data['fields']['show_tags'],
+					$data['fields']['tags'], $data['fields']['tag_name_format'], $data['fields']['tag_priority']
+				)
 			)
 			->setAttribute('aria-label', _xs('%1$s, Severity, %2$s', 'screen reader',
-				$problem['name'], getSeverityName($problem['severity'], $data['config'])
+				$problem['name'], getSeverityName($problem['severity'])
 			))
-	]));
+	];
+
+	if ($show_opdata == OPERATIONAL_DATA_SHOW_WITH_PROBLEM && $opdata) {
+		$problem_link = array_merge($problem_link, [' (', $opdata, ')']);
+	}
+
+	$description = (new CCol($problem_link));
 
 	$description_style = getSeverityStyle($problem['severity']);
 
@@ -169,14 +222,17 @@ foreach ($data['data']['problems'] as $eventid => $problem) {
 		$description->addClass($description_style);
 	}
 
-	if (!$show_recovery_data && $data['config'][$is_acknowledged ? 'problem_ack_style' : 'problem_unack_style']) {
+	if (!$show_recovery_data
+			&& (($is_acknowledged && $data['config']['problem_ack_style'])
+				|| (!$is_acknowledged && $data['config']['problem_unack_style']))) {
 		// blinking
 		$duration = time() - $problem['clock'];
+		$blink_period = timeUnitToSeconds($data['config']['blink_period']);
 
-		if ($data['config']['blink_period'] != 0 && $duration < $data['config']['blink_period']) {
+		if ($blink_period != 0 && $duration < $blink_period) {
 			$description
 				->addClass('blink')
-				->setAttribute('data-time-to-blink', $data['config']['blink_period'] - $duration)
+				->setAttribute('data-time-to-blink', $blink_period - $duration)
 				->setAttribute('data-toggle-class', ZBX_STYLE_BLINK_HIDDEN);
 		}
 	}
@@ -203,12 +259,15 @@ foreach ($data['data']['problems'] as $eventid => $problem) {
 		];
 	}
 
-	// Create acknowledge url.
-	$problem_update_url = (new CUrl('zabbix.php'))
-		->setArgument('action', 'acknowledge.edit')
-		->setArgument('eventids', [$problem['eventid']])
-		->setArgument('backurl', $backurl)
-		->getUrl();
+	// Create acknowledge link.
+	$problem_update_link = $data['allowed_ack']
+		? (new CLink($is_acknowledged ? _('Yes') : _('No')))
+			->addClass($is_acknowledged ? ZBX_STYLE_GREEN : ZBX_STYLE_RED)
+			->addClass(ZBX_STYLE_LINK_ALT)
+			->onClick('acknowledgePopUp('.json_encode(['eventids' => [$problem['eventid']]]).', this);')
+		: (new CSpan($is_acknowledged ? _('Yes') : _('No')))->addClass(
+			$is_acknowledged ? ZBX_STYLE_GREEN : ZBX_STYLE_RED
+		);
 
 	$table->addRow(array_merge($row, [
 		$show_recovery_data ? $cell_r_clock : null,
@@ -216,14 +275,12 @@ foreach ($data['data']['problems'] as $eventid => $problem) {
 		makeInformationList($info_icons),
 		$triggers_hosts[$trigger['triggerid']],
 		$description,
-		$data['fields']['show_latest_values'] ? CScreenProblem::getLatestValues($trigger['items']) : null,
+		($show_opdata == OPERATIONAL_DATA_SHOW_SEPARATELY ) ? $opdata : null,
 		(new CCol(zbx_date2age($problem['clock'], ($problem['r_eventid'] != 0) ? $problem['r_clock'] : 0)))
 			->addClass(ZBX_STYLE_NOWRAP),
-		(new CLink($problem['acknowledged'] == EVENT_ACKNOWLEDGED ? _('Yes') : _('No'), $problem_update_url))
-			->addClass($problem['acknowledged'] == EVENT_ACKNOWLEDGED ? ZBX_STYLE_GREEN : ZBX_STYLE_RED)
-			->addClass(ZBX_STYLE_LINK_ALT),
+		$problem_update_link,
 		makeEventActionsIcons($problem['eventid'], $data['data']['actions'], $data['data']['mediatypes'],
-			$data['data']['users'], $data['config']
+			$data['data']['users']
 		),
 		$data['fields']['show_tags'] ? $data['data']['tags'][$problem['eventid']] : null
 	]));
@@ -243,6 +300,17 @@ $output = [
 	'body' => $table->toString()
 ];
 
+if ($data['initial_load']) {
+	$output['script_inline'] =
+		'if (typeof refreshProblemsWidget !== typeof(Function)) {'.
+			'function refreshProblemsWidget(event, response, overlay) {'.
+				'refreshWidgetOnAcknowledgeCreate("problems", response, overlay);'.
+			'}'.
+
+			'$.subscribe("acknowledge.create", refreshProblemsWidget);'.
+		'}';
+}
+
 if (($messages = getMessages()) !== null) {
 	$output['messages'] = $messages->toString();
 }
@@ -252,4 +320,4 @@ if ($data['user']['debug_mode'] == GROUP_DEBUG_MODE_ENABLED) {
 	$output['debug'] = CProfiler::getInstance()->make()->toString();
 }
 
-echo (new CJson())->encode($output);
+echo json_encode($output);
