@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2019 Zabbix SIA
+** Copyright (C) 2001-2020 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -26,30 +26,37 @@ require_once dirname(__FILE__).'/include/forms.inc.php';
 
 $page['title'] = _('Configuration of host prototypes');
 $page['file'] = 'host_prototypes.php';
-$page['scripts'] = ['effects.js', 'class.cviewswitcher.js', 'multiselect.js'];
+$page['scripts'] = ['effects.js', 'class.cviewswitcher.js', 'multiselect.js', 'textareaflexible.js',
+	'class.cverticalaccordion.js', 'inputsecret.js', 'macrovalue.js', 'class.tab-indicators.js'
+];
 
 require_once dirname(__FILE__).'/include/page_header.php';
 
 // VAR	TYPE	OPTIONAL	FLAGS	VALIDATION	EXCEPTION
 $fields = [
-	'hostid' =>					[T_ZBX_INT, O_NO,	P_SYS,	DB_ID,		'(isset({form}) && ({form} == "update"))'],
+	'hostid' =>					[T_ZBX_INT, O_NO,	P_SYS,	DB_ID, '(isset({form}) && ({form} == "update")) || (isset({action}) && {action} == "hostprototype.updatediscover")'],
 	'parent_discoveryid' =>		[T_ZBX_INT, O_MAND, P_SYS,	DB_ID, null],
-	'host' =>		        	[T_ZBX_STR, O_OPT, null,		NOT_EMPTY,	'isset({add}) || isset({update})', _('Host name')],
-	'name' =>	            	[T_ZBX_STR, O_OPT, null,		null,		'isset({add}) || isset({update})'],
-	'status' =>		        	[T_ZBX_INT, O_OPT, null,		IN([HOST_STATUS_NOT_MONITORED, HOST_STATUS_MONITORED]), null],
+	'host' =>					[T_ZBX_STR, O_OPT, null,		NOT_EMPTY,	'isset({add}) || isset({update})', _('Host name')],
+	'name' =>					[T_ZBX_STR, O_OPT, null,		null,		'isset({add}) || isset({update})'],
+	'status' =>					[T_ZBX_INT, O_OPT, null,		IN([HOST_STATUS_NOT_MONITORED, HOST_STATUS_MONITORED]), null],
+	'discover' =>				[T_ZBX_INT, O_OPT, null, IN([ZBX_PROTOTYPE_DISCOVER, ZBX_PROTOTYPE_NO_DISCOVER]), null],
 	'inventory_mode' =>			[T_ZBX_INT, O_OPT, null, IN([HOST_INVENTORY_DISABLED, HOST_INVENTORY_MANUAL, HOST_INVENTORY_AUTOMATIC]), null],
-	'templates' =>		    	[T_ZBX_STR, O_OPT, null, NOT_EMPTY,	null],
-	'add_template' =>			[T_ZBX_STR, O_OPT, null,		null,	null],
-	'add_templates' =>		    [T_ZBX_STR, O_OPT, null, NOT_EMPTY,	null],
+	'templates' =>				[T_ZBX_STR, O_OPT, null, NOT_EMPTY,	null],
+	'add_templates' =>			[T_ZBX_STR, O_OPT, null, NOT_EMPTY,	null],
 	'group_links' =>			[T_ZBX_STR, O_OPT, null, NOT_EMPTY,	null],
 	'group_prototypes' =>		[T_ZBX_STR, O_OPT, null, NOT_EMPTY,	null],
 	'unlink' =>					[T_ZBX_STR, O_OPT, P_SYS|P_ACT,	null,		null],
 	'group_hostid' =>			[T_ZBX_INT, O_OPT, null,	DB_ID,		null],
 	'show_inherited_macros' =>	[T_ZBX_INT, O_OPT, null, IN([0,1]), null],
+	'tags' =>					[T_ZBX_STR, O_OPT, P_SYS,			null,		null],
+	'macros' =>					[T_ZBX_STR, O_OPT, P_SYS,			null,		null],
+	'custom_interfaces' =>		[T_ZBX_INT, O_OPT, null, IN([HOST_PROT_INTERFACES_INHERIT, HOST_PROT_INTERFACES_CUSTOM]), null],
+	'interfaces' =>				[T_ZBX_STR, O_OPT, null, null,		null],
+	'mainInterfaces' =>			[T_ZBX_INT, O_OPT, null, DB_ID,		null],
 	// actions
 	'action' =>					[T_ZBX_STR, O_OPT, P_SYS|P_ACT,
 									IN('"hostprototype.massdelete","hostprototype.massdisable",'.
-										'"hostprototype.massenable"'
+										'"hostprototype.massenable","hostprototype.updatediscover"'
 									),
 									null
 								],
@@ -61,11 +68,13 @@ $fields = [
 	'form' =>					[T_ZBX_STR, O_OPT, P_SYS,	null,		null],
 	'form_refresh' =>			[T_ZBX_INT, O_OPT, null,	null,		null],
 	// sort and sortorder
-	'sort' =>					[T_ZBX_STR, O_OPT, P_SYS, IN('"name","status"'),						null],
+	'sort' =>					[T_ZBX_STR, O_OPT, P_SYS, IN('"name","status","discover"'),						null],
 	'sortorder' =>				[T_ZBX_STR, O_OPT, P_SYS, IN('"'.ZBX_SORT_DOWN.'","'.ZBX_SORT_UP.'"'),	null]
 ];
 check_fields($fields);
 
+$hostid = getRequest('hostid', 0);
+
 // permissions
 if (getRequest('parent_discoveryid')) {
 	$discoveryRule = API::DiscoveryRule()->get([
@@ -79,15 +88,17 @@ if (getRequest('parent_discoveryid')) {
 		access_deny();
 	}
 
-	if (getRequest('hostid')) {
+	if ($hostid != 0) {
 		$hostPrototype = API::HostPrototype()->get([
-			'hostids' => getRequest('hostid'),
+			'hostids' => [$hostid],
 			'output' => API_OUTPUT_EXTEND,
 			'selectGroupLinks' => API_OUTPUT_EXTEND,
 			'selectGroupPrototypes' => API_OUTPUT_EXTEND,
 			'selectTemplates' => ['templateid', 'name'],
 			'selectParentHost' => ['hostid'],
-			'selectInventory' => ['inventory_mode'],
+			'selectMacros' => ['hostmacroid', 'macro', 'value', 'type', 'description'],
+			'selectTags' => ['tag', 'value'],
+			'selectInterfaces' => ['type', 'main', 'useip', 'ip', 'dns', 'port', 'details'],
 			'editable' => true
 		]);
 		$hostPrototype = reset($hostPrototype);
@@ -100,24 +111,35 @@ else {
 	access_deny();
 }
 
+$tags = getRequest('tags', []);
+foreach ($tags as $key => $tag) {
+	// remove empty new tag lines
+	if ($tag['tag'] === '' && $tag['value'] === '') {
+		unset($tags[$key]);
+	}
+}
+
+// Remove inherited macros data (actions: 'add', 'update' and 'form').
+$macros = cleanInheritedMacros(getRequest('macros', []));
+
+// Remove empty new macro lines.
+$macros = array_filter($macros, function($macro) {
+	$keys = array_flip(['hostmacroid', 'macro', 'value', 'description']);
+
+	return (bool) array_filter(array_intersect_key($macro, $keys));
+});
+
 /*
  * Actions
  */
-// add templates to the list
-if (getRequest('add_template')) {
-	foreach (getRequest('add_templates', []) as $templateId) {
-		$_REQUEST['templates'][$templateId] = $templateId;
-	}
-}
-// unlink templates
-elseif (getRequest('unlink')) {
+if (getRequest('unlink')) {
 	foreach (getRequest('unlink') as $templateId => $value) {
 		unset($_REQUEST['templates'][$templateId]);
 	}
 }
 elseif (isset($_REQUEST['delete']) && isset($_REQUEST['hostid'])) {
 	DBstart();
-	$result = API::HostPrototype()->delete([getRequest('hostid')]);
+	$result = API::HostPrototype()->delete([$hostid]);
 	$result = DBend($result);
 
 	if ($result) {
@@ -135,6 +157,24 @@ elseif (isset($_REQUEST['clone']) && isset($_REQUEST['hostid'])) {
 		}
 		unset($groupPrototype);
 	}
+
+	if ($macros && in_array(ZBX_MACRO_TYPE_SECRET, array_column($macros, 'type'))) {
+		// Reset macro type and value.
+		$macros = array_map(function($value) {
+			return ($value['type'] == ZBX_MACRO_TYPE_SECRET)
+				? ['value' => '', 'type' => ZBX_MACRO_TYPE_TEXT] + $value
+				: $value;
+		}, $macros);
+
+		$msg = [
+			'type' => 'error',
+			'message' => _('The cloned host prototype contains user defined macros with type "Secret text". The value and type of these macros were reset.'),
+			'source' => ''
+		];
+
+		echo makeMessageBox(false, [$msg], null, true, false)->addClass(ZBX_STYLE_MSG_WARNING);
+	}
+
 	$_REQUEST['form'] = 'clone';
 }
 elseif (hasRequest('add') || hasRequest('update')) {
@@ -144,14 +184,17 @@ elseif (hasRequest('add') || hasRequest('update')) {
 		'host' => getRequest('host', ''),
 		'name' => (getRequest('name', '') === '') ? getRequest('host', '') : getRequest('name', ''),
 		'status' => getRequest('status', HOST_STATUS_NOT_MONITORED),
+		'discover' => getRequest('discover', DB::getDefault('hosts', 'discover')),
 		'groupLinks' => [],
 		'groupPrototypes' => [],
-		'templates' => getRequest('templates', [])
+		'tags' => $tags,
+		'macros' => $macros,
+		'templates' => array_merge(getRequest('templates', []), getRequest('add_templates', [])),
+		'custom_interfaces' => getRequest('custom_interfaces', DB::getDefault('hosts', 'custom_interfaces'))
 	];
 
-	// If 'inventory' is present, API requires 'inventory_mode' to have a value, but templated prototypes don't have it.
 	if (hasRequest('inventory_mode')) {
-		$newHostPrototype['inventory']['inventory_mode'] = getRequest('inventory_mode');
+		$newHostPrototype['inventory_mode'] = getRequest('inventory_mode');
 	}
 
 	// API requires 'templateid' property.
@@ -170,8 +213,41 @@ elseif (hasRequest('add') || hasRequest('update')) {
 		}
 	}
 
-	if (getRequest('hostid')) {
-		$newHostPrototype['hostid'] = getRequest('hostid');
+	if ($newHostPrototype['custom_interfaces'] == HOST_PROT_INTERFACES_CUSTOM) {
+		$interfaces = getRequest('interfaces', []);
+
+		foreach ($interfaces as $key => $interface) {
+			// Proccess SNMP interface fields.
+			if ($interface['type'] == INTERFACE_TYPE_SNMP) {
+				if (!array_key_exists('details', $interface)) {
+					$interface['details'] = [];
+				}
+
+				$interfaces[$key]['details']['bulk'] = array_key_exists('bulk', $interface['details'])
+					? SNMP_BULK_ENABLED
+					: SNMP_BULK_DISABLED;
+			}
+
+			unset($interfaces[$key]['isNew'], $interfaces[$key]['items'], $interfaces[$key]['interfaceid']);
+			$interfaces[$key]['main'] = 0;
+		}
+
+		$main_interfaces = getRequest('mainInterfaces', []);
+
+		foreach ([INTERFACE_TYPE_AGENT, INTERFACE_TYPE_SNMP, INTERFACE_TYPE_JMX, INTERFACE_TYPE_IPMI] as $type) {
+			if (array_key_exists($type, $main_interfaces) && array_key_exists($main_interfaces[$type], $interfaces)) {
+				$interfaces[$main_interfaces[$type]]['main'] = INTERFACE_PRIMARY;
+			}
+		}
+	}
+	else {
+		$interfaces = [];
+	}
+
+	$newHostPrototype['interfaces'] = $interfaces;
+
+	if ($hostid != 0) {
+		$newHostPrototype['hostid'] = $hostid;
 
 		if (!$hostPrototype['templateid']) {
 			// add group prototypes based on existing host groups
@@ -210,6 +286,17 @@ elseif (hasRequest('add') || hasRequest('update')) {
 			];
 		}
 
+		/*
+		 * Sanitize macros array. When we clone, we have old hostmacroid.
+		 * We need delete them before we push array to API.
+		*/
+		foreach ($newHostPrototype['macros'] as &$macro) {
+			if (array_key_exists('hostmacroid', $macro)) {
+				unset($macro['hostmacroid']);
+			}
+		}
+		unset($macro);
+
 		$result = API::HostPrototype()->create($newHostPrototype);
 
 		show_messages($result, _('Host prototype added'), _('Cannot add host prototype'));
@@ -222,6 +309,14 @@ elseif (hasRequest('add') || hasRequest('update')) {
 		unset($_REQUEST['itemid'], $_REQUEST['form']);
 	}
 }
+elseif ($hostid != 0 && getRequest('action', '') === 'hostprototype.updatediscover') {
+	$result = API::HostPrototype()->update([
+		'hostid' => $hostid,
+		'discover' => getRequest('discover', DB::getDefault('hosts', 'discover'))
+	]);
+
+	show_messages($result, _('Host prototype updated'), _('Cannot update host prototype'));
+}
 // GO
 elseif (hasRequest('action') && str_in_array(getRequest('action'), ['hostprototype.massenable', 'hostprototype.massdisable']) && hasRequest('group_hostid')) {
 	$status = (getRequest('action') == 'hostprototype.massenable') ? HOST_STATUS_MONITORED : HOST_STATUS_NOT_MONITORED;
@@ -268,48 +363,75 @@ if (hasRequest('action') && hasRequest('group_hostid') && !$result) {
 	uncheckTableRows($discoveryRule['itemid'], zbx_objectValues($host_prototypes, 'hostid'));
 }
 
-$config = select_config();
-
 /*
  * Display
  */
 if (hasRequest('form')) {
+	// During clone "hostid" could've been reset.
+	$hostid = getRequest('hostid', 0);
+
 	$data = [
 		'discovery_rule' => $discoveryRule,
 		'host_prototype' => [
-			'hostid' => getRequest('hostid', 0),
-			'templateid' => getRequest('hostid') ? $hostPrototype['templateid'] : 0,
+			'hostid' => $hostid,
+			'templateid' => ($hostid == 0) ? 0 : $hostPrototype['templateid'],
 			'host' => getRequest('host'),
 			'name' => getRequest('name'),
 			'status' => getRequest('status', HOST_STATUS_NOT_MONITORED),
+			'discover' => getRequest('discover', DB::getDefault('hosts', 'discover')),
 			'templates' => [],
-			'inventory' => [
-				'inventory_mode' => getRequest('inventory_mode', $config['default_inventory_mode'])
-			],
-			'groupPrototypes' => getRequest('group_prototypes', [])
+			'add_templates' => [],
+			'inventory_mode' => getRequest('inventory_mode',
+				CSettingsHelper::get(CSettingsHelper::DEFAULT_INVENTORY_MODE)
+			),
+			'groupPrototypes' => getRequest('group_prototypes', []),
+			'macros' => $macros,
+			'custom_interfaces' => getRequest('custom_interfaces', DB::getDefault('hosts', 'custom_interfaces')),
+			'interfaces' => getRequest('interfaces', []),
+			'main_interfaces' => getRequest('mainInterfaces', []),
 		],
-		'groups' => [],
 		'show_inherited_macros' => getRequest('show_inherited_macros', 0),
+		'readonly' => ($hostid != 0 && $hostPrototype['templateid']),
+		'groups' => [],
+		'tags' => $tags,
+		// Parent discovery rules.
 		'templates' => []
 	];
 
-	// add already linked and new templates
-	$data['host_prototype']['templates'] = API::Template()->get([
-		'output' => ['templateid', 'name'],
-		'templateids' => getRequest('templates', [])
-	]);
+	// Add already linked and new templates.
+	$templates = [];
+	$request_templates = getRequest('templates', []);
+	$request_add_templates = getRequest('add_templates', []);
+
+	if ($request_templates || $request_add_templates) {
+		$templates = API::Template()->get([
+			'output' => ['templateid', 'name'],
+			'templateids' => array_merge($request_templates, $request_add_templates),
+			'preservekeys' => true
+		]);
+
+		$data['host_prototype']['templates'] = array_intersect_key($templates, array_flip($request_templates));
+		CArrayHelper::sort($data['host_prototype']['templates'], ['name']);
+
+		$data['host_prototype']['add_templates'] = array_intersect_key($templates, array_flip($request_add_templates));
+
+		foreach ($data['host_prototype']['add_templates'] as &$template) {
+			$template = CArrayHelper::renameKeys($template, ['templateid' => 'id']);
+		}
+		unset($template);
+	}
 
 	// add parent host
 	$parentHost = API::Host()->get([
 		'output' => API_OUTPUT_EXTEND,
 		'selectGroups' => ['groupid', 'name'],
 		'selectInterfaces' => API_OUTPUT_EXTEND,
-		'selectMacros' => ['macro', 'value'],
 		'hostids' => $discoveryRule['hostid'],
 		'templated_hosts' => true
 	]);
 	$parentHost = reset($parentHost);
 	$data['parent_host'] = $parentHost;
+	$data['parent_hostid'] = $parentHost['hostid'];
 
 	if (getRequest('group_links')) {
 		$data['groups'] = API::HostGroup()->get([
@@ -334,25 +456,45 @@ if (hasRequest('form')) {
 			// When opening existing host prototype, display all values from database.
 			$data['host_prototype'] = array_merge($data['host_prototype'], $hostPrototype);
 
-			if (!array_key_exists('inventory_mode', $data['host_prototype']['inventory'])) {
-				$data['host_prototype']['inventory']['inventory_mode'] = HOST_INVENTORY_DISABLED;
-			}
-
+			$groupids = zbx_objectValues($data['host_prototype']['groupLinks'], 'groupid');
 			$data['groups'] = API::HostGroup()->get([
 				'output' => ['groupid', 'name'],
-				'groupids' => zbx_objectValues($data['host_prototype']['groupLinks'], 'groupid'),
-				'editable' => true,
+				'groupids' => $groupids,
 				'preservekeys' => true
 			]);
+
+			$n = 0;
+			foreach ($groupids as $groupid) {
+				if (!array_key_exists($groupid, $data['groups'])) {
+					$postfix = (++$n > 1) ? ' ('.$n.')' : '';
+					$data['groups'][$groupid] = [
+						'groupid' => $groupid,
+						'name' => _('Inaccessible group').$postfix,
+						'inaccessible' => true
+					];
+				}
+			}
+
+			$data['tags'] = $data['host_prototype']['tags'];
 		}
 		else {
 			// Set default values for new host prototype.
 			$data['host_prototype']['status'] = HOST_STATUS_MONITORED;
 		}
 	}
+	else {
+		foreach ([INTERFACE_TYPE_AGENT, INTERFACE_TYPE_SNMP, INTERFACE_TYPE_JMX, INTERFACE_TYPE_IPMI] as $type) {
+			if (array_key_exists($type, $data['host_prototype']['main_interfaces'])) {
+				$interfaceid = $data['host_prototype']['main_interfaces'][$type];
+				$data['host_prototype']['interfaces'][$interfaceid]['main'] = '1';
+			}
+		}
+		$data['host_prototype']['interfaces'] = array_values($data['host_prototype']['interfaces']);
+	}
 
+	$data['allowed_ui_conf_templates'] = CWebUser::checkAccess(CRoleHelper::UI_CONFIGURATION_TEMPLATES);
 	$data['templates'] = makeHostPrototypeTemplatesHtml($data['host_prototype']['hostid'],
-		getHostPrototypeParentTemplates([$data['host_prototype']])
+		getHostPrototypeParentTemplates([$data['host_prototype']]), $data['allowed_ui_conf_templates']
 	);
 
 	// Select writable templates
@@ -368,13 +510,41 @@ if (hasRequest('form')) {
 		]);
 	}
 
-	// order linked templates
-	CArrayHelper::sort($data['host_prototype']['templates'], ['name']);
+	// tags
+	if (!$data['tags']) {
+		$data['tags'][] = ['tag' => '', 'value' => ''];
+	}
+	else {
+		CArrayHelper::sort($data['tags'], ['tag', 'value']);
+	}
 
-	// render view
-	$itemView = new CView('configuration.host.prototype.edit', $data);
-	$itemView->render();
-	$itemView->show();
+	$macros = $data['host_prototype']['macros'];
+
+	if ($data['show_inherited_macros']) {
+		$macros = mergeInheritedMacros($macros, getInheritedMacros(array_keys($templates), $data['parent_hostid']));
+	}
+
+	// Sort only after inherited macros are added. Otherwise the list will look chaotic.
+	$data['macros'] = array_values(order_macros($macros, 'macro'));
+
+	if (!$data['macros'] && !$data['readonly']) {
+		$macro = ['macro' => '', 'value' => '', 'description' => '', 'type' => ZBX_MACRO_TYPE_TEXT];
+
+		if ($data['show_inherited_macros']) {
+			$macro['inherited_type'] = ZBX_PROPERTY_OWN;
+		}
+
+		$data['macros'][] = $macro;
+	}
+
+	// This data is used in common.template.edit.js.php.
+	$data['macros_tab'] = [
+		'linked_templates' => array_map('strval', $templateids),
+		'add_templates' => array_map('strval', array_keys($data['host_prototype']['add_templates']))
+	];
+
+	// Render view.
+	echo (new CView('configuration.host.prototype.edit', $data))->getOutput();
 }
 else {
 	$sortField = getRequest('sort', CProfile::get('web.'.$page['file'].'.sort', 'name'));
@@ -392,21 +562,35 @@ else {
 	];
 
 	// get items
+	$limit = CSettingsHelper::get(CSettingsHelper::SEARCH_LIMIT) + 1;
 	$data['hostPrototypes'] = API::HostPrototype()->get([
 		'discoveryids' => $data['parent_discoveryid'],
 		'output' => API_OUTPUT_EXTEND,
 		'selectTemplates' => ['templateid', 'name'],
+		'selectTags' => ['tag', 'value'],
 		'editable' => true,
 		'sortfield' => $sortField,
-		'limit' => $config['search_limit'] + 1
+		'limit' => $limit
 	]);
 
 	order_result($data['hostPrototypes'], $sortField, $sortOrder);
 
-	$url = (new CUrl('host_prototypes.php'))
-		->setArgument('parent_discoveryid', $data['parent_discoveryid']);
+	// pager
+	if (hasRequest('page')) {
+		$page_num = getRequest('page');
+	}
+	elseif (isRequestMethod('get') && !hasRequest('cancel')) {
+		$page_num = 1;
+	}
+	else {
+		$page_num = CPagerHelper::loadPage($page['file']);
+	}
 
-	$data['paging'] = getPagingLine($data['hostPrototypes'], $sortOrder, $url);
+	CPagerHelper::savePage($page['file'], $page_num);
+
+	$data['paging'] = CPagerHelper::paginate($page_num, $data['hostPrototypes'], $sortOrder,
+		(new CUrl('host_prototypes.php'))->setArgument('parent_discoveryid', $data['parent_discoveryid'])
+	);
 
 	$data['parent_templates'] = getHostPrototypeParentTemplates($data['hostPrototypes']);
 
@@ -442,10 +626,11 @@ else {
 		]);
 	}
 
+	$data['tags'] = makeTags($data['hostPrototypes'], true, 'hostid');
+	$data['allowed_ui_conf_templates'] = CWebUser::checkAccess(CRoleHelper::UI_CONFIGURATION_TEMPLATES);
+
 	// render view
-	$itemView = new CView('configuration.host.prototype.list', $data);
-	$itemView->render();
-	$itemView->show();
+	echo (new CView('configuration.host.prototype.list', $data))->getOutput();
 }
 
 require_once dirname(__FILE__).'/include/page_footer.php';
