@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2019 Zabbix SIA
+** Copyright (C) 2001-2020 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -88,18 +88,16 @@ function discovery_object2str($object = null) {
  *
  * For action condition types such as: hosts, host groups, templates, proxies, triggers, discovery rules
  * and discovery checks, action condition values contain IDs. All unique IDs are first collected and then queried.
- * For other action condition types values are returned as they are or converted using simple string convertion
+ * For other action condition types values are returned as they are or converted using simple string conversion
  * functions according to action condition type.
  *
  * @param array $actions							array of actions
  * @param array $action['filter']					array containing arrays of action conditions and other data
  * @param array $action['filter']['conditions']		array of action conditions
- * @param array $config								array containing configuration parameters for getting trigger
- *													severity names
  *
  * @return array									returns an array of actions condition string values
  */
-function actionConditionValueToString(array $actions, array $config) {
+function actionConditionValueToString(array $actions) {
 	$result = [];
 
 	$groupIds = [];
@@ -158,7 +156,7 @@ function actionConditionValueToString(array $actions, array $config) {
 					break;
 
 				case CONDITION_TYPE_TRIGGER_SEVERITY:
-					$result[$i][$j] = getSeverityName($condition['value'], $config);
+					$result[$i][$j] = getSeverityName($condition['value']);
 					break;
 
 				case CONDITION_TYPE_DRULE:
@@ -340,6 +338,9 @@ function getConditionDescription($condition_type, $operator, $value, $value2) {
 			? [_('Problem is suppressed')]
 			: [_('Problem is not suppressed')];
 	}
+	elseif ($condition_type == CONDITION_TYPE_EVENT_ACKNOWLEDGED) {
+		return $value ? _('Event is acknowledged') : _('Event is not acknowledged');
+	}
 	else {
 		$description = [condition_type2str($condition_type)];
 		$description[] = ' ';
@@ -417,15 +418,15 @@ function getActionOperationDescriptions(array $actions, $type) {
 
 					case OPERATION_TYPE_GROUP_ADD:
 					case OPERATION_TYPE_GROUP_REMOVE:
-						foreach ($operation['groupids'] as $groupid) {
-							$groupids[$groupid] = true;
+						foreach ($operation['opgroup'] as $groupid) {
+							$groupids[$groupid['groupid']] = true;
 						}
 						break;
 
 					case OPERATION_TYPE_TEMPLATE_ADD:
 					case OPERATION_TYPE_TEMPLATE_REMOVE:
-						foreach ($operation['templateids'] as $templateid) {
-							$templateids[$templateid] = true;
+						foreach ($operation['optemplate'] as $templateid) {
+							$templateids[$templateid['templateid']] = true;
 						}
 						break;
 				}
@@ -540,7 +541,7 @@ function getActionOperationDescriptions(array $actions, $type) {
 		]);
 	}
 
-	// format the HTML ouput
+	// Format the HTML output.
 	foreach ($actions as $i => $action) {
 		if ($type == ACTION_OPERATION) {
 			foreach ($action['operations'] as $j => $operation) {
@@ -650,9 +651,9 @@ function getActionOperationDescriptions(array $actions, $type) {
 					case OPERATION_TYPE_GROUP_REMOVE:
 						$host_group_list = [];
 
-						foreach ($operation['groupids'] as $groupid) {
-							if (array_key_exists($groupid, $host_groups)) {
-								$host_group_list[] = $host_groups[$groupid]['name'];
+						foreach ($operation['opgroup'] as $groupid) {
+							if (array_key_exists($groupid['groupid'], $host_groups)) {
+								$host_group_list[] = $host_groups[$groupid['groupid']]['name'];
 							}
 						}
 
@@ -672,9 +673,9 @@ function getActionOperationDescriptions(array $actions, $type) {
 					case OPERATION_TYPE_TEMPLATE_REMOVE:
 						$template_list = [];
 
-						foreach ($operation['templateids'] as $templateid) {
-							if (array_key_exists($templateid, $templates)) {
-								$template_list[] = $templates[$templateid]['name'];
+						foreach ($operation['optemplate'] as $templateid) {
+							if (array_key_exists($templateid['templateid'], $templates)) {
+								$template_list[] = $templates[$templateid['templateid']]['name'];
 							}
 						}
 
@@ -808,22 +809,18 @@ function getActionOperationDescriptions(array $actions, $type) {
 /**
  * Gathers action operation script details and returns the HTML items representing action operation with hint.
  *
- * @param array  $operations								Array of action operations or recovery operations.
- * @param string $operation['operationtype']				Action operation type.
- *															Possible values: OPERATION_TYPE_MESSAGE, OPERATION_TYPE_COMMAND,
- *															OPERATION_TYPE_ACK_MESSAGE and OPERATION_TYPE_RECOVERY_MESSAGE
- * @param string $operation['opcommand']['type']			Action operation command type.
- *															Possible values: ZBX_SCRIPT_TYPE_IPMI, ZBX_SCRIPT_TYPE_SSH,
- *															ZBX_SCRIPT_TYPE_TELNET, ZBX_SCRIPT_TYPE_CUSTOM_SCRIPT
- *															and ZBX_SCRIPT_TYPE_GLOBAL_SCRIPT
- * @param string $operation['opmessage']['default_msg']		'1' to show default message (optional)
- * @param array  $defaultMessage							Array containing default subject and message set via action.
- * @param string $defaultMessage['subject']					Default subject.
- * @param string $defaultMessage['message']					Default message text.
+ * @param array  $operations                      Array of action operations.
+ * @param string $operation['operationtype']      Action operation type.
+ *                                                Possible values: OPERATION_TYPE_MESSAGE, OPERATION_TYPE_COMMAND,
+ *                                                OPERATION_TYPE_ACK_MESSAGE and OPERATION_TYPE_RECOVERY_MESSAGE
+ * @param string $operation['opcommand']['type']  Action operation command type.
+ *                                                Possible values: ZBX_SCRIPT_TYPE_IPMI, ZBX_SCRIPT_TYPE_SSH,
+ *                                                ZBX_SCRIPT_TYPE_TELNET, ZBX_SCRIPT_TYPE_CUSTOM_SCRIPT
+ *                                                and ZBX_SCRIPT_TYPE_GLOBAL_SCRIPT
  *
- * @return array											Returns an array of action operation hints.
+ * @return array  Returns an array of action operation hints.
  */
-function getActionOperationHints(array $operations, array $defaultMessage) {
+function getActionOperationHints(array $operations) {
 	$result = [];
 	$scriptids = [];
 	$scripts = [];
@@ -846,107 +843,59 @@ function getActionOperationHints(array $operations, array $defaultMessage) {
 	foreach ($operations as $key => $operation) {
 		$result[$key] = [];
 
-		switch ($operation['operationtype']) {
-			case OPERATION_TYPE_MESSAGE:
-				// show default entered action subject and message or show custom subject and message for each operation
-				$subject = (isset($operation['opmessage']['default_msg']) && $operation['opmessage']['default_msg'])
-					? $defaultMessage['subject']
-					: $operation['opmessage']['subject'];
+		if ($operation['operationtype'] == OPERATION_TYPE_COMMAND) {
+			switch ($operation['opcommand']['type']) {
+				case ZBX_SCRIPT_TYPE_IPMI:
+					$result[$key][] = [bold(_('Run IPMI command').': '), BR(),
+						italic(zbx_nl2br($operation['opcommand']['command']))
+					];
+					break;
 
-				$message = (isset($operation['opmessage']['default_msg']) && $operation['opmessage']['default_msg'])
-					? $defaultMessage['message']
-					: $operation['opmessage']['message'];
+				case ZBX_SCRIPT_TYPE_SSH:
+					$result[$key][] = [bold(_('Run SSH commands').': '), BR(),
+						italic(zbx_nl2br($operation['opcommand']['command']))
+					];
+					break;
 
-				$result_hint = [];
+				case ZBX_SCRIPT_TYPE_TELNET:
+					$result[$key][] = [bold(_('Run TELNET commands').': '), BR(),
+						italic(zbx_nl2br($operation['opcommand']['command']))
+					];
+					break;
 
-				if (trim($subject)) {
-					$result_hint = [bold($subject), BR(), BR()];
-				}
-
-				if (trim($message)) {
-					$result_hint[] = zbx_nl2br($message);
-				}
-
-				if ($result_hint) {
-					$result[$key][] = $result_hint;
-				}
-				break;
-
-			case OPERATION_TYPE_COMMAND:
-				switch ($operation['opcommand']['type']) {
-					case ZBX_SCRIPT_TYPE_IPMI:
-						$result[$key][] = [bold(_('Run IPMI command').': '), BR(),
-							italic(zbx_nl2br($operation['opcommand']['command']))
+				case ZBX_SCRIPT_TYPE_CUSTOM_SCRIPT:
+					if ($operation['opcommand']['execute_on'] == ZBX_SCRIPT_EXECUTE_ON_AGENT) {
+						$result[$key][] = [bold(_s('Run custom commands on %1$s', _('Zabbix agent')).': '),
+							BR(), italic(zbx_nl2br($operation['opcommand']['command']))
 						];
-						break;
-
-					case ZBX_SCRIPT_TYPE_SSH:
-						$result[$key][] = [bold(_('Run SSH commands').': '), BR(),
-							italic(zbx_nl2br($operation['opcommand']['command']))
+					}
+					elseif ($operation['opcommand']['execute_on'] == ZBX_SCRIPT_EXECUTE_ON_PROXY) {
+						$result[$key][] = [bold(_s('Run custom commands on %1$s', _('Zabbix server (proxy)')).': '),
+							BR(), italic(zbx_nl2br($operation['opcommand']['command']))
 						];
-						break;
-
-					case ZBX_SCRIPT_TYPE_TELNET:
-						$result[$key][] = [bold(_('Run TELNET commands').': '), BR(),
-							italic(zbx_nl2br($operation['opcommand']['command']))
+					}
+					else {
+						$result[$key][] = [bold(_s('Run custom commands on %1$s', _('Zabbix server')).': '),
+							BR(), italic(zbx_nl2br($operation['opcommand']['command']))
 						];
-						break;
+					}
+					break;
 
-					case ZBX_SCRIPT_TYPE_CUSTOM_SCRIPT:
-						if ($operation['opcommand']['execute_on'] == ZBX_SCRIPT_EXECUTE_ON_AGENT) {
-							$result[$key][] = [bold(_s('Run custom commands on %1$s', _('Zabbix agent')).': '),
-								BR(), italic(zbx_nl2br($operation['opcommand']['command']))
-							];
-						}
-						elseif ($operation['opcommand']['execute_on'] == ZBX_SCRIPT_EXECUTE_ON_PROXY) {
-							$result[$key][] = [bold(_s('Run custom commands on %1$s', _('Zabbix server (proxy)')).': '),
-								BR(), italic(zbx_nl2br($operation['opcommand']['command']))
-							];
-						}
-						else {
-							$result[$key][] = [bold(_s('Run custom commands on %1$s', _('Zabbix server')).': '),
-								BR(), italic(zbx_nl2br($operation['opcommand']['command']))
-							];
-						}
-						break;
+				case ZBX_SCRIPT_TYPE_GLOBAL_SCRIPT:
+					$scriptId = $operation['opcommand']['scriptid'];
 
-					case ZBX_SCRIPT_TYPE_GLOBAL_SCRIPT:
-						$scriptId = $operation['opcommand']['scriptid'];
-
-						if (isset($scripts[$scriptId])) {
-							$result[$key][] = [bold(_('Run global script').': '),
-								italic($scripts[$scriptId]['name'])
-							];
-						}
-						break;
-
-					default:
-						$result[$key][] = [bold(_('Run commands').': '), BR(),
-							italic(zbx_nl2br($operation['opcommand']['command']))
+					if (isset($scripts[$scriptId])) {
+						$result[$key][] = [bold(_('Run global script').': '),
+							italic($scripts[$scriptId]['name'])
 						];
-				}
-				break;
+					}
+					break;
 
-			case OPERATION_TYPE_ACK_MESSAGE:
-			case OPERATION_TYPE_RECOVERY_MESSAGE:
-				$result_hint = [];
-				$message = (array_key_exists('default_msg', $operation['opmessage'])
-					&& $operation['opmessage']['default_msg'])
-					? $defaultMessage
-					: $operation['opmessage'];
-
-				if (trim($message['subject'])) {
-					$result_hint = [bold($message['subject']), BR(), BR()];
-				}
-
-				if (trim($message['message'])) {
-					$result_hint[] = zbx_nl2br($message['message']);
-				}
-
-				if ($result_hint) {
-					$result[$key][] = $result_hint;
-				}
-				break;
+				default:
+					$result[$key][] = [bold(_('Run commands').': '), BR(),
+						italic(zbx_nl2br($operation['opcommand']['command']))
+					];
+			}
 		}
 	}
 
@@ -986,7 +935,7 @@ function get_conditions_by_eventsource($eventsource) {
 		CONDITION_TYPE_DSERVICE_TYPE,
 		CONDITION_TYPE_DUPTIME
 	];
-	$conditions[EVENT_SOURCE_AUTO_REGISTRATION] = [
+	$conditions[EVENT_SOURCE_AUTOREGISTRATION] = [
 		CONDITION_TYPE_HOST_NAME,
 		CONDITION_TYPE_HOST_METADATA,
 		CONDITION_TYPE_PROXY
@@ -1060,7 +1009,7 @@ function getAllowedOperations($eventsource) {
 		];
 	}
 
-	if ($eventsource == EVENT_SOURCE_AUTO_REGISTRATION) {
+	if ($eventsource == EVENT_SOURCE_AUTOREGISTRATION) {
 		$operations[ACTION_OPERATION] = [
 			OPERATION_TYPE_MESSAGE,
 			OPERATION_TYPE_COMMAND,
@@ -1712,14 +1661,13 @@ function getEventUpdates(array $event) {
  * @param array  $actions['actions']     Actions icon data.
  * @param array  $mediatypes             Mediatypes with maxattempts value and name.
  * @param array  $users                  User name, surname and alias.
- * @param array  $config                 Zabbix config.
  *
  * @return CCol|string
  */
-function makeEventActionsIcons($eventid, $actions, $mediatypes, $users, $config) {
+function makeEventActionsIcons($eventid, $actions, $mediatypes, $users) {
 	$messages_icon = makeEventMessagesIcon($actions['messages'][$eventid], $users);
-	$severities_icon = makeEventSeverityChangesIcon($actions['severities'][$eventid], $users, $config);
-	$actions_icon = makeEventActionsIcon($actions['actions'][$eventid], $users, $mediatypes, $config);
+	$severities_icon = makeEventSeverityChangesIcon($actions['severities'][$eventid], $users);
+	$actions_icon = makeEventActionsIcon($actions['actions'][$eventid], $users, $mediatypes);
 
 	$action_icons = [];
 	if ($messages_icon !== null) {
@@ -1796,11 +1744,10 @@ function makeEventMessagesIcon(array $data, array $users) {
  * @param string $data['current_severity']              Current severity.
  * @param int    $data['count']                         Total number of severity changes.
  * @param array  $users                                 User name, surname and alias.
- * @param array  $config                                Zabbix config.
  *
  * @return CSpan|null
  */
-function makeEventSeverityChangesIcon(array $data, array $users, array $config) {
+function makeEventSeverityChangesIcon(array $data, array $users) {
 	$total = $data['count'];
 
 	$table = (new CTableInfo())->setHeader([_('Time'), _('User'), _('Severity changes')]);
@@ -1812,8 +1759,8 @@ function makeEventSeverityChangesIcon(array $data, array $users, array $config)
 		$severity['action_type'] = ZBX_EVENT_HISTORY_MANUAL_UPDATE;
 
 		// severity changes
-		$old_severity_name = getSeverityName($severity['old_severity'], $config);
-		$new_severity_name = getSeverityName($severity['new_severity'], $config);
+		$old_severity_name = getSeverityName($severity['old_severity']);
+		$new_severity_name = getSeverityName($severity['new_severity']);
 
 		$table->addRow([
 			zbx_date2str(DATE_TIME_FORMAT_SECONDS, $severity['clock']),
@@ -1873,11 +1820,10 @@ function makeEventSeverityChangesIcon(array $data, array $users, array $config)
  * @param array  $users                             User name, surname and alias.
  * @param array  $mediatypes                        Mediatypes with maxattempts value and name.
  * @param string $mediatypes[]['name']              Mediatype name.
- * @param array  $config                            Zabbix config.
  *
  * @return CSpan|null
  */
-function makeEventActionsIcon(array $data, array $users, array $mediatypes, array $config) {
+function makeEventActionsIcon(array $data, array $users, array $mediatypes) {
 	// Number of meaningful actions.
 	$total = $data['count'];
 	// Number of all action entries.
@@ -1909,7 +1855,7 @@ function makeEventActionsIcon(array $data, array $users, array $mediatypes, arra
 		$table->addRow([
 			zbx_date2str(DATE_TIME_FORMAT_SECONDS, $action['clock']),
 			makeActionTableUser($action, $users),
-			makeActionTableIcon($action, $config),
+			makeActionTableIcon($action),
 			$message,
 			makeActionTableStatus($action),
 			makeActionTableInfo($action, $mediatypes)
@@ -1962,11 +1908,10 @@ function makeEventActionsIcon(array $data, array $users, array $mediatypes, arra
  * @param string $data['actions'][]['acknowledgeid']  Problem update action that was reason for alert (only for ZBX_EVENT_HISTORY_ALERT).
  * @param array  $users                               User name, surname and alias.
  * @param array  $mediatypes                          Mediatypes with maxattempts value.
- * @param array  $config                              Zabbix config.
  *
  * @return CTableInfo
  */
-function makeEventDetailsActionsTable(array $data, array $users, array $mediatypes, array $config) {
+function makeEventDetailsActionsTable(array $data, array $users, array $mediatypes) {
 	$table = (new CTableInfo())->setHeader([
 		_('Step'), _('Time'), _('User/Recipient'), _('Action'), _('Message/Command'), _('Status'), _('Info')
 	]);
@@ -1996,7 +1941,7 @@ function makeEventDetailsActionsTable(array $data, array $users, array $mediatyp
 			$esc_step,
 			zbx_date2str(DATE_TIME_FORMAT_SECONDS, $action['clock']),
 			makeEventDetailsTableUser($action, $users),
-			makeActionTableIcon($action, $config),
+			makeActionTableIcon($action),
 			$message,
 			makeActionTableStatus($action),
 			makeActionTableInfo($action, $mediatypes)
@@ -2013,12 +1958,13 @@ function makeEventDetailsActionsTable(array $data, array $users, array $mediatyp
  * @param string $actions[]['clock']        Time, when action was performed.
  * @param string $actions[]['message']      Message sent by alert, or written by manual update, or remote command text.
  * @param array  $users                     User name, surname and alias.
- * @param array  $config                    Zabbix config.
  *
  * @return CTable
  */
-function makeEventHistoryTable(array $actions, array $users, array $config) {
-	$table = (new CTable())->setHeader([_('Time'), _('User'), _('User action'), _('Message')]);
+function makeEventHistoryTable(array $actions, array $users) {
+	$table = (new CTable())
+		->addStyle('width: 100%;')
+		->setHeader([_('Time'), _('User'), _('User action'), _('Message')]);
 
 	foreach ($actions as $action) {
 		// Added in order to reuse makeActionTableUser() and makeActionTableIcon()
@@ -2027,8 +1973,8 @@ function makeEventHistoryTable(array $actions, array $users, array $config) {
 		$table->addRow([
 			zbx_date2str(DATE_TIME_FORMAT_SECONDS, $action['clock']),
 			makeActionTableUser($action, $users),
-			makeActionTableIcon($action, $config),
-			zbx_nl2br($action['message'])
+			makeActionTableIcon($action),
+			(new CCol(zbx_nl2br($action['message'])))->addClass(ZBX_STYLE_TABLE_FORMS_OVERFLOW_BREAK)
 		]);
 	}
 
@@ -2095,11 +2041,10 @@ function makeEventDetailsTableUser(array $action, array $users) {
  * @param int    $action['old_severity']  Severity before problem update. (only for ZBX_EVENT_HISTORY_MANUAL_UPDATE)
  * @param int    $action['new_severity']  Severity after problem update. (only for ZBX_EVENT_HISTORY_MANUAL_UPDATE)
  * @param int    $action['alerttype']     Type of alert. (only for ZBX_EVENT_HISTORY_ALERT)
- * @param array  $config                  Zabbix config.
  *
  * @return CSpan
  */
-function makeActionTableIcon(array $action, array $config) {
+function makeActionTableIcon(array $action) {
 	switch ($action['action_type']) {
 		case ZBX_EVENT_HISTORY_PROBLEM_EVENT:
 			return makeActionIcon(['icon' => ZBX_STYLE_PROBLEM_GENERATED, 'title' => _('Problem created')]);
@@ -2121,6 +2066,10 @@ function makeActionTableIcon(array $action, array $config) {
 				$action_icons[] = makeActionIcon(['icon' => ZBX_STYLE_ACTION_ICON_ACK, 'title' => _('Acknowledged')]);
 			}
 
+			if (($action['action'] & ZBX_PROBLEM_UPDATE_UNACKNOWLEDGE) == ZBX_PROBLEM_UPDATE_UNACKNOWLEDGE) {
+				$action_icons[] = makeActionIcon(['icon' => ZBX_STYLE_ACTION_ICON_UNACK, 'title' => _('Unacknowledged')]);
+			}
+
 			if (($action['action'] & ZBX_PROBLEM_UPDATE_MESSAGE) == ZBX_PROBLEM_UPDATE_MESSAGE) {
 				$action_icons[] = makeActionIcon(['icon' => ZBX_STYLE_ACTION_ICON_MSG, 'title' => _('Message')]);
 			}
@@ -2130,8 +2079,8 @@ function makeActionTableIcon(array $action, array $config) {
 					? ZBX_STYLE_ACTION_ICON_SEV_UP
 					: ZBX_STYLE_ACTION_ICON_SEV_DOWN;
 
-				$old_severity_name = getSeverityName($action['old_severity'], $config);
-				$new_severity_name = getSeverityName($action['new_severity'], $config);
+				$old_severity_name = getSeverityName($action['old_severity']);
+				$new_severity_name = getSeverityName($action['new_severity']);
 				$hint = $old_severity_name.'&nbsp;&rArr;&nbsp;'.$new_severity_name;
 
 				$action_icons[] = makeActionIcon(['icon' => $action_type, 'hint' => $hint]);
