@@ -24,6 +24,13 @@
  */
 class CDRule extends CApiService {
 
+	public const ACCESS_RULES = [
+		'get' => ['min_user_type' => USER_TYPE_ZABBIX_USER],
+		'create' => ['min_user_type' => USER_TYPE_ZABBIX_ADMIN],
+		'update' => ['min_user_type' => USER_TYPE_ZABBIX_ADMIN],
+		'delete' => ['min_user_type' => USER_TYPE_ZABBIX_ADMIN]
+	];
+
 	protected $tableName = 'drules';
 	protected $tableAlias = 'dr';
 	protected $sortColumns = ['druleid', 'name'];
@@ -139,7 +146,7 @@ class CDRule extends CApiService {
 
 		$sqlParts = $this->applyQueryOutputOptions($this->tableName(), $this->tableAlias(), $options, $sqlParts);
 		$sqlParts = $this->applyQuerySortOptions($this->tableName(), $this->tableAlias(), $options, $sqlParts);
-		$dbRes = DBselect($this->createSelectQueryFromParts($sqlParts), $sqlParts['limit']);
+		$dbRes = DBselect(self::createSelectQueryFromParts($sqlParts), $sqlParts['limit']);
 		while ($drule = DBfetch($dbRes)) {
 			if ($options['countOutput']) {
 				if ($options['groupCount']) {
@@ -542,10 +549,11 @@ class CDRule extends CApiService {
 					}
 
 					$length = mb_strlen($dcheck['key_']);
-					if ($length > 255) {
+					$max_length = DB::getFieldLength('dchecks', 'key_');
+					if ($length > $max_length) {
 						self::exception(ZBX_API_ERROR_PARAMETERS,
 							_s('Incorrect value for field "%1$s": %2$s.', 'key_',
-								_s('%1$d characters exceeds maximum length of %2$d characters', $length, 255)
+								_s('%1$d characters exceeds maximum length of %2$d characters', $length, $max_length)
 							)
 						);
 					}
