@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2018 Zabbix SIA
+** Copyright (C) 2001-2020 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -24,6 +24,13 @@
  */
 class CHttpTest extends CApiService {
 
+	public const ACCESS_RULES = [
+		'get' => ['min_user_type' => USER_TYPE_ZABBIX_USER],
+		'create' => ['min_user_type' => USER_TYPE_ZABBIX_ADMIN],
+		'update' => ['min_user_type' => USER_TYPE_ZABBIX_ADMIN],
+		'delete' => ['min_user_type' => USER_TYPE_ZABBIX_ADMIN]
+	];
+
 	protected $tableName = 'httptest';
 	protected $tableAlias = 'ht';
 	protected $sortColumns = ['httptestid', 'name'];
@@ -144,7 +151,7 @@ class CHttpTest extends CApiService {
 		if (!is_null($options['applicationids'])) {
 			zbx_value2array($options['applicationids']);
 
-			$sqlParts['where'][] = dbConditionInt('ht.applicationid', $options['applicationids']);
+			$sqlParts['where'][] = dbConditionId('ht.applicationid', $options['applicationids']);
 		}
 
 		// inherited
@@ -199,7 +206,7 @@ class CHttpTest extends CApiService {
 
 		$sqlParts = $this->applyQueryOutputOptions($this->tableName(), $this->tableAlias(), $options, $sqlParts);
 		$sqlParts = $this->applyQuerySortOptions($this->tableName(), $this->tableAlias(), $options, $sqlParts);
-		$res = DBselect($this->createSelectQueryFromParts($sqlParts), $sqlParts['limit']);
+		$res = DBselect(self::createSelectQueryFromParts($sqlParts), $sqlParts['limit']);
 		while ($httpTest = DBfetch($res)) {
 			if ($options['countOutput']) {
 				if ($options['groupCount']) {
@@ -254,7 +261,6 @@ class CHttpTest extends CApiService {
 	 * @return array
 	 */
 	public function create($httptests) {
-		$httptests = $this->convertHttpPairs($httptests);
 		$this->validateCreate($httptests);
 
 		$httptests = Manager::HttpTest()->persist($httptests);
@@ -274,7 +280,7 @@ class CHttpTest extends CApiService {
 			'hostid' =>				['type' => API_ID, 'flags' => API_REQUIRED],
 			'name' =>				['type' => API_STRING_UTF8, 'flags' => API_REQUIRED | API_NOT_EMPTY, 'length' => DB::getFieldLength('httptest', 'name')],
 			'applicationid' =>		['type' => API_ID],
-			'delay' =>				['type' => API_TIME_UNIT, 'flags' => API_ALLOW_USER_MACRO, 'in' => '1:'.SEC_PER_DAY],
+			'delay' =>				['type' => API_TIME_UNIT, 'flags' => API_NOT_EMPTY | API_ALLOW_USER_MACRO, 'in' => '1:'.SEC_PER_DAY],
 			'retries' =>			['type' => API_INT32, 'in' => '1:10'],
 			'agent' =>				['type' => API_STRING_UTF8, 'length' => DB::getFieldLength('httptest', 'agent')],
 			'http_proxy' =>			['type' => API_STRING_UTF8, 'length' => DB::getFieldLength('httptest', 'http_proxy')],
@@ -287,7 +293,7 @@ class CHttpTest extends CApiService {
 				'value' =>				['type' => API_STRING_UTF8, 'flags' => API_REQUIRED | API_NOT_EMPTY, 'length' => DB::getFieldLength('httptest_field', 'value')]
 			]],
 			'status' =>				['type' => API_INT32, 'in' => implode(',', [HTTPTEST_STATUS_ACTIVE, HTTPTEST_STATUS_DISABLED])],
-			'authentication' =>		['type' => API_INT32, 'in' => implode(',', [HTTPTEST_AUTH_NONE, HTTPTEST_AUTH_BASIC, HTTPTEST_AUTH_NTLM])],
+			'authentication' =>		['type' => API_INT32, 'in' => implode(',', [HTTPTEST_AUTH_NONE, HTTPTEST_AUTH_BASIC, HTTPTEST_AUTH_NTLM, HTTPTEST_AUTH_KERBEROS, HTTPTEST_AUTH_DIGEST])],
 			'http_user' =>			['type' => API_STRING_UTF8, 'length' => DB::getFieldLength('httptest', 'http_user')],
 			'http_password' =>		['type' => API_STRING_UTF8, 'length' => DB::getFieldLength('httptest', 'http_password')],
 			'verify_peer' =>		['type' => API_INT32, 'in' => implode(',', [HTTPTEST_VERIFY_PEER_OFF, HTTPTEST_VERIFY_PEER_ON])],
@@ -313,8 +319,8 @@ class CHttpTest extends CApiService {
 					'value' =>				['type' => API_STRING_UTF8, 'flags' => API_REQUIRED | API_NOT_EMPTY, 'length' => DB::getFieldLength('httpstep_field', 'value')]
 				]],
 				'follow_redirects' =>	['type' => API_INT32, 'in' => implode(',', [HTTPTEST_STEP_FOLLOW_REDIRECTS_OFF, HTTPTEST_STEP_FOLLOW_REDIRECTS_ON])],
-				'retrieve_mode' =>		['type' => API_INT32, 'in' => implode(',', [HTTPTEST_STEP_RETRIEVE_MODE_CONTENT, HTTPTEST_STEP_RETRIEVE_MODE_HEADERS])],
-				'timeout' =>			['type' => API_TIME_UNIT, 'flags' => API_ALLOW_USER_MACRO, 'in' => '0:'.SEC_PER_HOUR],
+				'retrieve_mode' =>		['type' => API_INT32, 'in' => implode(',', [HTTPTEST_STEP_RETRIEVE_MODE_CONTENT, HTTPTEST_STEP_RETRIEVE_MODE_HEADERS, HTTPTEST_STEP_RETRIEVE_MODE_BOTH])],
+				'timeout' =>			['type' => API_TIME_UNIT, 'flags' => API_NOT_EMPTY | API_ALLOW_USER_MACRO, 'in' => '1:'.SEC_PER_HOUR],
 				'required' =>			['type' => API_STRING_UTF8, 'length' => DB::getFieldLength('httpstep', 'required')],
 				'status_codes' =>		['type' => API_STRING_UTF8, 'length' => DB::getFieldLength('httpstep', 'status_codes')]
 			]]
@@ -343,7 +349,6 @@ class CHttpTest extends CApiService {
 	 * @return array
 	 */
 	public function update($httptests) {
-		$httptests = $this->convertHttpPairs($httptests);
 		$this->validateUpdate($httptests, $db_httptests);
 
 		Manager::HttpTest()->persist($httptests);
@@ -369,7 +374,7 @@ class CHttpTest extends CApiService {
 			'httptestid' =>			['type' => API_ID, 'flags' => API_REQUIRED],
 			'name' =>				['type' => API_STRING_UTF8, 'flags' => API_NOT_EMPTY, 'length' => DB::getFieldLength('httptest', 'name')],
 			'applicationid' =>		['type' => API_ID],
-			'delay' =>				['type' => API_TIME_UNIT, 'flags' => API_ALLOW_USER_MACRO, 'in' => '1:'.SEC_PER_DAY],
+			'delay' =>				['type' => API_TIME_UNIT, 'flags' => API_NOT_EMPTY | API_ALLOW_USER_MACRO, 'in' => '1:'.SEC_PER_DAY],
 			'retries' =>			['type' => API_INT32, 'in' => '1:10'],
 			'agent' =>				['type' => API_STRING_UTF8, 'length' => DB::getFieldLength('httptest', 'agent')],
 			'http_proxy' =>			['type' => API_STRING_UTF8, 'length' => DB::getFieldLength('httptest', 'http_proxy')],
@@ -382,7 +387,7 @@ class CHttpTest extends CApiService {
 				'value' =>				['type' => API_STRING_UTF8, 'flags' => API_REQUIRED | API_NOT_EMPTY, 'length' => DB::getFieldLength('httptest_field', 'value')]
 			]],
 			'status' =>				['type' => API_INT32, 'in' => implode(',', [HTTPTEST_STATUS_ACTIVE, HTTPTEST_STATUS_DISABLED])],
-			'authentication' =>		['type' => API_INT32, 'in' => implode(',', [HTTPTEST_AUTH_NONE, HTTPTEST_AUTH_BASIC, HTTPTEST_AUTH_NTLM])],
+			'authentication' =>		['type' => API_INT32, 'in' => implode(',', [HTTPTEST_AUTH_NONE, HTTPTEST_AUTH_BASIC, HTTPTEST_AUTH_NTLM, HTTPTEST_AUTH_KERBEROS, HTTPTEST_AUTH_DIGEST])],
 			'http_user' =>			['type' => API_STRING_UTF8, 'length' => DB::getFieldLength('httptest', 'http_user')],
 			'http_password' =>		['type' => API_STRING_UTF8, 'length' => DB::getFieldLength('httptest', 'http_password')],
 			'verify_peer' =>		['type' => API_INT32, 'in' => implode(',', [HTTPTEST_VERIFY_PEER_OFF, HTTPTEST_VERIFY_PEER_ON])],
@@ -409,8 +414,8 @@ class CHttpTest extends CApiService {
 					'value' =>				['type' => API_STRING_UTF8, 'flags' => API_REQUIRED | API_NOT_EMPTY, 'length' => DB::getFieldLength('httpstep_field', 'value')]
 				]],
 				'follow_redirects' =>	['type' => API_INT32, 'in' => implode(',', [HTTPTEST_STEP_FOLLOW_REDIRECTS_OFF, HTTPTEST_STEP_FOLLOW_REDIRECTS_ON])],
-				'retrieve_mode' =>		['type' => API_INT32, 'in' => implode(',', [HTTPTEST_STEP_RETRIEVE_MODE_CONTENT, HTTPTEST_STEP_RETRIEVE_MODE_HEADERS])],
-				'timeout' =>			['type' => API_TIME_UNIT, 'flags' => API_ALLOW_USER_MACRO, 'in' => '0:'.SEC_PER_HOUR],
+				'retrieve_mode' =>		['type' => API_INT32, 'in' => implode(',', [HTTPTEST_STEP_RETRIEVE_MODE_CONTENT, HTTPTEST_STEP_RETRIEVE_MODE_HEADERS, HTTPTEST_STEP_RETRIEVE_MODE_BOTH])],
+				'timeout' =>			['type' => API_TIME_UNIT, 'flags' => API_NOT_EMPTY | API_ALLOW_USER_MACRO, 'in' => '1:'.SEC_PER_HOUR],
 				'required' =>			['type' => API_STRING_UTF8, 'length' => DB::getFieldLength('httpstep', 'required')],
 				'status_codes' =>		['type' => API_STRING_UTF8, 'length' => DB::getFieldLength('httpstep', 'status_codes')]
 			]]
@@ -425,6 +430,9 @@ class CHttpTest extends CApiService {
 				'status', 'authentication', 'http_user', 'http_password', 'verify_peer', 'verify_host',
 				'ssl_cert_file', 'ssl_key_file', 'ssl_key_password', 'templateid'
 			],
+			'selectSteps' => ['httpstepid', 'name', 'no', 'url', 'timeout', 'posts', 'required',
+				'status_codes', 'follow_redirects', 'retrieve_mode', 'post_type'
+			],
 			'httptestids' => zbx_objectValues($httptests, 'httptestid'),
 			'editable' => true,
 			'preservekeys' => true
@@ -433,21 +441,10 @@ class CHttpTest extends CApiService {
 		foreach ($db_httptests as &$db_httptest) {
 			$db_httptest['headers'] = [];
 			$db_httptest['variables'] = [];
-			$db_httptest['steps'] = [];
+			$db_httptest['steps'] = zbx_toHash($db_httptest['steps'], 'httpstepid');
 		}
 		unset($db_httptest);
 
-		$db_httpsteps = DB::select('httpstep', [
-			'output' => ['httpstepid', 'httptestid', 'name', 'no', 'url', 'timeout', 'posts', 'required',
-				'status_codes', 'follow_redirects', 'retrieve_mode'
-			],
-			'filter' => ['httptestid' => array_keys($db_httptests)]
-		]);
-
-		foreach ($db_httpsteps as $db_httpstep) {
-			$db_httptests[$db_httpstep['httptestid']]['steps'][$db_httpstep['httpstepid']] = $db_httpstep;
-		}
-
 		$names_by_hostid = [];
 
 		foreach ($httptests as $httptest) {
@@ -596,7 +593,7 @@ class CHttpTest extends CApiService {
 		}
 
 		if ($del_itemids) {
-			API::Item()->delete($del_itemids, true);
+			CItemManager::delete($del_itemids);
 		}
 
 		DB::delete('httptest', ['httptestid' => $del_httptestids]);
@@ -768,16 +765,16 @@ class CHttpTest extends CApiService {
 
 	/**
 	 * Validate http response code range.
-	 * Range can be empty string, can be set as user macro or be numeric and contain ',' and '-'.
+	 * Range can be empty string or list of comma separated numeric strings or user macros.
 	 *
-	 * Examples: '100-199, 301, 404, 500-550' or '{$USER_MACRO123}'
+	 * Examples: '100-199, 301, 404, 500-550, {$MACRO}-200, {$MACRO}-{$MACRO}'
 	 *
 	 * @param array $httptests
 	 *
 	 * @throws APIException if the status code range is invalid.
 	 */
 	private function checkStatusCodes(array $httptests) {
-		$user_macro_parser = new CUserMacroParser();
+		$ranges_parser = new CRangesParser(['usermacros' => true]);
 
 		foreach ($httptests as $httptest) {
 			if (!array_key_exists('steps', $httptest)) {
@@ -785,29 +782,14 @@ class CHttpTest extends CApiService {
 			}
 
 			foreach ($httptest['steps'] as $httpstep) {
-				if (!array_key_exists('status_codes', $httpstep)) {
+				if (!array_key_exists('status_codes', $httpstep) || $httpstep['status_codes'] === '') {
 					continue;
 				}
 
-				$status_codes = $httpstep['status_codes'];
-
-				if ($status_codes === '' || $user_macro_parser->parse($status_codes) == CParser::PARSE_SUCCESS) {
-					continue;
-				}
-
-				foreach (explode(',', $status_codes) as $range) {
-					$range = explode('-', $range);
-					if (count($range) > 2) {
-						self::exception(ZBX_API_ERROR_PARAMETERS, _s('Invalid response code "%1$s".', $status_codes));
-					}
-
-					foreach ($range as $value) {
-						if (!is_numeric($value)) {
-							self::exception(ZBX_API_ERROR_PARAMETERS,
-								_s('Invalid response code "%1$s".', $status_codes)
-							);
-						}
-					}
+				if ($ranges_parser->parse($httpstep['status_codes']) != CParser::PARSE_SUCCESS) {
+					self::exception(ZBX_API_ERROR_PARAMETERS,
+						_s('Invalid response code "%1$s".', $httpstep['status_codes'])
+					);
 				}
 			}
 		}
@@ -897,7 +879,7 @@ class CHttpTest extends CApiService {
 
 				$db_httpsteps = API::getApiService()->select('httpstep', [
 					'output' => $this->outputExtend($options['selectSteps'], ['httptestid', 'httpstepid', 'post_type']),
-					'filters' => ['httptestid' => $httpTestIds],
+					'filter' => ['httptestid' => $httpTestIds],
 					'preservekeys' => true
 				]);
 				$relationMap = $this->createRelationMap($db_httpsteps, 'httptestid', 'httpstepid');
@@ -965,10 +947,8 @@ class CHttpTest extends CApiService {
 	 */
 	private function validateAuthParameters(array &$httptests, $method, array $db_httptests = null) {
 		foreach ($httptests as &$httptest) {
-			if (array_key_exists('authentication', $httptest)
-					|| array_key_exists('http_user', $httptest)
+			if (array_key_exists('authentication', $httptest) || array_key_exists('http_user', $httptest)
 					|| array_key_exists('http_password', $httptest)) {
-
 				$httptest += [
 					'authentication' => ($method === 'validateUpdate')
 						? $db_httptests[$httptest['httptestid']]['authentication']
@@ -986,21 +966,6 @@ class CHttpTest extends CApiService {
 						}
 					}
 				}
-				else {
-					foreach (['http_user', 'http_password'] as $field_name) {
-						$httptest += [
-							$field_name => ($method === 'validateUpdate')
-								? $db_httptests[$httptest['httptestid']][$field_name]
-								: ''
-						];
-
-						if ($httptest[$field_name] === '') {
-							self::exception(ZBX_API_ERROR_PARAMETERS,
-								_s('Incorrect value for field "%1$s": %2$s.', $field_name, _('cannot be empty'))
-							);
-						}
-					}
-				}
 			}
 		}
 		unset($httptest);
@@ -1078,21 +1043,21 @@ class CHttpTest extends CApiService {
 					else {
 						$db_httptest = $db_httptests[$httptest['httptestid']];
 						$db_httpstep = $db_httptest['steps'][$httpstep['httpstepid']];
-						$httpstep += ['retrieve_mode' => $db_httpstep['retrieve_mode']];
 						$httpstep += [
-							'posts' => ($httpstep['retrieve_mode'] == HTTPTEST_STEP_RETRIEVE_MODE_CONTENT)
+							'retrieve_mode' => $db_httpstep['retrieve_mode'],
+							'required' => $db_httpstep['required'],
+							'posts' => ($db_httpstep['retrieve_mode'] != HTTPTEST_STEP_RETRIEVE_MODE_HEADERS)
 								? $db_httpstep['posts']
-								: '',
-							'required' => ($httpstep['retrieve_mode'] == HTTPTEST_STEP_RETRIEVE_MODE_CONTENT)
-								? $db_httpstep['required']
 								: ''
 						];
 					}
 
 					if ($httpstep['retrieve_mode'] == HTTPTEST_STEP_RETRIEVE_MODE_HEADERS) {
-						if (($httpstep['posts'] !== '' && $httpstep['posts'] !== []) || $httpstep['required'] !== '') {
+						if ($httpstep['posts'] !== '' && $httpstep['posts'] !== []) {
+							$field_name = $httpstep['required'] !== '' ? 'required' : 'posts';
+
 							self::exception(ZBX_API_ERROR_PARAMETERS,
-								_s('Incorrect value for field "%1$s": %2$s.', $field_name, _('should be empty'))
+								_s('Incorrect value for field "%1$s": %2$s.', 'posts', _('should be empty'))
 							);
 						}
 					}
@@ -1102,72 +1067,4 @@ class CHttpTest extends CApiService {
 		}
 		unset($httptest);
 	}
-
-	/**
-	 * Convert string to HTTP pair array.
-	 *
-	 * @param string $data
-	 * @param string $delimiter
-	 *
-	 * @return mixed
-	 */
-	private function convertHTTPPairString($data, $delimiter) {
-		/* converts to pair array */
-		$pairs = array_values(array_filter(explode("\n", str_replace("\r", "\n", $data))));
-		foreach ($pairs as &$pair) {
-			$pair = explode($delimiter, $pair, 2);
-			$pair = [
-				'name' => $pair[0],
-				'value' => array_key_exists(1, $pair) ? $pair[1] : ''
-			];
-		}
-		unset($pair);
-
-		return $pairs;
-	}
-
-	/**
-	 * Convert headers and variables from string to HTTP pair array.
-	 * @deprecated conversion will be removed in future
-	 *
-	 * @param array  $httptests
-	 *
-	 * @return array
-	 */
-	private function convertHttpPairs($httptests) {
-		reset($httptests);
-
-		if (!is_int(key($httptests))) {
-			$httptests = [$httptests];
-		}
-
-		$fields = [
-			'headers' => ':',
-			'variables' => '='
-		];
-
-		foreach ($httptests as &$httptest) {
-			foreach ($fields as $field => $delimiter) {
-				if (is_array($httptest) && array_key_exists($field, $httptest) && is_string($httptest[$field])) {
-					$this->deprecated('using string format for field "'.$field.'" is deprecated.');
-					$httptest[$field] = $this->convertHTTPPairString($httptest[$field], $delimiter);
-				}
-			}
-
-			if (array_key_exists('steps', $httptest) && is_array($httptest['steps'])) {
-				foreach ($httptest['steps'] as &$step) {
-					foreach ($fields as $field => $delimiter) {
-						if (is_array($step) && array_key_exists($field, $step) && is_string($step[$field])) {
-							$this->deprecated('using string format for field "'.$field.'" is deprecated.');
-							$step[$field] = $this->convertHTTPPairString($step[$field], $delimiter);
-						}
-					}
-				}
-				unset($step);
-			}
-		}
-		unset($httptest);
-
-		return $httptests;
-	}
 }
