@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2019 Zabbix SIA
+** Copyright (C) 2001-2020 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -24,6 +24,13 @@
  */
 abstract class CItemGeneral extends CApiService {
 
+	public const ACCESS_RULES = [
+		'get' => ['min_user_type' => USER_TYPE_ZABBIX_USER],
+		'create' => ['min_user_type' => USER_TYPE_ZABBIX_ADMIN],
+		'update' => ['min_user_type' => USER_TYPE_ZABBIX_ADMIN],
+		'delete' => ['min_user_type' => USER_TYPE_ZABBIX_ADMIN]
+	];
+
 	const ERROR_EXISTS_TEMPLATE = 'existsTemplate';
 	const ERROR_EXISTS = 'exists';
 	const ERROR_NO_INTERFACE = 'noInterface';
@@ -48,7 +55,6 @@ abstract class CItemGeneral extends CApiService {
 		// host - value should be null for template items
 		$this->fieldRules = [
 			'type'					=> ['template' => 1],
-			'snmp_community'		=> [],
 			'snmp_oid'				=> ['template' => 1],
 			'hostid'				=> [],
 			'name'					=> ['template' => 1],
@@ -59,16 +65,10 @@ abstract class CItemGeneral extends CApiService {
 			'history'				=> [],
 			'trends'				=> [],
 			'status'				=> [],
+			'discover'				=> [],
 			'value_type'			=> ['template' => 1],
 			'trapper_hosts'			=> [],
 			'units'					=> ['template' => 1],
-			'snmpv3_contextname'	=> [],
-			'snmpv3_securityname'	=> [],
-			'snmpv3_securitylevel'	=> [],
-			'snmpv3_authprotocol'	=> [],
-			'snmpv3_authpassphrase'	=> [],
-			'snmpv3_privprotocol'	=> [],
-			'snmpv3_privpassphrase'	=> [],
 			'formula'				=> ['template' => 1],
 			'error'					=> ['system' => 1],
 			'lastlogsize'			=> ['system' => 1],
@@ -86,14 +86,15 @@ abstract class CItemGeneral extends CApiService {
 			'flags'					=> [],
 			'filter'				=> [],
 			'interfaceid'			=> ['host' => 1],
-			'port'					=> [],
 			'inventory_link'		=> [],
 			'lifetime'				=> [],
 			'preprocessing'			=> ['template' => 1],
+			'overrides'				=> ['template' => 1],
 			'jmx_endpoint'			=> [],
 			'url'					=> ['template' => 1],
 			'timeout'				=> ['template' => 1],
 			'query_fields'			=> ['template' => 1],
+			'parameters'			=> ['template' => 1],
 			'posts'					=> ['template' => 1],
 			'status_codes'			=> ['template' => 1],
 			'follow_redirects'		=> ['template' => 1],
@@ -275,6 +276,10 @@ abstract class CItemGeneral extends CApiService {
 
 				// apply rules
 				foreach ($this->fieldRules as $field => $rules) {
+					if ($fullItem['type'] == ITEM_TYPE_SCRIPT) {
+						$rules['template'] = 1;
+					}
+
 					if ((0 != $fullItem['templateid'] && isset($rules['template'])) || isset($rules['system'])) {
 						unset($item[$field]);
 
@@ -327,53 +332,59 @@ abstract class CItemGeneral extends CApiService {
 				}
 			}
 
+			if ($fullItem['type'] == ITEM_TYPE_CALCULATED) {
+				$api_input_rules = ['type' => API_OBJECT, 'fields' => [
+					'params' => ['type' => API_CALC_FORMULA, 'flags' => $this instanceof CItemPrototype ? API_ALLOW_LLD_MACRO : 0],
+				]];
+
+				$data = array_intersect_key($item, $api_input_rules['fields']);
+
+				if (!CApiInputValidator::validate($api_input_rules, $data, '/'.($inum + 1), $error)) {
+					self::exception(ZBX_API_ERROR_PARAMETERS, $error);
+				}
+			}
+
+			if ($fullItem['type'] == ITEM_TYPE_SCRIPT) {
+				if ($update) {
+					if ($dbItems[$item['itemid']]['type'] == $fullItem['type']) {
+						$flags = API_NOT_EMPTY;
+					}
+					else {
+						$flags = API_REQUIRED | API_NOT_EMPTY;
+					}
+				}
+				else {
+					$flags = API_REQUIRED | API_NOT_EMPTY;
+				}
+
+				$api_input_rules = ['type' => API_OBJECT, 'fields' => [
+					'params' => ['type' => API_STRING_UTF8, 'flags' => $flags, 'length' => DB::getFieldLength('items', 'params')],
+					'timeout' => [
+						'type' => API_TIME_UNIT, 'flags' => ($this instanceof CItemPrototype)
+							? $flags | API_ALLOW_USER_MACRO | API_ALLOW_LLD_MACRO
+							: $flags | API_ALLOW_USER_MACRO,
+						'in' => '1:'.SEC_PER_MIN
+					],
+					'parameters' => ['type' => API_OBJECTS, 'flags' => API_NORMALIZE, 'uniq' => [['name']], 'fields' => [
+						'name' =>		['type' => API_STRING_UTF8, 'flags' => API_REQUIRED | API_NOT_EMPTY, 'length' => DB::getFieldLength('item_parameter', 'name')],
+						'value' =>		['type' => API_STRING_UTF8, 'flags' => API_REQUIRED, 'length' => DB::getFieldLength('item_parameter', 'value')]
+					]]
+				]];
+
+				$data = array_intersect_key($item, $api_input_rules['fields']);
+
+				if (!CApiInputValidator::validate($api_input_rules, $data, '/'.($inum + 1), $error)) {
+					self::exception(ZBX_API_ERROR_PARAMETERS, $error);
+				}
+			}
+
 			$host = $dbHosts[$fullItem['hostid']];
 
 			// Validate update interval.
-			if (!in_array($fullItem['type'], [ITEM_TYPE_TRAPPER, ITEM_TYPE_SNMPTRAP, ITEM_TYPE_DEPENDENT])) {
-				if ($update_interval_parser->parse($fullItem['delay']) != CParser::PARSE_SUCCESS) {
-					self::exception(ZBX_API_ERROR_PARAMETERS,
-						_s('Incorrect value for field "%1$s": %2$s.', 'delay', _('invalid delay'))
-					);
-				}
-
-				$delay = $update_interval_parser->getDelay();
-
-				// Check if not macros. If delay is a macro, skip this step, otherwise check if delay is valid.
-				if ($delay[0] !== '{') {
-					$delay_sec = timeUnitToSeconds($delay);
-					$intervals = $update_interval_parser->getIntervals();
-					$flexible_intervals = $update_interval_parser->getIntervals(ITEM_DELAY_FLEXIBLE);
-					$has_scheduling_intervals = (bool) $update_interval_parser->getIntervals(ITEM_DELAY_SCHEDULING);
-					$has_macros = false;
-
-					foreach ($intervals as $interval) {
-						if (strpos($interval['interval'], '{') !== false) {
-							$has_macros = true;
-							break;
-						}
-					}
-
-					// If delay is 0, there must be at least one either flexible or scheduling interval.
-					if ($delay_sec == 0 && !$intervals) {
-						self::exception(ZBX_API_ERROR_PARAMETERS,
-							_('Item will not be refreshed. Specified update interval requires having at least one either flexible or scheduling interval.')
-						);
-					}
-					elseif ($delay_sec < 0 || $delay_sec > SEC_PER_DAY) {
-						self::exception(ZBX_API_ERROR_PARAMETERS,
-							_('Item will not be refreshed. Update interval should be between 1s and 1d. Also Scheduled/Flexible intervals can be used.')
-						);
-					}
-
-					// If there are scheduling intervals or intervals with macros, skip the next check calculation.
-					if (!$has_macros && !$has_scheduling_intervals && $flexible_intervals
-							&& calculateItemNextCheck(0, $delay_sec, $flexible_intervals, time()) == ZBX_JAN_2038) {
-						self::exception(ZBX_API_ERROR_PARAMETERS,
-							_('Item will not be refreshed. Please enter a correct update interval.')
-						);
-					}
-				}
+			if (!in_array($fullItem['type'], [ITEM_TYPE_TRAPPER, ITEM_TYPE_SNMPTRAP, ITEM_TYPE_DEPENDENT])
+					&& ($fullItem['type'] != ITEM_TYPE_ZABBIX_ACTIVE || strncmp($fullItem['key_'], 'mqtt.get', 8) !== 0)
+					&& !validateDelay($update_interval_parser, 'delay', $fullItem['delay'], $error)) {
+				self::exception(ZBX_API_ERROR_PARAMETERS, $error);
 			}
 
 			// For non-numeric types, whichever value was entered in trends field, is overwritten to zero.
@@ -533,20 +544,28 @@ abstract class CItemGeneral extends CApiService {
 
 			// ssh, telnet
 			if ($fullItem['type'] == ITEM_TYPE_SSH || $fullItem['type'] == ITEM_TYPE_TELNET) {
-				if (zbx_empty($fullItem['username'])) {
+				if ($fullItem['username'] === '') {
 					self::exception(ZBX_API_ERROR_PARAMETERS, _('No authentication user name specified.'));
 				}
 
 				if ($fullItem['type'] == ITEM_TYPE_SSH && $fullItem['authtype'] == ITEM_AUTHTYPE_PUBLICKEY) {
-					if (zbx_empty($fullItem['publickey'])) {
+					if ($fullItem['publickey'] === '') {
 						self::exception(ZBX_API_ERROR_PARAMETERS, _('No public key file specified.'));
 					}
-					if (zbx_empty($fullItem['privatekey'])) {
+					if ($fullItem['privatekey'] === '') {
 						self::exception(ZBX_API_ERROR_PARAMETERS, _('No private key file specified.'));
 					}
 				}
 			}
 
+			// Prevent IPMI sensor field being empty if item key is not "ipmi.get".
+			if ($fullItem['type'] == ITEM_TYPE_IPMI && $fullItem['key_'] !== 'ipmi.get'
+					&& (!array_key_exists('ipmi_sensor', $fullItem) || $fullItem['ipmi_sensor'] === '')) {
+				self::exception(ZBX_API_ERROR_PARAMETERS, _s('Incorrect value for field "%1$s": %2$s.',
+					'ipmi_sensor', _('cannot be empty')
+				));
+			}
+
 			// snmp trap
 			if ($fullItem['type'] == ITEM_TYPE_SNMPTRAP
 					&& $fullItem['key_'] !== 'snmptrap.fallback' && $item_key_parser->getKey() !== 'snmptrap') {
@@ -554,43 +573,11 @@ abstract class CItemGeneral extends CApiService {
 			}
 
 			// snmp oid
-			if ((in_array($fullItem['type'], [ITEM_TYPE_SNMPV1, ITEM_TYPE_SNMPV2C, ITEM_TYPE_SNMPV3]))
-					&& zbx_empty($fullItem['snmp_oid'])) {
+			if ($fullItem['type'] == ITEM_TYPE_SNMP
+					&& (!array_key_exists('snmp_oid', $fullItem) || $fullItem['snmp_oid'] === '')) {
 				self::exception(ZBX_API_ERROR_PARAMETERS, _('No SNMP OID specified.'));
 			}
 
-			// snmp community
-			if (in_array($fullItem['type'], [ITEM_TYPE_SNMPV1, ITEM_TYPE_SNMPV2C])
-					&& zbx_empty($fullItem['snmp_community'])) {
-				self::exception(ZBX_API_ERROR_PARAMETERS, _('No SNMP community specified.'));
-			}
-
-			// snmp port
-			if (isset($fullItem['port']) && !zbx_empty($fullItem['port']) && !validatePortNumberOrMacro($fullItem['port'])) {
-				self::exception(ZBX_API_ERROR_PARAMETERS,
-					_s('Item "%1$s:%2$s" has invalid port: "%3$s".', $fullItem['name'], $fullItem['key_'], $fullItem['port']));
-			}
-
-			if (isset($fullItem['snmpv3_securitylevel']) && $fullItem['snmpv3_securitylevel'] != ITEM_SNMPV3_SECURITYLEVEL_NOAUTHNOPRIV) {
-				// snmpv3 authprotocol
-				if (str_in_array($fullItem['snmpv3_securitylevel'], [ITEM_SNMPV3_SECURITYLEVEL_AUTHNOPRIV, ITEM_SNMPV3_SECURITYLEVEL_AUTHPRIV])) {
-					if (isset($fullItem['snmpv3_authprotocol']) && (zbx_empty($fullItem['snmpv3_authprotocol'])
-							|| !str_in_array($fullItem['snmpv3_authprotocol'],
-								[ITEM_AUTHPROTOCOL_MD5, ITEM_AUTHPROTOCOL_SHA]))) {
-						self::exception(ZBX_API_ERROR_PARAMETERS, _s('Incorrect authentication protocol for item "%1$s".', $fullItem['name']));
-					}
-				}
-
-				// snmpv3 privprotocol
-				if ($fullItem['snmpv3_securitylevel'] == ITEM_SNMPV3_SECURITYLEVEL_AUTHPRIV) {
-					if (isset($fullItem['snmpv3_privprotocol']) && (zbx_empty($fullItem['snmpv3_privprotocol'])
-							|| !str_in_array($fullItem['snmpv3_privprotocol'],
-								[ITEM_PRIVPROTOCOL_DES, ITEM_PRIVPROTOCOL_AES]))) {
-						self::exception(ZBX_API_ERROR_PARAMETERS, _s('Incorrect privacy protocol for item "%1$s".', $fullItem['name']));
-					}
-				}
-			}
-
 			if (isset($item['applications']) && $item['applications']) {
 				/*
 				 * 'flags' is available for update and item prototypes.
@@ -643,7 +630,9 @@ abstract class CItemGeneral extends CApiService {
 		}
 
 		if (array_key_exists('type', $item) &&
-				($item['type'] == ITEM_TYPE_DEPENDENT || $item['type'] == ITEM_TYPE_TRAPPER)) {
+				($item['type'] == ITEM_TYPE_DEPENDENT || $item['type'] == ITEM_TYPE_TRAPPER
+					|| ($item['type'] == ITEM_TYPE_ZABBIX_ACTIVE && array_key_exists('key_', $item)
+						&& strncmp($item['key_'], 'mqtt.get', 8) === 0))) {
 			$item['delay'] = 0;
 		}
 
@@ -756,7 +745,7 @@ abstract class CItemGeneral extends CApiService {
 				if ($this instanceof CItemPrototype) {
 					unset($new_item['ruleid']);
 				}
-				$upd_items[] = $new_item;
+				$upd_items[$new_item['itemid']] = $new_item;
 			}
 			else {
 				$ins_items[] = $new_item;
@@ -1235,7 +1224,9 @@ abstract class CItemGeneral extends CApiService {
 	 *                                                                  21 - ZBX_PREPROC_SCRIPT;
 	 *                                                                  22 - ZBX_PREPROC_PROMETHEUS_PATTERN;
 	 *                                                                  23 - ZBX_PREPROC_PROMETHEUS_TO_JSON;
-	 *                                                                  24 - ZBX_PREPROC_CSV_TO_JSON.
+	 *                                                                  24 - ZBX_PREPROC_CSV_TO_JSON;
+	 *                                                                  25 - ZBX_PREPROC_STR_REPLACE;
+	 *                                                                  26 - ZBX_PREPROC_VALIDATE_NOT_SUPPORTED;
 	 * @param string $item['preprocessing'][]['params']                Additional parameters used by preprocessing
 	 *                                                                 option. Multiple parameters are separated by LF
 	 *                                                                 (\n) character.
@@ -1351,6 +1342,7 @@ abstract class CItemGeneral extends CApiService {
 
 					case ZBX_PREPROC_REGSUB:
 					case ZBX_PREPROC_ERROR_FIELD_REGEX:
+					case ZBX_PREPROC_STR_REPLACE:
 						// Check if 'params' are not empty and if second parameter contains (after \n) is not empty.
 						if (is_array($preprocessing['params'])) {
 							self::exception(ZBX_API_ERROR_PARAMETERS, _('Incorrect arguments passed to function.'));
@@ -1370,7 +1362,9 @@ abstract class CItemGeneral extends CApiService {
 							));
 						}
 
-						if (!array_key_exists(1, $params) || $params[1] === '') {
+						if (($preprocessing['type'] == ZBX_PREPROC_REGSUB
+								|| $preprocessing['type'] == ZBX_PREPROC_ERROR_FIELD_REGEX)
+								&& (!array_key_exists(1, $params) || $params[1] === '')) {
 							self::exception(ZBX_API_ERROR_PARAMETERS, _s('Incorrect value for field "%1$s": %2$s.',
 								'params', _('second parameter is expected')
 							));
@@ -1596,6 +1590,27 @@ abstract class CItemGeneral extends CApiService {
 							}
 						}
 						break;
+
+					case ZBX_PREPROC_VALIDATE_NOT_SUPPORTED:
+						// Check if 'params' is empty, because it must be empty.
+						if (is_array($preprocessing['params'])) {
+							self::exception(ZBX_API_ERROR_PARAMETERS, _('Incorrect arguments passed to function.'));
+						}
+						elseif ($preprocessing['params'] !== '' && $preprocessing['params'] !== null
+								&& $preprocessing['params'] !== false) {
+							self::exception(ZBX_API_ERROR_PARAMETERS,
+								_s('Incorrect value for field "%1$s": %2$s.', 'params', _('should be empty'))
+							);
+						}
+
+						$preprocessing_types = array_column($item['preprocessing'], 'type');
+
+						if (count(array_keys($preprocessing_types, ZBX_PREPROC_VALIDATE_NOT_SUPPORTED)) > 1) {
+							self::exception(ZBX_API_ERROR_PARAMETERS,
+								_('Only one not supported value check is allowed.')
+							);
+						}
+						break;
 				}
 
 				switch ($preprocessing['type']) {
@@ -1605,6 +1620,7 @@ abstract class CItemGeneral extends CApiService {
 					case ZBX_PREPROC_THROTTLE_VALUE:
 					case ZBX_PREPROC_THROTTLE_TIMED_VALUE:
 					case ZBX_PREPROC_SCRIPT:
+					case ZBX_PREPROC_STR_REPLACE:
 						if (is_array($preprocessing['error_handler'])) {
 							self::exception(ZBX_API_ERROR_PARAMETERS, _('Incorrect arguments passed to function.'));
 						}
@@ -1758,6 +1774,123 @@ abstract class CItemGeneral extends CApiService {
 		}
 	}
 
+	/**
+	 * Create item parameters.
+	 *
+	 * @param array $items                             Array of items.
+	 * @param array $items[]['parameters']             Item parameters.
+	 * @param array $items[]['parameters'][]['name']   Parameter name.
+	 * @param array $items[]['parameters'][]['value']  Parameter value.
+	 * @param array $itemids                           Array of item IDs that were created before.
+	 */
+	protected function createItemParameters(array $items, array $itemids): void {
+		$item_parameters = [];
+
+		foreach ($items as $key => $item) {
+			$items[$key]['itemid'] = $itemids[$key];
+
+			if (!array_key_exists('parameters', $item) || !$item['parameters']) {
+				continue;
+			}
+
+			foreach ($item['parameters'] as $parameter) {
+				$item_parameters[] = [
+					'itemid' => $items[$key]['itemid'],
+					'name' => $parameter['name'],
+					'value' => $parameter['value']
+				];
+			}
+		}
+
+		if ($item_parameters) {
+			DB::insertBatch('item_parameter', $item_parameters);
+		}
+	}
+
+	/**
+	 * Update item parameters.
+	 *
+	 * @param array      $items                             Array of items.
+	 * @param int|string $items[]['itemid']                 Item ID.
+	 * @param int|string $items[]['type']                   Item type.
+	 * @param array      $items[]['parameters']             Item parameters.
+	 * @param array      $items[]['parameters'][]['name']   Parameter name.
+	 * @param array      $items[]['parameters'][]['value']  Parameter value.
+	 */
+	protected function updateItemParameters(array $items): void {
+		$db_item_parameters_by_itemid = [];
+
+		foreach ($items as $item) {
+			if ($item['type'] != ITEM_TYPE_SCRIPT || array_key_exists('parameters', $item)) {
+				$db_item_parameters_by_itemid[$item['itemid']] = [];
+			}
+		}
+
+		if (!$db_item_parameters_by_itemid) {
+			return;
+		}
+
+		$options = [
+			'output' => ['item_parameterid', 'itemid', 'name', 'value'],
+			'filter' => ['itemid' => array_keys($db_item_parameters_by_itemid)]
+		];
+		$result = DBselect(DB::makeSql('item_parameter', $options));
+
+		while ($row = DBfetch($result)) {
+			$db_item_parameters_by_itemid[$row['itemid']][$row['name']] = [
+				'item_parameterid' => $row['item_parameterid'],
+				'value' => $row['value']
+			];
+		}
+
+		$ins_item_parameters = [];
+		$upd_item_parameters = [];
+		$del_item_parameterids = [];
+
+		foreach ($db_item_parameters_by_itemid as $itemid => $db_item_parameters) {
+			$item = $items[$itemid];
+
+			if ($item['type'] == ITEM_TYPE_SCRIPT && array_key_exists('parameters', $item)) {
+				foreach ($item['parameters'] as $parameter) {
+					if (array_key_exists($parameter['name'], $db_item_parameters)) {
+						if ($db_item_parameters[$parameter['name']]['value'] !== $parameter['value']) {
+							$upd_item_parameters[] = [
+								'values' => ['value' => $parameter['value']],
+								'where' => [
+									'item_parameterid' => $db_item_parameters[$parameter['name']]['item_parameterid']
+								]
+							];
+						}
+						unset($db_item_parameters[$parameter['name']]);
+					}
+					else {
+						$ins_item_parameters[] = [
+							'itemid' => $itemid,
+							'name' => $parameter['name'],
+							'value' => $parameter['value']
+						];
+					}
+				}
+			}
+
+			$del_item_parameterids = array_merge($del_item_parameterids,
+				array_column($db_item_parameters, 'item_parameterid')
+			);
+		}
+
+		if ($del_item_parameterids) {
+			DB::delete('item_parameter', ['item_parameterid' => $del_item_parameterids]);
+		}
+
+		if ($upd_item_parameters) {
+			DB::update('item_parameter', $upd_item_parameters);
+		}
+
+		if ($ins_item_parameters) {
+			DB::insertBatch('item_parameter', $ins_item_parameters);
+		}
+	}
+
 	/**
 	 * Check if any item from list already exists.
 	 * If items have item ids it will check for existing item with different itemid.
@@ -1842,6 +1975,26 @@ abstract class CItemGeneral extends CApiService {
 			}
 		}
 
+		if (!$options['countOutput'] && $this->outputIsRequested('parameters', $options['output'])) {
+			$item_parameters = DBselect(
+				'SELECT ip.itemid,ip.name,ip.value'.
+				' FROM item_parameter ip'.
+				' WHERE '.dbConditionInt('ip.itemid', array_keys($result))
+			);
+
+			foreach ($result as &$item) {
+				$item['parameters'] = [];
+			}
+			unset($item);
+
+			while ($row = DBfetch($item_parameters)) {
+				$result[$row['itemid']]['parameters'][] = [
+					'name' =>  $row['name'],
+					'value' =>  $row['value']
+				];
+			}
+		}
+
 		return $result;
 	}
 
@@ -2041,7 +2194,7 @@ abstract class CItemGeneral extends CApiService {
 	}
 
 	/**
-	 * Validate depth and ammount of elements in the tree of the dependent items.
+	 * Validate depth and amount of elements in the tree of the dependent items.
 	 *
 	 * @param array  $dependent_items
 	 * @param string $dependent_items[<master_itemid>][]  List if the dependent item IDs ("false" for new items)
@@ -2195,7 +2348,8 @@ abstract class CItemGeneral extends CApiService {
 			'authtype' => [
 				'type' => API_INT32,
 				'in' => implode(',', [
-					HTTPTEST_AUTH_NONE, HTTPTEST_AUTH_BASIC, HTTPTEST_AUTH_NTLM, HTTPTEST_AUTH_KERBEROS
+					HTTPTEST_AUTH_NONE, HTTPTEST_AUTH_BASIC, HTTPTEST_AUTH_NTLM, HTTPTEST_AUTH_KERBEROS,
+					HTTPTEST_AUTH_DIGEST
 				])
 			]
 		];
@@ -2204,7 +2358,7 @@ abstract class CItemGeneral extends CApiService {
 
 		if (array_key_exists('authtype', $data)
 				&& ($data['authtype'] == HTTPTEST_AUTH_BASIC || $data['authtype'] == HTTPTEST_AUTH_NTLM
-					|| $data['authtype'] == HTTPTEST_AUTH_KERBEROS)) {
+					|| $data['authtype'] == HTTPTEST_AUTH_KERBEROS || $data['authtype'] == HTTPTEST_AUTH_DIGEST)) {
 			$rules += [
 				'username' => [ 'type' => API_STRING_UTF8, 'length' => DB::getFieldLength('items', 'username')],
 				'password' => [ 'type' => API_STRING_UTF8, 'length' => DB::getFieldLength('items', 'password')]
@@ -2245,8 +2399,6 @@ abstract class CItemGeneral extends CApiService {
 			self::exception(ZBX_API_ERROR_PARAMETERS, $error);
 		}
 
-		$json = new CJson();
-
 		if (array_key_exists('query_fields', $item)) {
 			if (!is_array($item['query_fields'])) {
 				self::exception(ZBX_API_ERROR_PARAMETERS,
@@ -2262,9 +2414,7 @@ abstract class CItemGeneral extends CApiService {
 				}
 			}
 
-			$json_string = $json->encode($item['query_fields']);
-
-			if (strlen($json_string) > DB::getFieldLength('items', 'query_fields')) {
+			if (strlen(json_encode($item['query_fields'])) > DB::getFieldLength('items', 'query_fields')) {
 				self::exception(ZBX_API_ERROR_PARAMETERS, _s('Invalid parameter "%1$s": %2$s.', 'query_fields',
 					_('cannot convert to JSON, result value too long')
 				));
@@ -2352,9 +2502,9 @@ abstract class CItemGeneral extends CApiService {
 					$shift = $shift + 1 - strlen($substr);
 				}
 
-				$json->decode($posts);
+				json_decode($posts);
 
-				if ($json->hasError()) {
+				if (json_last_error()) {
 					self::exception(ZBX_API_ERROR_PARAMETERS,
 						_s('Invalid parameter "%1$s": %2$s.', 'posts', _('JSON is expected'))
 					);
@@ -2362,4 +2512,75 @@ abstract class CItemGeneral extends CApiService {
 			}
 		}
 	}
+
+	/**
+	 * Remove NCLOB value type fields from resulting query SELECT part if DISTINCT will be used.
+	 *
+	 * @param string $table_name     Table name.
+	 * @param string $table_alias    Table alias value.
+	 * @param array  $options        Array of query options.
+	 * @param array  $sql_parts      Array of query parts already initialized from $options.
+	 *
+	 * @return array    The resulting SQL parts array.
+	 */
+	protected function applyQueryOutputOptions($table_name, $table_alias, array $options, array $sql_parts) {
+		if (!$options['countOutput'] && self::dbDistinct($sql_parts)) {
+			$schema = $this->getTableSchema();
+			$nclob_fields = [];
+
+			foreach ($schema['fields'] as $field_name => $field) {
+				if ($field['type'] == DB::FIELD_TYPE_NCLOB
+						&& $this->outputIsRequested($field_name, $options['output'])) {
+					$nclob_fields[] = $field_name;
+				}
+			}
+
+			if ($nclob_fields) {
+				$output = ($options['output'] === API_OUTPUT_EXTEND)
+					? array_keys($schema['fields'])
+					: $options['output'];
+
+				$options['output'] = array_diff($output, $nclob_fields);
+			}
+		}
+
+		return parent::applyQueryOutputOptions($table_name, $table_alias, $options, $sql_parts);
+	}
+
+	/**
+	 * Add NCLOB type fields if there was DISTINCT in query.
+	 *
+	 * @param array $options    Array of query options.
+	 * @param array $result     Query results.
+	 *
+	 * @return array    The result array with added NCLOB fields.
+	 */
+	protected function addNclobFieldValues(array $options, array $result): array {
+		$schema = $this->getTableSchema();
+		$nclob_fields = [];
+
+		foreach ($schema['fields'] as $field_name => $field) {
+			if ($field['type'] == DB::FIELD_TYPE_NCLOB && $this->outputIsRequested($field_name, $options['output'])) {
+				$nclob_fields[] = $field_name;
+			}
+		}
+
+		if (!$nclob_fields) {
+			return $result;
+		}
+
+		$pk = $schema['key'];
+		$options = [
+			'output' => $nclob_fields,
+			'filter' => [$pk => array_keys($result)]
+		];
+
+		$db_items = DBselect(DB::makeSql($this->tableName, $options));
+
+		while ($db_item = DBfetch($db_items)) {
+			$result[$db_item[$pk]] += $db_item;
+		}
+
+		return $result;
+	}
 }
