@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2019 Zabbix SIA
+** Copyright (C) 2001-2020 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -92,7 +92,7 @@ class CTemplate extends CHostGeneral {
 			'selectGraphs'				=> null,
 			'selectApplications'		=> null,
 			'selectMacros'				=> null,
-			'selectScreens'				=> null,
+			'selectDashboards'			=> null,
 			'selectHttpTests'			=> null,
 			'selectTags'				=> null,
 			'countOutput'				=> false,
@@ -263,7 +263,7 @@ class CTemplate extends CHostGeneral {
 
 		$sqlParts = $this->applyQueryOutputOptions($this->tableName(), $this->tableAlias(), $options, $sqlParts);
 		$sqlParts = $this->applyQuerySortOptions($this->tableName(), $this->tableAlias(), $options, $sqlParts);
-		$res = DBselect($this->createSelectQueryFromParts($sqlParts), $sqlParts['limit']);
+		$res = DBselect(self::createSelectQueryFromParts($sqlParts), $sqlParts['limit']);
 		while ($template = DBfetch($res)) {
 			if ($options['countOutput']) {
 				if ($options['groupCount']) {
@@ -316,7 +316,6 @@ class CTemplate extends CHostGeneral {
 			$templates[$key]['groups'] = zbx_toArray($template['groups']);
 		}
 
-		$ins_tags = [];
 		foreach ($templates as $template) {
 			// if visible name is not given or empty it should be set to host name
 			if ((!isset($template['name']) || zbx_empty(trim($template['name']))) && isset($template['host'])) {
@@ -334,12 +333,6 @@ class CTemplate extends CHostGeneral {
 
 			$templateIds[] = $templateId;
 
-			if (array_key_exists('tags', $template)) {
-				foreach ($template['tags'] as $tag) {
-					$ins_tags[] = ['hostid' => $templateId] + $tag;
-				}
-			}
-
 			foreach ($template['groups'] as $group) {
 				$hostGroupId = get_dbid('hosts_groups', 'hostgroupid');
 
@@ -353,6 +346,10 @@ class CTemplate extends CHostGeneral {
 				}
 			}
 
+			if (array_key_exists('tags', $template) && $template['tags']) {
+				$this->createTags([$templateId => zbx_toArray($template['tags'])]);
+			}
+
 			$template['templateid'] = $templateId;
 
 			$result = $this->massAdd([
@@ -367,10 +364,6 @@ class CTemplate extends CHostGeneral {
 			}
 		}
 
-		if ($ins_tags) {
-			DB::insert('host_tag', $ins_tags);
-		}
-
 		return ['templateids' => $templateIds];
 	}
 
@@ -384,7 +377,7 @@ class CTemplate extends CHostGeneral {
 	protected function validateCreate(array $templates) {
 		$groupIds = [];
 
-		foreach ($templates as $template) {
+		foreach ($templates as &$template) {
 			// check if hosts have at least 1 group
 			if (!isset($template['groups']) || !$template['groups']) {
 				self::exception(ZBX_API_ERROR_PARAMETERS,
@@ -395,9 +388,18 @@ class CTemplate extends CHostGeneral {
 			$template['groups'] = zbx_toArray($template['groups']);
 
 			foreach ($template['groups'] as $group) {
+				if (!is_array($group) || (is_array($group) && !array_key_exists('groupid', $group))) {
+					self::exception(ZBX_API_ERROR_PARAMETERS,
+						_s('Incorrect value for field "%1$s": %2$s.', 'groups',
+							_s('the parameter "%1$s" is missing', 'groupid')
+						)
+					);
+				}
+
 				$groupIds[$group['groupid']] = $group['groupid'];
 			}
 		}
+		unset($template);
 
 		$dbHostGroups = API::HostGroup()->get([
 			'output' => ['groupid'],
@@ -509,7 +511,7 @@ class CTemplate extends CHostGeneral {
 		$macros = [];
 		foreach ($templates as &$template) {
 			if (isset($template['macros'])) {
-				$macros[$template['templateid']] = $template['macros'];
+				$macros[$template['templateid']] = zbx_toArray($template['macros']);
 
 				unset($template['macros']);
 			}
@@ -538,7 +540,7 @@ class CTemplate extends CHostGeneral {
 			}
 		}
 
-		$this->updateTags($templates, 'templateid');
+		$this->updateTags(array_column($templates, 'tags', 'templateid'));
 
 		return ['templateids' => zbx_objectValues($templates, 'templateid')];
 	}
@@ -611,7 +613,7 @@ class CTemplate extends CHostGeneral {
 			'preservekeys' => true
 		]);
 		if ($del_rules) {
-			API::DiscoveryRule()->delete(array_keys($del_rules), true);
+			CDiscoveryRuleManager::delete(array_keys($del_rules));
 		}
 
 		// delete the items
@@ -716,6 +718,65 @@ class CTemplate extends CHostGeneral {
 			API::Application()->delete(array_keys($delApplications), true);
 		}
 
+		// Get host prototype operations from LLD overrides where this template is linked.
+		$lld_override_operationids = [];
+
+		$db_lld_override_operationids = DBselect(
+			'SELECT loo.lld_override_operationid'.
+			' FROM lld_override_operation loo'.
+			' WHERE EXISTS('.
+				'SELECT NULL'.
+				' FROM lld_override_optemplate lot'.
+				' WHERE lot.lld_override_operationid=loo.lld_override_operationid'.
+				' AND '.dbConditionId('lot.templateid', $templateids).
+			')'
+		);
+		while ($db_lld_override_operationid = DBfetch($db_lld_override_operationids)) {
+			$lld_override_operationids[] = $db_lld_override_operationid['lld_override_operationid'];
+		}
+
+		if ($lld_override_operationids) {
+			DB::delete('lld_override_optemplate', ['templateid' => $templateids]);
+
+			// Make sure there no other operations left to safely delete the operation.
+			$delete_lld_override_operationids = [];
+
+			$db_delete_lld_override_operationids = DBselect(
+				'SELECT loo.lld_override_operationid'.
+				' FROM lld_override_operation loo'.
+				' WHERE NOT EXISTS ('.
+						'SELECT NULL'.
+						' FROM lld_override_opstatus los'.
+						' WHERE los.lld_override_operationid=loo.lld_override_operationid'.
+					')'.
+					' AND NOT EXISTS ('.
+						'SELECT NULL'.
+						' FROM lld_override_opdiscover lod'.
+						' WHERE lod.lld_override_operationid=loo.lld_override_operationid'.
+					')'.
+					' AND NOT EXISTS ('.
+						'SELECT NULL'.
+						' FROM lld_override_opinventory loi'.
+						' WHERE loi.lld_override_operationid=loo.lld_override_operationid'.
+					')'.
+					' AND NOT EXISTS ('.
+						'SELECT NULL'.
+						' FROM lld_override_optemplate lot'.
+						' WHERE lot.lld_override_operationid=loo.lld_override_operationid'.
+					')'.
+					' AND '.dbConditionId('loo.lld_override_operationid', $lld_override_operationids)
+			);
+
+			while ($db_delete_lld_override_operationid = DBfetch($db_delete_lld_override_operationids)) {
+				$delete_lld_override_operationids[] = $db_delete_lld_override_operationid['lld_override_operationid'];
+			}
+
+			if ($delete_lld_override_operationids) {
+				DB::delete('lld_override_operation', ['lld_override_operationid' => $delete_lld_override_operationids]);
+			}
+		}
+
+		// Finally delete the template.
 		DB::delete('hosts', ['hostid' => $templateids]);
 
 		// TODO: remove info from API
@@ -950,7 +1011,7 @@ class CTemplate extends CHostGeneral {
 			DB::delete('hostmacro', ['hostid' => $templateIds]);
 
 			$this->massAdd([
-				'hosts' => $templates,
+				'templates' => $templates,
 				'macros' => $data['macros']
 			]);
 		}
@@ -1223,38 +1284,36 @@ class CTemplate extends CHostGeneral {
 			}
 		}
 
-		// Adding screens
-		if ($options['selectScreens'] !== null) {
-			if ($options['selectScreens'] != API_OUTPUT_COUNT) {
-				$screens = API::TemplateScreen()->get([
-					'output' => $this->outputExtend($options['selectScreens'], ['templateid']),
-					'templateids' => $templateids,
-					'nopermissions' => true
+		// Adding dashboards.
+		if ($options['selectDashboards'] !== null) {
+			if ($options['selectDashboards'] != API_OUTPUT_COUNT) {
+				$dashboards = API::TemplateDashboard()->get([
+					'output' => $this->outputExtend($options['selectDashboards'], ['templateid']),
+					'templateids' => $templateids
 				]);
 				if (!is_null($options['limitSelects'])) {
-					order_result($screens, 'name');
+					order_result($dashboards, 'name');
 				}
 
-				// preservekeys is not supported by templatescreen.get, so we're building a map using array keys
+				// Build relation map.
 				$relationMap = new CRelationMap();
-				foreach ($screens as $key => $screen) {
-					$relationMap->addRelation($screen['templateid'], $key);
+				foreach ($dashboards as $key => $dashboard) {
+					$relationMap->addRelation($dashboard['templateid'], $key);
 				}
 
-				$screens = $this->unsetExtraFields($screens, ['templateid'], $options['selectScreens']);
-				$result = $relationMap->mapMany($result, $screens, 'screens', $options['limitSelects']);
+				$dashboards = $this->unsetExtraFields($dashboards, ['templateid'], $options['selectDashboards']);
+				$result = $relationMap->mapMany($result, $dashboards, 'dashboards', $options['limitSelects']);
 			}
 			else {
-				$screens = API::TemplateScreen()->get([
+				$dashboards = API::TemplateDashboard()->get([
 					'templateids' => $templateids,
-					'nopermissions' => true,
 					'countOutput' => true,
 					'groupCount' => true
 				]);
-				$screens = zbx_toHash($screens, 'templateid');
+				$dashboards = zbx_toHash($dashboards, 'templateid');
 				foreach ($result as $templateid => $template) {
-					$result[$templateid]['screens'] = array_key_exists($templateid, $screens)
-						? $screens[$templateid]['rowscount']
+					$result[$templateid]['dashboards'] = array_key_exists($templateid, $dashboards)
+						? $dashboards[$templateid]['rowscount']
 						: '0';
 				}
 			}
