@@ -24,6 +24,15 @@
  */
 class CTrigger extends CTriggerGeneral {
 
+	public const ACCESS_RULES = [
+		'get' => ['min_user_type' => USER_TYPE_ZABBIX_USER],
+		'create' => ['min_user_type' => USER_TYPE_ZABBIX_ADMIN],
+		'update' => ['min_user_type' => USER_TYPE_ZABBIX_ADMIN],
+		'delete' => ['min_user_type' => USER_TYPE_ZABBIX_ADMIN],
+		'adddependencies' => ['min_user_type' => USER_TYPE_ZABBIX_ADMIN],
+		'deletedependencies' => ['min_user_type' => USER_TYPE_ZABBIX_ADMIN]
+	];
+
 	protected $tableName = 'triggers';
 	protected $tableAlias = 't';
 	protected $sortColumns = ['triggerid', 'description', 'status', 'priority', 'lastchange', 'hostname'];
@@ -419,10 +428,10 @@ class CTrigger extends CTriggerGeneral {
 
 		// only_true
 		if ($options['only_true'] !== null) {
-			$config = select_config();
 			$sqlParts['where']['ot'] = '((t.value='.TRIGGER_VALUE_TRUE.')'.
 				' OR ((t.value='.TRIGGER_VALUE_FALSE.')'.
-					' AND (t.lastchange>'.(time() - timeUnitToSeconds($config['ok_period'])).
+					' AND (t.lastchange>'.
+					(time() - timeUnitToSeconds(CSettingsHelper::get(CSettingsHelper::OK_PERIOD))).
 				'))'.
 			')';
 		}
@@ -449,7 +458,7 @@ class CTrigger extends CTriggerGeneral {
 
 		// return count or grouped counts via direct SQL count
 		if ($options['countOutput'] && !$this->requiresPostSqlFiltering($options)) {
-			$dbRes = DBselect($this->createSelectQueryFromParts($sqlParts), $options['limit']);
+			$dbRes = DBselect(self::createSelectQueryFromParts($sqlParts), $options['limit']);
 			while ($trigger = DBfetch($dbRes)) {
 				if ($options['groupCount']) {
 					$result[] = $trigger;
@@ -461,7 +470,7 @@ class CTrigger extends CTriggerGeneral {
 			return $result;
 		}
 
-		$result = zbx_toHash($this->customFetch($this->createSelectQueryFromParts($sqlParts), $options), 'triggerid');
+		$result = zbx_toHash($this->customFetch(self::createSelectQueryFromParts($sqlParts), $options), 'triggerid');
 
 		// return count for post SQL filtered result sets
 		if ($options['countOutput']) {
@@ -523,10 +532,7 @@ class CTrigger extends CTriggerGeneral {
 
 		$this->validateCreate($triggers);
 		$this->createReal($triggers);
-
-		foreach ($triggers as $trigger) {
-			$this->inherit($trigger);
-		}
+		$this->inherit($triggers);
 
 		// clear all dependencies on inherited triggers
 		$this->deleteDependencies($triggers);
@@ -559,7 +565,6 @@ class CTrigger extends CTriggerGeneral {
 	 */
 	public function update(array $triggers) {
 		$triggers = zbx_toArray($triggers);
-		$db_triggers = [];
 
 		$this->validateUpdate($triggers, $db_triggers);
 
@@ -584,10 +589,9 @@ class CTrigger extends CTriggerGeneral {
 		}
 
 		$this->updateReal($triggers, $db_triggers);
+		$this->inherit($triggers);
 
 		foreach ($triggers as $trigger) {
-			$this->inherit($trigger);
-
 			// replace dependencies
 			if (array_key_exists('dependencies', $trigger)) {
 				$this->deleteDependencies($trigger);
@@ -1039,7 +1043,7 @@ class CTrigger extends CTriggerGeneral {
 							foreach ($depTemplateIds as $depTemplateId) {
 								if (!isset($templates[$depTemplateId])) {
 									self::exception(ZBX_API_ERROR_PARAMETERS,
-										_s('Not all templates are linked to "%s".', reset($templates))
+										_s('Not all templates are linked to "%1$s".', reset($templates))
 									);
 								}
 							}
@@ -1294,7 +1298,7 @@ class CTrigger extends CTriggerGeneral {
 			unset($trigger);
 
 			$sql_select = ['triggerid'];
-			foreach (['parent_triggerid'] as $field) {
+			foreach (['parent_triggerid', 'ts_delete'] as $field) {
 				if ($this->outputIsRequested($field, $options['selectTriggerDiscovery'])) {
 					$sql_select[] = $field;
 				}
