@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2019 Zabbix SIA
+** Copyright (C) 2001-2020 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -24,9 +24,16 @@
  */
 class CTriggerPrototype extends CTriggerGeneral {
 
+	public const ACCESS_RULES = [
+		'get' => ['min_user_type' => USER_TYPE_ZABBIX_USER],
+		'create' => ['min_user_type' => USER_TYPE_ZABBIX_ADMIN],
+		'update' => ['min_user_type' => USER_TYPE_ZABBIX_ADMIN],
+		'delete' => ['min_user_type' => USER_TYPE_ZABBIX_ADMIN]
+	];
+
 	protected $tableName = 'triggers';
 	protected $tableAlias = 't';
-	protected $sortColumns = ['triggerid', 'description', 'status', 'priority'];
+	protected $sortColumns = ['triggerid', 'description', 'status', 'priority', 'discover'];
 
 	/**
 	 * Get trigger prototypes from database.
@@ -371,7 +378,7 @@ class CTriggerPrototype extends CTriggerGeneral {
 
 		$sqlParts = $this->applyQueryOutputOptions($this->tableName(), $this->tableAlias(), $options, $sqlParts);
 		$sqlParts = $this->applyQuerySortOptions($this->tableName(), $this->tableAlias(), $options, $sqlParts);
-		$dbRes = DBselect($this->createSelectQueryFromParts($sqlParts), $sqlParts['limit']);
+		$dbRes = DBselect(self::createSelectQueryFromParts($sqlParts), $sqlParts['limit']);
 		while ($triggerPrototype = DBfetch($dbRes)) {
 			if ($options['countOutput']) {
 				if ($options['groupCount']) {
@@ -431,10 +438,7 @@ class CTriggerPrototype extends CTriggerGeneral {
 
 		$this->validateCreate($trigger_prototypes);
 		$this->createReal($trigger_prototypes);
-
-		foreach ($trigger_prototypes as $trigger_prototype) {
-			$this->inherit($trigger_prototype);
-		}
+		$this->inherit($trigger_prototypes);
 
 		$addDependencies = false;
 
@@ -462,14 +466,10 @@ class CTriggerPrototype extends CTriggerGeneral {
 	 */
 	public function update(array $trigger_prototypes) {
 		$trigger_prototypes = zbx_toArray($trigger_prototypes);
-		$db_triggers = [];
 
 		$this->validateUpdate($trigger_prototypes, $db_triggers);
 		$this->updateReal($trigger_prototypes, $db_triggers);
-
-		foreach ($trigger_prototypes as $trigger_prototype) {
-			$this->inherit($trigger_prototype);
-		}
+		$this->inherit($trigger_prototypes);
 
 		$updateDependencies = false;
 
@@ -867,7 +867,7 @@ class CTriggerPrototype extends CTriggerGeneral {
 							foreach ($depTemplateIds as $depTemplateId) {
 								if (!isset($templates[$depTemplateId])) {
 									self::exception(ZBX_API_ERROR_PARAMETERS,
-										_s('Not all templates are linked to "%s".', reset($templates))
+										_s('Not all templates are linked to "%1$s".', reset($templates))
 									);
 								}
 							}
