@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2019 Zabbix SIA
+** Copyright (C) 2001-2020 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -80,35 +80,29 @@ class CConfigurationExport {
 		];
 
 		$this->dataFields = [
-			'item' => ['hostid', 'type', 'snmp_community', 'snmp_oid', 'name', 'key_', 'delay', 'history', 'trends',
-				'status', 'value_type', 'trapper_hosts', 'units', 'snmpv3_contextname', 'snmpv3_securityname',
-				'snmpv3_securitylevel', 'snmpv3_authprotocol', 'snmpv3_authpassphrase', 'snmpv3_privprotocol',
-				'snmpv3_privpassphrase', 'valuemapid', 'params', 'ipmi_sensor', 'authtype', 'username', 'password',
-				'publickey', 'privatekey', 'interfaceid', 'port', 'description', 'inventory_link', 'flags',
-				'logtimefmt', 'jmx_endpoint', 'master_itemid', 'timeout', 'url', 'query_fields', 'posts',
+			'item' => ['hostid', 'type', 'snmp_oid', 'name', 'key_', 'delay', 'history', 'trends', 'status',
+				'value_type', 'trapper_hosts', 'units', 'valuemapid', 'params', 'ipmi_sensor', 'authtype', 'username',
+				'password', 'publickey', 'privatekey', 'interfaceid', 'description', 'inventory_link', 'flags',
+				'logtimefmt', 'jmx_endpoint', 'master_itemid', 'timeout', 'url', 'query_fields', 'parameters', 'posts',
 				'status_codes', 'follow_redirects', 'post_type', 'http_proxy', 'headers', 'retrieve_mode',
 				'request_method', 'output_format', 'ssl_cert_file', 'ssl_key_file', 'ssl_key_password', 'verify_peer',
 				'verify_host', 'allow_traps'
 			],
-			'drule' => ['itemid', 'hostid', 'type', 'snmp_community', 'snmp_oid', 'name', 'key_', 'delay', 'history',
-				'trends', 'status', 'value_type', 'trapper_hosts', 'units', 'snmpv3_contextname', 'snmpv3_securityname',
-				'snmpv3_securitylevel', 'snmpv3_authprotocol', 'snmpv3_authpassphrase', 'snmpv3_privprotocol',
-				'snmpv3_privpassphrase', 'formula', 'valuemapid', 'params', 'ipmi_sensor', 'authtype', 'username',
-				'password', 'publickey', 'privatekey', 'interfaceid', 'port', 'description', 'inventory_link', 'flags',
-				'filter', 'lifetime', 'jmx_endpoint', 'master_itemid', 'timeout', 'url', 'query_fields', 'posts',
-				'status_codes', 'follow_redirects', 'post_type', 'http_proxy', 'headers', 'retrieve_mode',
+			'drule' => ['itemid', 'hostid', 'type', 'snmp_oid', 'name', 'key_', 'delay', 'history', 'trends', 'status',
+				'value_type', 'trapper_hosts', 'units', 'formula', 'valuemapid', 'params', 'ipmi_sensor', 'authtype',
+				'username', 'password', 'publickey', 'privatekey', 'interfaceid', 'description', 'inventory_link',
+				'flags', 'filter', 'lifetime', 'jmx_endpoint', 'master_itemid', 'timeout', 'url', 'query_fields',
+				'posts', 'status_codes', 'follow_redirects', 'post_type', 'http_proxy', 'headers', 'retrieve_mode',
 				'request_method', 'output_format', 'ssl_cert_file', 'ssl_key_file', 'ssl_key_password', 'verify_peer',
-				'verify_host', 'allow_traps'
+				'verify_host', 'allow_traps', 'parameters'
 			],
-			'item_prototype' => ['hostid', 'type', 'snmp_community', 'snmp_oid', 'name', 'key_', 'delay', 'history',
-				'trends', 'status', 'value_type', 'trapper_hosts', 'units', 'snmpv3_contextname', 'snmpv3_securityname',
-				'snmpv3_securitylevel', 'snmpv3_authprotocol', 'snmpv3_authpassphrase', 'snmpv3_privprotocol',
-				'snmpv3_privpassphrase', 'valuemapid', 'params', 'ipmi_sensor', 'authtype', 'username', 'password',
-				'publickey', 'privatekey', 'interfaceid', 'port', 'description', 'inventory_link', 'flags',
-				'logtimefmt', 'jmx_endpoint', 'master_itemid', 'timeout', 'url', 'query_fields', 'posts',
+			'item_prototype' => ['hostid', 'type', 'snmp_oid', 'name', 'key_', 'delay', 'history', 'trends', 'status',
+				'value_type', 'trapper_hosts', 'units', 'valuemapid', 'params', 'ipmi_sensor', 'authtype', 'username',
+				'password', 'publickey', 'privatekey', 'interfaceid', 'description', 'inventory_link', 'flags',
+				'logtimefmt', 'jmx_endpoint', 'master_itemid', 'timeout', 'url', 'query_fields', 'parameters', 'posts',
 				'status_codes', 'follow_redirects', 'post_type', 'http_proxy', 'headers', 'retrieve_mode',
 				'request_method', 'output_format', 'ssl_cert_file', 'ssl_key_file', 'ssl_key_password', 'verify_peer',
-				'verify_host', 'allow_traps'
+				'verify_host', 'allow_traps', 'discover'
 			]
 		];
 	}
@@ -142,7 +136,8 @@ class CConfigurationExport {
 		try {
 			$this->gatherData();
 
-			$schema = (new CImportValidatorFactory('xml'))
+			// Parameter in CImportValidatorFactory is irrelavant here, since export does not validate data.
+			$schema = (new CImportValidatorFactory(CExportWriterFactory::YAML))
 				->getObject(ZABBIX_EXPORT_VERSION)
 				->getSchema();
 
@@ -180,7 +175,7 @@ class CConfigurationExport {
 			}
 
 			if ($this->data['maps']) {
-				$this->builder->buildMaps($this->data['maps']);
+				$this->builder->buildMaps($schema['rules']['maps'], $this->data['maps']);
 			}
 
 			if ($this->data['mediaTypes']) {
@@ -285,6 +280,7 @@ class CConfigurationExport {
 			'selectGroups' => ['groupid', 'name'],
 			'selectParentTemplates' => API_OUTPUT_EXTEND,
 			'selectMacros' => API_OUTPUT_EXTEND,
+			'selectDashboards' => API_OUTPUT_EXTEND,
 			'selectTags' => ['tag', 'value'],
 			'templateids' => $templateids,
 			'preservekeys' => true
@@ -294,7 +290,7 @@ class CConfigurationExport {
 			// merge host groups with all groups
 			$this->data['groups'] += zbx_toHash($template['groups'], 'groupid');
 
-			$template['screens'] = [];
+			$template['dashboards'] = [];
 			$template['applications'] = [];
 			$template['discoveryRules'] = [];
 			$template['items'] = [];
@@ -303,7 +299,7 @@ class CConfigurationExport {
 		unset($template);
 
 		if ($templates) {
-			$templates = $this->gatherTemplateScreens($templates);
+			$templates = $this->gatherTemplateDashboards($templates);
 			$templates = $this->gatherApplications($templates);
 			$templates = $this->gatherItems($templates);
 			$templates = $this->gatherDiscoveryRules($templates);
@@ -325,7 +321,7 @@ class CConfigurationExport {
 				'name', 'description', 'tls_connect', 'tls_accept', 'tls_issuer', 'tls_subject', 'tls_psk_identity',
 				'tls_psk', 'inventory_mode'
 			],
-			'selectInterfaces' => ['interfaceid', 'main', 'type', 'useip', 'ip', 'dns', 'port', 'bulk'],
+			'selectInterfaces' => API_OUTPUT_EXTEND,
 			'selectInventory' => API_OUTPUT_EXTEND,
 			'selectMacros' => API_OUTPUT_EXTEND,
 			'selectGroups' => ['groupid', 'name'],
@@ -358,30 +354,88 @@ class CConfigurationExport {
 	}
 
 	/**
-	 * Get template screens from database.
+	 * Get template dashboards from database.
 	 *
 	 * @param array $templates
 	 *
 	 * @return array
 	 */
-	protected function gatherTemplateScreens(array $templates) {
-		$screens = API::TemplateScreen()->get([
+	protected function gatherTemplateDashboards(array $templates) {
+		$dashboards = API::TemplateDashboard()->get([
 			'output' => API_OUTPUT_EXTEND,
-			'selectScreenItems' => API_OUTPUT_EXTEND,
+			'selectWidgets' => API_OUTPUT_EXTEND,
 			'templateids' => array_keys($templates),
-			'noInheritance' => true,
 			'preservekeys' => true
 		]);
 
-		$this->prepareScreenExport($screens);
+		foreach ($dashboards as $dashboard) {
+			$dashboard['widgets'] = $this->prepareDashboardWidgets($dashboard['widgets']);
 
-		foreach ($screens as $screen) {
-			$templates[$screen['templateid']]['screens'][] = $screen;
+			$templates[$dashboard['templateid']]['dashboards'][] = $dashboard;
 		}
 
 		return $templates;
 	}
 
+	/**
+	 * Get dashboard widgets related objects from database.
+	 *
+	 * @param array $widgets
+	 *
+	 * @return array
+	 */
+	protected function prepareDashboardWidgets(array $widgets): array {
+		$hostids = [];
+		$itemids = [];
+		$graphids = [];
+
+		// Collect ids.
+		foreach ($widgets as $widget) {
+			foreach ($widget['fields'] as $field) {
+				switch ($field['type']) {
+					case ZBX_WIDGET_FIELD_TYPE_HOST:
+						$hostids[$field['value']] = true;
+						break;
+					case ZBX_WIDGET_FIELD_TYPE_ITEM:
+					case ZBX_WIDGET_FIELD_TYPE_ITEM_PROTOTYPE:
+						$itemids[$field['value']] = true;
+						break;
+					case ZBX_WIDGET_FIELD_TYPE_GRAPH:
+					case ZBX_WIDGET_FIELD_TYPE_GRAPH_PROTOTYPE:
+						$graphids[$field['value']] = true;
+						break;
+				}
+			}
+		}
+
+		$hosts = $this->getHostsReferences(array_keys($hostids));
+		$items = $this->getItemsReferences(array_keys($itemids));
+		$graphs = $this->getGraphsReferences(array_keys($graphids));
+
+		// Replace ids.
+		foreach ($widgets as &$widget) {
+			foreach ($widget['fields'] as &$field) {
+				switch ($field['type']) {
+					case ZBX_WIDGET_FIELD_TYPE_HOST:
+						$field['value'] = $hosts[$field['value']];
+						break;
+					case ZBX_WIDGET_FIELD_TYPE_ITEM:
+					case ZBX_WIDGET_FIELD_TYPE_ITEM_PROTOTYPE:
+						$field['value'] = $items[$field['value']];
+						break;
+					case ZBX_WIDGET_FIELD_TYPE_GRAPH:
+					case ZBX_WIDGET_FIELD_TYPE_GRAPH_PROTOTYPE:
+						$field['value'] = $graphs[$field['value']];
+						break;
+				}
+			}
+			unset($field);
+		}
+		unset($widget);
+
+		return $widgets;
+	}
+
 	/**
 	 * Get proxies from database.
 	 *
@@ -561,6 +615,7 @@ class CConfigurationExport {
 			'selectFilter' => ['evaltype', 'formula', 'conditions'],
 			'selectLLDMacroPaths' => ['lld_macro', 'path'],
 			'selectPreprocessing' => ['type', 'params', 'error_handler', 'error_handler_params'],
+			'selectOverrides' => ['name', 'step', 'stop', 'filter', 'operations'],
 			'hostids' => array_keys($hosts),
 			'inherited' => false,
 			'preservekeys' => true
@@ -603,20 +658,106 @@ class CConfigurationExport {
 	 * @return array
 	 */
 	protected function prepareDiscoveryRules(array $items) {
+		$templateids = [];
+
 		foreach ($items as &$item) {
 			$item['itemPrototypes'] = [];
 			$item['graphPrototypes'] = [];
 			$item['triggerPrototypes'] = [];
 			$item['hostPrototypes'] = [];
 
-			// unset unnecessary condition fields
+			// Unset unnecessary condition fields.
 			foreach ($item['filter']['conditions'] as &$condition) {
 				unset($condition['item_conditionid'], $condition['itemid']);
 			}
 			unset($condition);
+
+			// Unset unnecessary filter field and prepare the operations.
+			if ($item['overrides']) {
+				foreach ($item['overrides'] as &$override) {
+					if (array_key_exists('filter', $override)) {
+						if (!$override['filter']['conditions']) {
+							unset($override['filter']);
+						}
+						unset($override['filter']['eval_formula']);
+					}
+
+					foreach ($override['operations'] as &$operation) {
+						if (array_key_exists('opstatus', $operation)) {
+							$operation['status'] = $operation['opstatus']['status'];
+							unset($operation['opstatus']);
+						}
+						if (array_key_exists('opdiscover', $operation)) {
+							$operation['discover'] = $operation['opdiscover']['discover'];
+							unset($operation['opdiscover']);
+						}
+						if (array_key_exists('opperiod', $operation)) {
+							$operation['delay'] = $operation['opperiod']['delay'];
+							unset($operation['opperiod']);
+						}
+						if (array_key_exists('ophistory', $operation)) {
+							$operation['history'] = $operation['ophistory']['history'];
+							unset($operation['ophistory']);
+						}
+						if (array_key_exists('optrends', $operation)) {
+							$operation['trends'] = $operation['optrends']['trends'];
+							unset($operation['optrends']);
+						}
+						if (array_key_exists('opseverity', $operation)) {
+							$operation['severity'] = $operation['opseverity']['severity'];
+							unset($operation['opseverity']);
+						}
+						if (array_key_exists('optag', $operation)) {
+							$operation['tags'] = [];
+							foreach ($operation['optag'] as $tag) {
+								$operation['tags'][] = $tag;
+							}
+							unset($operation['optag']);
+						}
+						if (array_key_exists('optemplate', $operation)) {
+							foreach ($operation['optemplate'] as $template) {
+								$templateids[$template['templateid']] = true;
+							}
+						}
+						if (array_key_exists('opinventory', $operation)) {
+							$operation['inventory_mode'] = $operation['opinventory']['inventory_mode'];
+							unset($operation['opinventory']);
+						}
+					}
+					unset($operation);
+				}
+				unset($override);
+
+			}
 		}
 		unset($item);
 
+		if ($templateids) {
+			$templates = API::Template()->get([
+				'output' => ['name'],
+				'templateids' => array_keys($templateids),
+				'preservekeys' => true
+			]);
+			foreach ($items as &$item) {
+				if ($item['overrides']) {
+					foreach ($item['overrides'] as &$override) {
+						foreach ($override['operations'] as &$operation) {
+							if (array_key_exists('optemplate', $operation)) {
+								$operation['templates'] = [];
+								foreach ($operation['optemplate'] as $template) {
+									$operation['templates'][] = ['name' => $templates[$template['templateid']]['name']];
+								}
+								unset($operation['optemplate']);
+							}
+						}
+						unset($operation);
+					}
+					unset($override);
+				}
+			}
+			unset($item);
+		}
+
 		// gather item prototypes
 		$item_prototypes = API::ItemPrototype()->get([
 			'output' => $this->dataFields['item_prototype'],
@@ -721,8 +862,9 @@ class CConfigurationExport {
 
 		// gather trigger prototypes
 		$triggers = API::TriggerPrototype()->get([
-			'output' => ['expression', 'description', 'url', 'status', 'priority', 'comments', 'type', 'recovery_mode',
-				'recovery_expression', 'correlation_mode', 'correlation_tag', 'manual_close', 'opdata'
+			'output' => ['expression', 'description', 'url', 'status', 'priority', 'comments', 'type', 'flags',
+				'recovery_mode', 'recovery_expression', 'correlation_mode', 'correlation_tag', 'manual_close', 'opdata',
+				'discover', 'event_name'
 			],
 			'selectDiscoveryRule' => API_OUTPUT_EXTEND,
 			'selectDependencies' => ['expression', 'description', 'recovery_expression'],
@@ -747,6 +889,9 @@ class CConfigurationExport {
 			'selectGroupPrototypes' => API_OUTPUT_EXTEND,
 			'selectDiscoveryRule' => API_OUTPUT_EXTEND,
 			'selectTemplates' => API_OUTPUT_EXTEND,
+			'selectMacros' => API_OUTPUT_EXTEND,
+			'selectTags' => ['tag', 'value'],
+			'selectInterfaces' => ['main', 'type', 'useip', 'ip', 'dns', 'port', 'details'],
 			'inherited' => false,
 			'preservekeys' => true
 		]);
@@ -986,8 +1131,9 @@ class CConfigurationExport {
 		$hostIds = array_merge($hostIds, $templateIds);
 
 		$triggers = API::Trigger()->get([
-			'output' => ['expression', 'description', 'url', 'status', 'priority', 'comments', 'type', 'recovery_mode',
-				'recovery_expression', 'correlation_mode', 'correlation_tag', 'manual_close', 'opdata'
+			'output' => ['expression', 'description', 'url', 'status', 'priority', 'comments', 'type', 'flags',
+				'recovery_mode', 'recovery_expression', 'correlation_mode', 'correlation_tag', 'manual_close', 'opdata',
+				'event_name'
 			],
 			'selectDependencies' => ['expression', 'description', 'recovery_expression'],
 			'selectItems' => ['itemid', 'flags', 'type', 'templateid'],
@@ -1116,9 +1262,17 @@ class CConfigurationExport {
 				'script', 'timeout', 'process_tags', 'show_event_menu', 'event_menu_url', 'event_menu_name',
 				'description', 'parameters'
 			],
+			'selectMessageTemplates' => ['eventsource', 'recovery', 'subject', 'message'],
 			'mediatypeids' => $mediatypeids,
 			'preservekeys' => true
 		]);
+
+		foreach ($this->data['mediaTypes'] as &$media_type) {
+			if ($media_type['type'] == MEDIA_TYPE_WEBHOOK) {
+				CArrayHelper::sort($media_type['parameters'], ['name']);
+			}
+		}
+		unset($media_type);
 	}
 
 	/**
@@ -1351,10 +1505,14 @@ class CConfigurationExport {
 						break;
 				}
 
-				$selement['iconid_off'] = $selement['iconid_off'] > 0 ? $images[$selement['iconid_off']] : '';
-				$selement['iconid_on'] = $selement['iconid_on'] > 0 ? $images[$selement['iconid_on']] : '';
-				$selement['iconid_disabled'] = $selement['iconid_disabled'] > 0 ? $images[$selement['iconid_disabled']] : '';
-				$selement['iconid_maintenance'] = $selement['iconid_maintenance'] > 0 ? $images[$selement['iconid_maintenance']] : '';
+				$selement['iconid_off'] = ($selement['iconid_off'] > 0) ? $images[$selement['iconid_off']] : [];
+				$selement['iconid_on'] = ($selement['iconid_on'] > 0) ? $images[$selement['iconid_on']] : [];
+				$selement['iconid_disabled'] = ($selement['iconid_disabled'] > 0)
+					? $images[$selement['iconid_disabled']]
+					: [];
+				$selement['iconid_maintenance'] = ($selement['iconid_maintenance'] > 0)
+					? $images[$selement['iconid_maintenance']]
+					: [];
 			}
 			unset($selement);
 
