@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2019 Zabbix SIA
+** Copyright (C) 2001-2020 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -43,57 +43,254 @@ class CConfigurationExportBuilder {
 		return ['zabbix_export' => $this->data];
 	}
 
+	/**
+	 * Build data structure.
+	 *
+	 * @param array  $schema    Tag schema from validation class.
+	 * @param array  $data      Export data.
+	 * @param string $main_tag  Main element (for error reporting).
+	 *
+	 * @return array
+	 */
+	protected function build(array $schema, array $data, $main_tag = null) {
+		$n = 0;
+		$result = [];
+
+		$rules = $schema['rules'];
+
+		if ($schema['type'] & XML_INDEXED_ARRAY) {
+			$rules = $schema['rules'][$schema['prefix']]['rules'];
+		}
+
+		foreach ($data as $row) {
+			$store = [];
+			foreach ($rules as $tag => $val) {
+				$is_required = $val['type'] & XML_REQUIRED;
+				$is_string = $val['type'] & XML_STRING;
+				$is_array = $val['type'] & XML_ARRAY;
+				$is_indexed_array = $val['type'] & XML_INDEXED_ARRAY;
+				$has_data = array_key_exists($tag, $row);
+				$default_value = array_key_exists('default', $val) ? $val['default'] : null;
+
+				if (!$default_value && !$has_data) {
+					if ($is_required) {
+						throw new Exception(_s('Invalid tag "%1$s": %2$s.', $main_tag,
+							_s('the tag "%1$s" is missing', $tag)
+						));
+					}
+					continue;
+				}
+
+				$value = $has_data ? $row[$tag] : $default_value;
+
+				if (!$is_required && $has_data && $default_value == $value) {
+					continue;
+				}
+
+				if (array_key_exists('export', $val)) {
+					$store[$tag] = call_user_func($val['export'], $row);
+					continue;
+				}
+
+				if (($is_indexed_array || $is_array) && $has_data) {
+					$temp_store = $this->build($val, $is_array ? [$value] : $value, $tag);
+					if ($is_required || $temp_store) {
+						$store[$tag] = $temp_store;
+					}
+					continue;
+				}
+
+				if ($is_string && $value !== null) {
+					$value = str_replace("\r\n", "\n", $value);
+				}
+
+				if (array_key_exists('in', $val)) {
+					if (!array_key_exists($value, $val['in'])) {
+						throw new Exception(_s('Invalid tag "%1$s": %2$s.', $tag,
+							_s('unexpected constant value "%1$s"', $value)
+						));
+					}
+
+					$store[$tag] = $val['in'][$value];
+				}
+				else {
+					$store[$tag] = $value;
+				}
+			}
+
+			if ($schema['type'] & XML_INDEXED_ARRAY) {
+				$result[$n++] = $store;
+			}
+			else {
+				$result = $store;
+			}
+		}
+
+		return $result;
+	}
+
 	/**
 	 * Format groups.
 	 *
-	 * @param array $groups
+	 * @param array $schema  Tag schema from validation class.
+	 * @param array $groups  Export data.
 	 */
-	public function buildGroups(array $groups) {
-		$this->data['groups'] = $this->formatGroups($groups);
+	public function buildGroups(array $schema, array $groups) {
+		$groups = $this->formatGroups($groups);
+
+		$this->data['groups'] = $this->build($schema, $groups, 'groups');
+	}
+
+	/**
+	 * Format templates.
+	 *
+	 * @param array $schema           Tag schema from validation class.
+	 * @param array $templates        Export data.
+	 * @param array $simple_triggers  Simple triggers.
+	 */
+	public function buildTemplates(array $schema, array $templates, array $simple_triggers) {
+		$templates = $this->formatTemplates($templates, $simple_triggers);
+
+		$this->data['templates'] = $this->build($schema, $templates, 'templates');
+	}
+
+	/**
+	 * Format hosts.
+	 *
+	 * @param array $schema           Tag schema from validation class.
+	 * @param array $hosts            Export data.
+	 * @param array $simple_triggers  Simple triggers.
+	 */
+	public function buildHosts(array $schema, array $hosts, array $simple_triggers) {
+		$hosts = $this->formatHosts($hosts, $simple_triggers);
+
+		$this->data['hosts'] = $this->build($schema, $hosts, 'hosts');
+	}
+
+	/**
+	 * Format triggers.
+	 *
+	 * @param array $schema    Tag schema from validation class.
+	 * @param array $triggers  Export data.
+	 */
+	public function buildTriggers(array $schema, array $triggers) {
+		$triggers = $this->formatTriggers($triggers);
+
+		$this->data['triggers'] = $this->build($schema, $triggers, 'triggers');
+	}
+
+	/**
+	 * Format graphs.
+	 *
+	 * @param array $schema  Tag schema from validation class.
+	 * @param array $graphs  Export data.
+	 */
+	public function buildGraphs(array $schema, array $graphs) {
+		$graphs = $this->formatGraphs($graphs);
+
+		$this->data['graphs'] = $this->build($schema, $graphs, 'graphs');
+	}
+
+	/**
+	 * Format screens.
+	 *
+	 * @param array $screens
+	 */
+	public function buildScreens(array $screens) {
+		$this->data['screens'] = $this->formatScreens($screens);
+	}
+
+	/**
+	 * Format media types.
+	 *
+	 * @param array $schema       Tag schema from validation class.
+	 * @param array $media_types  Export data.
+	 */
+	public function buildMediaTypes(array $schema, array $media_types) {
+		$media_types = $this->formatMediaTypes($media_types);
+
+		$this->data['media_types'] = $this->build($schema, $media_types, 'media_types');
+	}
+
+	/**
+	 * Format valuemaps.
+	 *
+	 * @param array $schema     Tag schema from validation class.
+	 * @param array $valuemaps  Export data.
+	 */
+	public function buildValueMaps(array $schema, array $valuemaps) {
+		$valuemaps = $this->formatValueMaps($valuemaps);
+
+		$this->data['value_maps'] = $this->build($schema, $valuemaps, 'value_maps');
+	}
+
+	/**
+	 * Separate simple triggers.
+	 *
+	 * @param array $triggers
+	 *
+	 * @return array
+	 */
+	public function extractSimpleTriggers(array &$triggers) {
+		$simple_triggers = [];
+
+		foreach ($triggers as $triggerid => $trigger) {
+			if (count($trigger['items']) == 1 && $trigger['items'][0]['type'] != ITEM_TYPE_HTTPTEST
+					&& $trigger['items'][0]['templateid'] == 0) {
+				$simple_triggers[] = $trigger;
+				unset($triggers[$triggerid]);
+			}
+		}
+
+		return $simple_triggers;
 	}
 
 	/**
 	 * Format templates.
 	 *
 	 * @param array $templates
+	 * @param array $simple_triggers
 	 */
-	public function buildTemplates(array $templates) {
-		$this->data['templates'] = [];
+	protected function formatTemplates(array $templates, array $simple_triggers = null) {
+		$result = [];
 
 		CArrayHelper::sort($templates, ['host']);
 
 		foreach ($templates as $template) {
-			$this->data['templates'][] = [
+			$result[] = [
 				'template' => $template['host'],
 				'name' => $template['name'],
 				'description' => $template['description'],
 				'groups' => $this->formatGroups($template['groups']),
 				'applications' => $this->formatApplications($template['applications']),
-				'items' => $this->formatItems($template['items']),
+				'items' => $this->formatItems($template['items'], $simple_triggers),
 				'discovery_rules' => $this->formatDiscoveryRules($template['discoveryRules']),
 				'httptests' => $this->formatHttpTests($template['httptests']),
 				'macros' => $this->formatMacros($template['macros']),
 				'templates' => $this->formatTemplateLinkage($template['parentTemplates']),
-				'screens' => $this->formatScreens($template['screens']),
+				'dashboards' => $this->formatDashboards($template['dashboards']),
 				'tags' => $this->formatTags($template['tags'])
 			];
 		}
+
+		return $result;
 	}
 
 	/**
 	 * Format hosts.
 	 *
-	 * @param $hosts
+	 * @param array $hosts
+	 * @param array $simple_triggers
 	 */
-	public function buildHosts(array $hosts) {
-		$this->data['hosts'] = [];
+	protected function formatHosts(array $hosts, array $simple_triggers = null) {
+		$result = [];
 
 		CArrayHelper::sort($hosts, ['host']);
 
 		foreach ($hosts as $host) {
 			$host = $this->createInterfaceReferences($host);
 
-			$this->data['hosts'][] = [
+			$result[] = [
 				'host' => $host['host'],
 				'name' => $host['name'],
 				'description' => $host['description'],
@@ -113,41 +310,17 @@ class CConfigurationExportBuilder {
 				'groups' => $this->formatGroups($host['groups']),
 				'interfaces' => $this->formatHostInterfaces($host['interfaces']),
 				'applications' => $this->formatApplications($host['applications']),
-				'items' => $this->formatItems($host['items']),
+				'items' => $this->formatItems($host['items'], $simple_triggers),
 				'discovery_rules' => $this->formatDiscoveryRules($host['discoveryRules']),
 				'httptests' => $this->formatHttpTests($host['httptests']),
 				'macros' => $this->formatMacros($host['macros']),
+				'inventory_mode' => $host['inventory_mode'],
 				'inventory' => $this->formatHostInventory($host['inventory']),
 				'tags' => $this->formatTags($host['tags'])
 			];
 		}
-	}
 
-	/**
-	 * Format graphs.
-	 *
-	 * @param array $graphs
-	 */
-	public function buildGraphs(array $graphs) {
-		$this->data['graphs'] = $this->formatGraphs($graphs);
-	}
-
-	/**
-	 * Format triggers.
-	 *
-	 * @param array $triggers
-	 */
-	public function buildTriggers(array $triggers) {
-		$this->data['triggers'] = $this->formatTriggers($triggers);
-	}
-
-	/**
-	 * Format screens.
-	 *
-	 * @param array $screens
-	 */
-	public function buildScreens(array $screens) {
-		$this->data['screens'] = $this->formatScreens($screens);
+		return $result;
 	}
 
 	/**
@@ -170,9 +343,10 @@ class CConfigurationExportBuilder {
 	/**
 	 * Format maps.
 	 *
-	 * @param array $maps
+	 * @param array $schema  Tag schema from validation class.
+	 * @param array $maps    Export data.
 	 */
-	public function buildMaps(array $maps) {
+	public function buildMaps(array $schema, array $maps) {
 		$this->data['maps'] = [];
 
 		CArrayHelper::sort($maps, ['name']);
@@ -215,6 +389,8 @@ class CConfigurationExportBuilder {
 				'links' => $this->formatMapLinks($map['links'], $tmpSelements)
 			];
 		}
+
+		$this->data['maps'] = $this->build($schema, $this->data['maps'], 'maps');
 	}
 
 	/**
@@ -239,22 +415,83 @@ class CConfigurationExportBuilder {
 		return $result;
 	}
 
+	/**
+	 * Format media types.
+	 *
+	 * @param array $media_types
+	 */
+	protected function formatMediaTypes(array $media_types) {
+		$result = [];
+
+		CArrayHelper::sort($media_types, ['name']);
+
+		foreach ($media_types as $media_type) {
+			$message_templates = [];
+
+			foreach ($media_type['message_templates'] as $message_template) {
+				$message_templates[] = [
+					'event_source' => $message_template['eventsource'],
+					'operation_mode' => $message_template['recovery'],
+					'subject' => $message_template['subject'],
+					'message' => $message_template['message']
+				];
+			}
+
+			$result[] = [
+				'name' => $media_type['name'],
+				'type' => $media_type['type'],
+				'smtp_server' => $media_type['smtp_server'],
+				'smtp_port' => $media_type['smtp_port'],
+				'smtp_helo' => $media_type['smtp_helo'],
+				'smtp_email' => $media_type['smtp_email'],
+				'smtp_security' => $media_type['smtp_security'],
+				'smtp_verify_host' => $media_type['smtp_verify_host'],
+				'smtp_verify_peer' => $media_type['smtp_verify_peer'],
+				'smtp_authentication' => $media_type['smtp_authentication'],
+				'username' => $media_type['username'],
+				'password' => $media_type['passwd'],
+				'content_type' => $media_type['content_type'],
+				'script_name' => $media_type['exec_path'],
+				'parameters' => ($media_type['type'] == MEDIA_TYPE_WEBHOOK)
+					? $media_type['parameters']
+					: $media_type['exec_params'],
+				'gsm_modem' => $media_type['gsm_modem'],
+				'status' => $media_type['status'],
+				'max_sessions' => $media_type['maxsessions'],
+				'attempts' => $media_type['maxattempts'],
+				'attempt_interval' => $media_type['attempt_interval'],
+				'script' => $media_type['script'],
+				'timeout' => $media_type['timeout'],
+				'process_tags' => $media_type['process_tags'],
+				'show_event_menu' => $media_type['show_event_menu'],
+				'event_menu_url' => $media_type['event_menu_url'],
+				'event_menu_name' => $media_type['event_menu_name'],
+				'description' => $media_type['description'],
+				'message_templates' => $message_templates
+			];
+		}
+
+		return $result;
+	}
+
 	/**
 	 * Format value maps.
 	 *
 	 * @param array $valuemaps
 	 */
-	public function buildValueMaps(array $valuemaps) {
-		$this->data['value_maps'] = [];
+	protected function formatValueMaps(array $valuemaps) {
+		$result = [];
 
 		CArrayHelper::sort($valuemaps, ['name']);
 
 		foreach ($valuemaps as $valuemap) {
-			$this->data['value_maps'][] = [
+			$result[] = [
 				'name' => $valuemap['name'],
 				'mappings' => $this->formatMappings($valuemap['mappings'])
 			];
 		}
+
+		return $result;
 	}
 
 	/**
@@ -316,23 +553,24 @@ class CConfigurationExportBuilder {
 
 		CArrayHelper::sort($discoveryRules, ['key_']);
 
+		$simple_trigger_prototypes = [];
+
 		foreach ($discoveryRules as $discoveryRule) {
+			foreach ($discoveryRule['triggerPrototypes'] as $i => $trigger_prototype) {
+				if (count($trigger_prototype['items']) == 1) {
+					$simple_trigger_prototypes[] = $trigger_prototype;
+					unset($discoveryRule['triggerPrototypes'][$i]);
+				}
+			}
+
 			$data = [
 				'name' => $discoveryRule['name'],
 				'type' => $discoveryRule['type'],
-				'snmp_community' => $discoveryRule['snmp_community'],
 				'snmp_oid' => $discoveryRule['snmp_oid'],
 				'key' => $discoveryRule['key_'],
 				'delay' => $discoveryRule['delay'],
 				'status' => $discoveryRule['status'],
 				'allowed_hosts' => $discoveryRule['trapper_hosts'],
-				'snmpv3_contextname' => $discoveryRule['snmpv3_contextname'],
-				'snmpv3_securityname' => $discoveryRule['snmpv3_securityname'],
-				'snmpv3_securitylevel' => $discoveryRule['snmpv3_securitylevel'],
-				'snmpv3_authprotocol' => $discoveryRule['snmpv3_authprotocol'],
-				'snmpv3_authpassphrase' => $discoveryRule['snmpv3_authpassphrase'],
-				'snmpv3_privprotocol' => $discoveryRule['snmpv3_privprotocol'],
-				'snmpv3_privpassphrase' => $discoveryRule['snmpv3_privpassphrase'],
 				'params' => $discoveryRule['params'],
 				'ipmi_sensor' => $discoveryRule['ipmi_sensor'],
 				'authtype' => $discoveryRule['authtype'],
@@ -340,11 +578,10 @@ class CConfigurationExportBuilder {
 				'password' => $discoveryRule['password'],
 				'publickey' => $discoveryRule['publickey'],
 				'privatekey' => $discoveryRule['privatekey'],
-				'port' => $discoveryRule['port'],
 				'filter' => $discoveryRule['filter'],
 				'lifetime' => $discoveryRule['lifetime'],
 				'description' => $discoveryRule['description'],
-				'item_prototypes' => $this->formatItems($discoveryRule['itemPrototypes']),
+				'item_prototypes' => $this->formatItems($discoveryRule['itemPrototypes'], $simple_trigger_prototypes),
 				'trigger_prototypes' => $this->formatTriggers($discoveryRule['triggerPrototypes']),
 				'graph_prototypes' => $this->formatGraphs($discoveryRule['graphPrototypes']),
 				'host_prototypes' => $this->formatHostPrototypes($discoveryRule['hostPrototypes']),
@@ -352,6 +589,7 @@ class CConfigurationExportBuilder {
 				'timeout' => $discoveryRule['timeout'],
 				'url' => $discoveryRule['url'],
 				'query_fields' => $discoveryRule['query_fields'],
+				'parameters' => $discoveryRule['parameters'],
 				'posts' => $discoveryRule['posts'],
 				'status_codes' => $discoveryRule['status_codes'],
 				'follow_redirects' => $discoveryRule['follow_redirects'],
@@ -367,7 +605,8 @@ class CConfigurationExportBuilder {
 				'verify_peer' => $discoveryRule['verify_peer'],
 				'verify_host' => $discoveryRule['verify_host'],
 				'lld_macro_paths' => $discoveryRule['lld_macro_paths'],
-				'preprocessing' => $discoveryRule['preprocessing']
+				'preprocessing' => self::formatPreprocessingSteps($discoveryRule['preprocessing']),
+				'overrides' => $discoveryRule['overrides']
 			];
 
 			if (isset($discoveryRule['interface_ref'])) {
@@ -407,6 +646,27 @@ class CConfigurationExportBuilder {
 		return $result;
 	}
 
+	/**
+	 * Format preprocessing steps.
+	 *
+	 * @param array $preprocessing_steps
+	 *
+	 * @static
+	 *
+	 * @return array
+	 */
+	private static function formatPreprocessingSteps(array $preprocessing_steps) {
+		foreach ($preprocessing_steps as &$preprocessing_step) {
+			$preprocessing_step['parameters'] = ($preprocessing_step['type'] == ZBX_PREPROC_SCRIPT)
+				? [$preprocessing_step['params']]
+				: explode("\n", $preprocessing_step['params']);
+			unset($preprocessing_step['params']);
+		}
+		unset($preprocessing_step);
+
+		return $preprocessing_steps;
+	}
+
 	/**
 	 * Format web scenarios.
 	 *
@@ -502,7 +762,7 @@ class CConfigurationExportBuilder {
 		CArrayHelper::sort($graphs, ['name']);
 
 		foreach ($graphs as $graph) {
-			$result[] = [
+			$data = [
 				'name' => $graph['name'],
 				'width' => $graph['width'],
 				'height' => $graph['height'],
@@ -521,6 +781,12 @@ class CConfigurationExportBuilder {
 				'ymax_item_1' => $graph['ymax_itemid'],
 				'graph_items' => $this->formatGraphItems($graph['gitems'])
 			];
+
+			if ($graph['flags'] == ZBX_FLAG_DISCOVERY_PROTOTYPE) {
+				$data['discover'] = $graph['discover'];
+			}
+
+			$result[] = $data;
 		}
 
 		return $result;
@@ -543,9 +809,15 @@ class CConfigurationExportBuilder {
 				'host' => $hostPrototype['host'],
 				'name' => $hostPrototype['name'],
 				'status' => $hostPrototype['status'],
+				'discover' => $hostPrototype['discover'],
 				'group_links' => $this->formatGroupLinks($hostPrototype['groupLinks']),
 				'group_prototypes' => $this->formatGroupPrototypes($hostPrototype['groupPrototypes']),
-				'templates' => $this->formatTemplateLinkage($hostPrototype['templates'])
+				'macros' => $this->formatMacros($hostPrototype['macros']),
+				'tags' => $this->formatTags($hostPrototype['tags']),
+				'templates' => $this->formatTemplateLinkage($hostPrototype['templates']),
+				'inventory_mode' => $hostPrototype['inventory_mode'],
+				'custom_interfaces' => $hostPrototype['custom_interfaces'],
+				'interfaces' => $this->formatHostPrototypeInterfaces($hostPrototype['interfaces'])
 			];
 		}
 
@@ -628,11 +900,13 @@ class CConfigurationExportBuilder {
 		CArrayHelper::sort($triggers, ['description', 'expression', 'recovery_expression']);
 
 		foreach ($triggers as $trigger) {
-			$result[] = [
+			$data = [
 				'expression' => $trigger['expression'],
 				'recovery_mode' => $trigger['recovery_mode'],
 				'recovery_expression' => $trigger['recovery_expression'],
 				'name' => $trigger['description'],
+				'event_name' => $trigger['event_name'],
+				'opdata' => $trigger['opdata'],
 				'correlation_mode' => $trigger['correlation_mode'],
 				'correlation_tag' => $trigger['correlation_tag'],
 				'url' => $trigger['url'],
@@ -644,6 +918,12 @@ class CConfigurationExportBuilder {
 				'dependencies' => $this->formatDependencies($trigger['dependencies']),
 				'tags' => $this->formatTags($trigger['tags'])
 			];
+
+			if ($trigger['flags'] == ZBX_FLAG_DISCOVERY_PROTOTYPE) {
+				$data['discover'] = $trigger['discover'];
+			}
+
+			$result[] = $data;
 		}
 
 		return $result;
@@ -669,7 +949,7 @@ class CConfigurationExportBuilder {
 				'ip' => $interface['ip'],
 				'dns' => $interface['dns'],
 				'port' => $interface['port'],
-				'bulk' => $interface['bulk'],
+				'details' => $interface['details'],
 				'interface_ref' => $interface['interface_ref']
 			];
 		}
@@ -677,6 +957,36 @@ class CConfigurationExportBuilder {
 		return $result;
 	}
 
+	/**
+	 * Format host prototype interfaces.
+	 *
+	 * @param array $interfaces
+	 *
+	 * @return array
+	 */
+	protected function formatHostPrototypeInterfaces(array $interfaces): array {
+		$result = [];
+
+		CArrayHelper::sort($interfaces, ['type', 'ip', 'dns', 'port']);
+
+		foreach ($interfaces as $num => $interface) {
+			$result[$num] = [
+				'default' => $interface['main'],
+				'type' => $interface['type'],
+				'useip' => $interface['useip'],
+				'ip' => $interface['ip'],
+				'dns' => $interface['dns'],
+				'port' => $interface['port']
+			];
+
+			if ($interface['type'] == INTERFACE_TYPE_SNMP) {
+				$result[$num]['details'] = $interface['details'];
+			}
+		}
+
+		return $result;
+	}
+
 	/**
 	 * Format groups.
 	 *
@@ -702,11 +1012,13 @@ class CConfigurationExportBuilder {
 	 * Format items.
 	 *
 	 * @param array $items
+	 * @param array $simple_triggers
 	 *
 	 * @return array
 	 */
-	protected function formatItems(array $items) {
+	protected function formatItems(array $items, array $simple_triggers) {
 		$result = [];
+		$expression_data = $simple_triggers ? new CTriggerExpression() : null;
 
 		CArrayHelper::sort($items, ['key_']);
 
@@ -714,7 +1026,6 @@ class CConfigurationExportBuilder {
 			$data = [
 				'name' => $item['name'],
 				'type' => $item['type'],
-				'snmp_community' => $item['snmp_community'],
 				'snmp_oid' => $item['snmp_oid'],
 				'key' => $item['key_'],
 				'delay' => $item['delay'],
@@ -724,13 +1035,6 @@ class CConfigurationExportBuilder {
 				'value_type' => $item['value_type'],
 				'allowed_hosts' => $item['trapper_hosts'],
 				'units' => $item['units'],
-				'snmpv3_contextname' => $item['snmpv3_contextname'],
-				'snmpv3_securityname' => $item['snmpv3_securityname'],
-				'snmpv3_securitylevel' => $item['snmpv3_securitylevel'],
-				'snmpv3_authprotocol' => $item['snmpv3_authprotocol'],
-				'snmpv3_authpassphrase' => $item['snmpv3_authpassphrase'],
-				'snmpv3_privprotocol' => $item['snmpv3_privprotocol'],
-				'snmpv3_privpassphrase' => $item['snmpv3_privpassphrase'],
 				'params' => $item['params'],
 				'ipmi_sensor' => $item['ipmi_sensor'],
 				'authtype' => $item['authtype'],
@@ -738,17 +1042,17 @@ class CConfigurationExportBuilder {
 				'password' => $item['password'],
 				'publickey' => $item['publickey'],
 				'privatekey' => $item['privatekey'],
-				'port' => $item['port'],
 				'description' => $item['description'],
 				'inventory_link' => $item['inventory_link'],
 				'applications' => $this->formatApplications($item['applications']),
 				'valuemap' => $item['valuemap'],
 				'logtimefmt' => $item['logtimefmt'],
-				'preprocessing' => $item['preprocessing'],
+				'preprocessing' => self::formatPreprocessingSteps($item['preprocessing']),
 				'jmx_endpoint' => $item['jmx_endpoint'],
 				'timeout' => $item['timeout'],
 				'url' => $item['url'],
 				'query_fields' => $item['query_fields'],
+				'parameters' => $item['parameters'],
 				'posts' => $item['posts'],
 				'status_codes' => $item['status_codes'],
 				'follow_redirects' => $item['follow_redirects'],
@@ -770,6 +1074,7 @@ class CConfigurationExportBuilder {
 
 			if ($item['flags'] == ZBX_FLAG_DISCOVERY_PROTOTYPE) {
 				$data['application_prototypes'] = $this->formatApplications($item['applicationPrototypes']);
+				$data['discover'] = $item['discover'];
 			}
 
 			$data['master_item'] = $master_item;
@@ -801,6 +1106,44 @@ class CConfigurationExportBuilder {
 				$data['headers'] = $headers;
 			}
 
+			if ($simple_triggers) {
+				$triggers = [];
+				$prefix_length = strlen($item['host'].':'.$item['key_'].'.');
+
+				foreach ($simple_triggers as $simple_trigger) {
+					if (bccomp($item['itemid'], $simple_trigger['items'][0]['itemid']) == 0) {
+						if ($expression_data->parse($simple_trigger['expression'])) {
+							foreach (array_reverse($expression_data->expressions) as $expression) {
+								if ($expression['host'] === $item['host'] && $expression['item'] === $item['key_']) {
+									$simple_trigger['expression'] = substr_replace($simple_trigger['expression'], '',
+										$expression['pos'] + 1, $prefix_length
+									);
+								}
+							}
+						}
+
+						if ($simple_trigger['recovery_mode'] == ZBX_RECOVERY_MODE_RECOVERY_EXPRESSION
+								&& $expression_data->parse($simple_trigger['recovery_expression'])) {
+							foreach (array_reverse($expression_data->expressions) as $expression) {
+								if ($expression['host'] === $item['host'] && $expression['item'] === $item['key_']) {
+									$simple_trigger['recovery_expression'] = substr_replace(
+										$simple_trigger['recovery_expression'], '', $expression['pos'] + 1,
+										$prefix_length
+									);
+								}
+							}
+						}
+
+						$triggers[] = $simple_trigger;
+					}
+				}
+
+				if ($triggers) {
+					$key = array_key_exists('discoveryRule', $item) ? 'trigger_prototypes' : 'triggers';
+					$data[$key] = $this->formatTriggers($triggers);
+				}
+			}
+
 			$result[] = $data;
 		}
 
@@ -843,7 +1186,9 @@ class CConfigurationExportBuilder {
 		foreach ($macros as $macro) {
 			$result[] = [
 				'macro' => $macro['macro'],
-				'value' => $macro['value']
+				'type' => $macro['type'],
+				'value' => array_key_exists('value', $macro) ? $macro['value'] : '',
+				'description' => $macro['description']
 			];
 		}
 
@@ -956,6 +1301,79 @@ class CConfigurationExportBuilder {
 		return $result;
 	}
 
+	/**
+	 * Format dashboards.
+	 *
+	 * @param array $dashboards
+	 *
+	 * @return array
+	 */
+	protected function formatDashboards(array $dashboards) {
+		$result = [];
+
+		CArrayHelper::sort($dashboards, ['name']);
+
+		foreach ($dashboards as $dashboard) {
+			$result[] = [
+				'name' => $dashboard['name'],
+				'widgets' => $this->formatWidgets($dashboard['widgets'])
+			];
+		}
+
+		return $result;
+	}
+
+	/**
+	 * Format widgets.
+	 *
+	 * @param array $widgets
+	 *
+	 * @return array
+	 */
+	protected function formatWidgets(array $widgets) {
+		$result = [];
+
+		CArrayHelper::sort($widgets, ['name']);
+
+		foreach ($widgets as $widget) {
+			$result[] = [
+				'type' => $widget['type'],
+				'name' => $widget['name'],
+				'x' => $widget['x'],
+				'y' => $widget['y'],
+				'width' => $widget['width'],
+				'height' => $widget['height'],
+				'hide_header' => $widget['view_mode'],
+				'fields' => $this->formatWidgetFields($widget['fields'])
+			];
+		}
+
+		return $result;
+	}
+
+	/**
+	 * Format widget fields.
+	 *
+	 * @param array $widgets
+	 *
+	 * @return array
+	 */
+	protected function formatWidgetFields(array $fields) {
+		$result = [];
+
+		CArrayHelper::sort($fields, ['type']);
+
+		foreach ($fields as $field) {
+			$result[] = [
+				'type' => $field['type'],
+				'name' => $field['name'],
+				'value' => $field['value']
+			];
+		}
+
+		return $result;
+	}
+
 	/**
 	 * Format graph items.
 	 *
@@ -1053,9 +1471,9 @@ class CConfigurationExportBuilder {
 			$link['selementpos1'] = $flipped_selements[$link['selementid1']];
 			$link['selementpos2'] = $flipped_selements[$link['selementid2']];
 
-			// Sort selements positons asc.
+			// Sort selements by position asc.
 			if ($link['selementpos2'] < $link['selementpos1']) {
-				zbx_swap($link['selementpos1'], $link['selementpos2']);
+				[$link['selementpos1'], $link['selementpos2']] = [$link['selementpos2'], $link['selementpos1']];
 			}
 		}
 		unset($link);
