@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2019 Zabbix SIA
+** Copyright (C) 2001-2020 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -129,13 +129,20 @@ class CXmlExportWriter extends CExportWriter {
 			'headers' => 'header',
 			'variables' => 'variable',
 			'query_fields' => 'query_field',
+			'parameters' => 'parameter',
 			'posts' => 'post_field',
 			'shapes' => 'shape',
 			'lines' => 'line',
 			'lld_macro_paths' => 'lld_macro_path',
 			'tls_accept' => 'option',
 			'media_types' => 'media_type',
-			'parameters' => 'parameter'
+			'parameters' => 'parameter',
+			'message_templates' => 'message_template',
+			'overrides' => 'override',
+			'operations' => 'operation',
+			'dashboards' => 'dashboard',
+			'widgets' => 'widget',
+			'fields' => 'field'
 		];
 
 		return isset($map[$name]) ? $map[$name] : false;
