@@ -80,6 +80,10 @@ class CHelpItems {
 					'key' => 'kernel.maxproc',
 					'description' => _('Maximum number of processes supported by OS. Returns integer')
 				],
+				[
+					'key' => 'modbus.get[endpoint,<slaveid>,<function>,<address>,<count>,<type>,<endianness>,<offset>]',
+					'description' => _('Reads modbus data. Returns various types')
+				],
 				[
 					'key' => 'net.dns[<ip>,name,<type>,<timeout>,<count>,<protocol>]',
 					'description' => _('Checks if DNS service is up. Returns 0 - DNS is down (server did not respond or DNS resolution failed); 1 - DNS is up')
@@ -398,6 +402,14 @@ class CHelpItems {
 					'key' => 'logrt.count[file_regexp,<regexp>,<encoding>,<maxproclines>,<mode>,<maxdelay>,<options>]',
 					'description' => _('Count of matched lines in log file monitoring with log rotation support. Returns integer')
 				],
+				[
+					'key' => 'modbus.get[endpoint,<slaveid>,<function>,<address>,<count>,<type>,<endianness>,<offset>]',
+					'description' => _('Reads modbus data. Returns various types')
+				],
+				[
+					'key' => 'mqtt.get[<broker_url>,topic]',
+					'description' => _('Value of MQTT topic. Format of returned data depends on the topic content. If wildcards are used, returns topic values in JSON')
+				],
 				[
 					'key' => 'net.dns[<ip>,name,<type>,<timeout>,<count>,<protocol>]',
 					'description' => _('Checks if DNS service is up. Returns 0 - DNS is down (server did not respond or DNS resolution failed); 1 - DNS is up')
@@ -673,8 +685,20 @@ class CHelpItems {
 			],
 			ITEM_TYPE_AGGREGATE => [
 				[
-					'key' => 'grpfunc[group,key,func,<param>]',
-					'description' => _('Aggregate checks do not require any agent running on a host being monitored. Zabbix server collects aggregate information by doing direct database queries. See Zabbix Manual.')
+					'key' => 'grpavg[group,key,func,<param>]',
+					'description' => _('Calculates the average value, based on the various parameters supplied. Zabbix server collects aggregate information by doing direct database queries.')
+				],
+				[
+					'key' => 'grpmin[group,key,func,<param>]',
+					'description' => _('Calculates the minimum value, based on the various parameters supplied. Zabbix server collects aggregate information by doing direct database queries.')
+				],
+				[
+					'key' => 'grpmax[group,key,func,<param>]',
+					'description' => _('Calculates the maximum value, based on the various parameters supplied. Zabbix server collects aggregate information by doing direct database queries.')
+				],
+				[
+					'key' => 'grpsum[group,key,func,<param>]',
+					'description' => _('Calculates the sum of values, based on the various parameters supplied. Zabbix server collects aggregate information by doing direct database queries.')
 				]
 			],
 			ITEM_TYPE_SIMPLE => [
@@ -718,6 +742,10 @@ class CHelpItems {
 					'key' => 'vmware.fullname[<url>]',
 					'description' => _('VMware service full name, <url> - VMware service URL')
 				],
+				[
+					'key' => 'vmware.dc.discovery[<url>]',
+					'description' => _('VMware datacenters and their IDs. Returns JSON')
+				],
 				[
 					'key' => 'vmware.datastore.read[<url>,<datastore>,<mode>]',
 					'description' => _('VMware datastore read statistics, <url> - VMware service URL, <datastore> - datastore name, <mode> - latency/maxlatency - average or maximum')
@@ -1008,7 +1036,7 @@ class CHelpItems {
 				],
 				[
 					'key' => 'zabbix[proxy,<name>,<param>]',
-					'description' => _('Time of proxy last access. Name - proxy name. Param - lastaccess. Unix timestamp.')
+					'description' => _('Time of proxy last access. Name - proxy name. Valid params are: lastaccess - Unix timestamp, delay - seconds.')
 				],
 				[
 					'key' => 'zabbix[proxy_history]',
@@ -1058,6 +1086,10 @@ class CHelpItems {
 					'key' => 'zabbix[vcache,cache,<parameter>]',
 					'description' => _('Value cache effectiveness. Valid parameters are: requests, hits and misses.')
 				],
+				[
+					'key' => 'zabbix[version]',
+					'description' => _('Version of Zabbix server or proxy')
+				],
 				[
 					'key' => 'zabbix[vmware,buffer,<mode>]',
 					'description' => _('VMware cache statistics. Valid modes are: total, free, pfree, used and pused.')
@@ -1069,15 +1101,15 @@ class CHelpItems {
 			],
 			ITEM_TYPE_DB_MONITOR => [
 				[
-					'key' => 'db.odbc.select[<unique short description>,dsn]',
+					'key' => 'db.odbc.select[<unique short description>,<dsn>,<connection string>]',
 					'description' => _('Return first column of the first row of the SQL query result.')
 				],
 				[
-					'key' => 'db.odbc.discovery[<unique short description>,dsn]',
+					'key' => 'db.odbc.discovery[<unique short description>,<dsn>,<connection string>]',
 					'description' => _('Transform SQL query result into a JSON array for low-level discovery.')
 				],
 				[
-					'key' => 'db.odbc.get[<unique short description>,dsn]',
+					'key' => 'db.odbc.get[<unique short description>,<dsn>,<connection string>]',
 					'description' => _('Transform SQL query result into a JSON array.')
 				]
 			],
@@ -1094,6 +1126,12 @@ class CHelpItems {
 					'key' => 'jmx.get[<discovery mode>,<object name>]',
 					'description' => _('Return a JSON array with MBean objects or their attributes. Compared to jmx.discovery it does not define LLD macros. Can be used for LLD.')
 				]
+			],
+			ITEM_TYPE_IPMI => [
+				[
+					'key' => 'ipmi.get',
+					'description' => _('IPMI sensor IDs and other sensor-related parameters. Returns JSON.')
+				]
 			]
 		];
 	}
