@@ -28,7 +28,7 @@ class CMacrosResolver extends CMacrosResolverGeneral {
 	 */
 	protected $configs = [
 		'scriptConfirmation' => [
-			'types' => ['host', 'interfaceWithoutPort', 'user'],
+			'types' => ['host', 'interfaceWithoutPort', 'user', 'user_data'],
 			'method' => 'resolveTexts'
 		],
 		'httpTestName' => [
@@ -43,6 +43,26 @@ class CMacrosResolver extends CMacrosResolverGeneral {
 			'types' => ['host', 'user'],
 			'method' => 'resolveTexts'
 		],
+		'hostInterfaceDetailsSecurityname' => [
+			'types' => ['user'],
+			'method' => 'resolveTexts'
+		],
+		'hostInterfaceDetailsAuthPassphrase' => [
+			'types' => ['user'],
+			'method' => 'resolveTexts'
+		],
+		'hostInterfaceDetailsPrivPassphrase' => [
+			'types' => ['user'],
+			'method' => 'resolveTexts'
+		],
+		'hostInterfaceDetailsContextName' => [
+			'types' => ['user'],
+			'method' => 'resolveTexts'
+		],
+		'hostInterfaceDetailsCommunity' => [
+			'types' => ['user'],
+			'method' => 'resolveTexts'
+		],
 		'hostInterfacePort' => [
 			'types' => ['user'],
 			'method' => 'resolveTexts'
@@ -115,6 +135,10 @@ class CMacrosResolver extends CMacrosResolverGeneral {
 			$types['macros']['interface'] = ['{IPADDRESS}', '{HOST.IP}', '{HOST.DNS}', '{HOST.CONN}'];
 		}
 
+		if ($this->isTypeAvailable('user_data')) {
+			$types['macros']['user_data'] = ['{USER.ALIAS}', '{USER.FULLNAME}', '{USER.NAME}', '{USER.SURNAME}'];
+		}
+
 		if ($this->isTypeAvailable('user')) {
 			$types['usermacros'] = true;
 		}
@@ -148,6 +172,39 @@ class CMacrosResolver extends CMacrosResolverGeneral {
 					}
 					$interface_hostids[$hostid] = true;
 				}
+
+				if (array_key_exists('user_data', $matched_macros['macros'])
+						&& $matched_macros['macros']['user_data']) {
+					foreach ($matched_macros['macros']['user_data'] as $macro) {
+						switch ($macro) {
+							case '{USER.ALIAS}':
+								$macros[$hostid][$macro] = CWebUser::$data['alias'];
+								break;
+
+							case '{USER.FULLNAME}':
+								$fullname = [];
+
+								foreach (['name', 'surname'] as $field) {
+									if (CWebUser::$data[$field] !== '') {
+										$fullname[] = CWebUser::$data[$field];
+									}
+								}
+
+								$macros[$hostid][$macro] = $fullname
+									? implode(' ', array_merge($fullname, ['('.CWebUser::$data['alias'].')']))
+									: CWebUser::$data['alias'];
+								break;
+
+							case '{USER.NAME}':
+								$macros[$hostid][$macro] = CWebUser::$data['name'];
+								break;
+
+							case '{USER.SURNAME}':
+								$macros[$hostid][$macro] = CWebUser::$data['surname'];
+								break;
+						}
+					}
+				}
 			}
 
 			if ($this->isTypeAvailable('user') && $matched_macros['usermacros']) {
@@ -323,7 +380,8 @@ class CMacrosResolver extends CMacrosResolverGeneral {
 			'host' => [],
 			'interface' => [],
 			'item' => [],
-			'references' => []
+			'references' => [],
+			'log' => []
 		];
 		$usermacros = [];
 		$macro_values = [];
@@ -332,7 +390,10 @@ class CMacrosResolver extends CMacrosResolverGeneral {
 			'macros_n' => [
 				'host' => ['{HOSTNAME}', '{HOST.HOST}', '{HOST.NAME}'],
 				'interface' => ['{IPADDRESS}', '{HOST.IP}', '{HOST.DNS}', '{HOST.CONN}', '{HOST.PORT}'],
-				'item' => ['{ITEM.LASTVALUE}', '{ITEM.VALUE}']
+				'item' => ['{ITEM.LASTVALUE}', '{ITEM.VALUE}'],
+				'log' => ['{ITEM.LOG.DATE}', '{ITEM.LOG.TIME}', '{ITEM.LOG.AGE}', '{ITEM.LOG.SOURCE}',
+					'{ITEM.LOG.SEVERITY}', '{ITEM.LOG.NSEVERITY}', '{ITEM.LOG.EVENTID}'
+				]
 			],
 			'macro_funcs_n' => [
 				'item' => ['{ITEM.LASTVALUE}', '{ITEM.VALUE}']
@@ -377,6 +438,14 @@ class CMacrosResolver extends CMacrosResolverGeneral {
 					}
 				}
 
+				foreach ($matched_macros['macros_n']['log'] as $token => $data) {
+					$macro_values[$triggerid][$token] = UNRESOLVED_MACRO_STRING;
+
+					if (array_key_exists($data['f_num'], $functionids)) {
+						$macros['log'][$functionids[$data['f_num']]][$data['macro']][] = ['token' => $token];
+					}
+				}
+
 				foreach ($matched_macros['macro_funcs_n']['item'] as $token => $data) {
 					$macro_values[$triggerid][$token] = UNRESOLVED_MACRO_STRING;
 
@@ -410,6 +479,7 @@ class CMacrosResolver extends CMacrosResolverGeneral {
 			$macro_values = $this->getHostMacros($macros['host'], $macro_values);
 			$macro_values = $this->getIpMacros($macros['interface'], $macro_values);
 			$macro_values = $this->getItemMacros($macros['item'], $macro_values);
+			$macro_values = $this->getItemLogMacros($macros['log'], $macro_values);
 
 			if ($usermacros) {
 				// Get hosts for triggers.
@@ -476,7 +546,8 @@ class CMacrosResolver extends CMacrosResolverGeneral {
 		$macros = [
 			'host' => [],
 			'interface' => [],
-			'item' => []
+			'item' => [],
+			'log' => [],
 		];
 		$usermacros = [];
 		$macro_values = [];
@@ -485,7 +556,10 @@ class CMacrosResolver extends CMacrosResolverGeneral {
 			'macros_n' => [
 				'host' => ['{HOSTNAME}', '{HOST.HOST}', '{HOST.NAME}'],
 				'interface' => ['{IPADDRESS}', '{HOST.IP}', '{HOST.DNS}', '{HOST.CONN}', '{HOST.PORT}'],
-				'item' => ['{ITEM.LASTVALUE}', '{ITEM.VALUE}']
+				'item' => ['{ITEM.LASTVALUE}', '{ITEM.VALUE}'],
+				'log' => ['{ITEM.LOG.DATE}', '{ITEM.LOG.TIME}', '{ITEM.LOG.AGE}', '{ITEM.LOG.SOURCE}',
+					'{ITEM.LOG.SEVERITY}', '{ITEM.LOG.NSEVERITY}', '{ITEM.LOG.EVENTID}'
+				]
 			],
 			'macro_funcs_n' => [
 				'item' => ['{ITEM.LASTVALUE}', '{ITEM.VALUE}']
@@ -528,6 +602,14 @@ class CMacrosResolver extends CMacrosResolverGeneral {
 				}
 			}
 
+			foreach ($matched_macros['macros_n']['log'] as $token => $data) {
+				$macro_values[$triggerid][$token] = UNRESOLVED_MACRO_STRING;
+
+				if (array_key_exists($data['f_num'], $functionids)) {
+					$macros['log'][$functionids[$data['f_num']]][$data['macro']][] = ['token' => $token];
+				}
+			}
+
 			foreach ($matched_macros['macro_funcs_n']['item'] as $token => $data) {
 				$macro_values[$triggerid][$token] = UNRESOLVED_MACRO_STRING;
 
@@ -549,6 +631,7 @@ class CMacrosResolver extends CMacrosResolverGeneral {
 		$macro_values = $this->getHostMacros($macros['host'], $macro_values);
 		$macro_values = $this->getIpMacros($macros['interface'], $macro_values);
 		$macro_values = $this->getItemMacros($macros['item'], $macro_values, $triggers, $options);
+		$macro_values = $this->getItemLogMacros($macros['log'], $macro_values);
 
 		if ($usermacros) {
 			// Get hosts for triggers.
@@ -632,7 +715,8 @@ class CMacrosResolver extends CMacrosResolverGeneral {
 			'host' => [],
 			'interface' => [],
 			'item' => [],
-			'event' => []
+			'event' => [],
+			'log' => []
 		];
 		$usermacros = [];
 		$macro_values = [];
@@ -645,7 +729,10 @@ class CMacrosResolver extends CMacrosResolverGeneral {
 			'macros_n' => [
 				'host' => ['{HOST.ID}', '{HOST.HOST}', '{HOST.NAME}'],
 				'interface' => ['{HOST.IP}', '{HOST.DNS}', '{HOST.CONN}', '{HOST.PORT}'],
-				'item' => ['{ITEM.LASTVALUE}', '{ITEM.VALUE}']
+				'item' => ['{ITEM.LASTVALUE}', '{ITEM.VALUE}'],
+				'log' => ['{ITEM.LOG.DATE}', '{ITEM.LOG.TIME}', '{ITEM.LOG.AGE}', '{ITEM.LOG.SOURCE}',
+					'{ITEM.LOG.SEVERITY}', '{ITEM.LOG.NSEVERITY}', '{ITEM.LOG.EVENTID}'
+				]
 			],
 			'macro_funcs_n' => [
 				'item' => ['{ITEM.LASTVALUE}', '{ITEM.VALUE}']
@@ -694,6 +781,14 @@ class CMacrosResolver extends CMacrosResolverGeneral {
 			}
 		}
 
+		foreach ($matched_macros['macros_n']['log'] as $token => $data) {
+			$macro_values[$triggerid][$token] = UNRESOLVED_MACRO_STRING;
+
+			if (array_key_exists($data['f_num'], $functionids)) {
+				$macros['log'][$functionids[$data['f_num']]][$data['macro']][] = ['token' => $token];
+			}
+		}
+
 		foreach ($matched_macros['macro_funcs_n']['item'] as $token => $data) {
 			$macro_values[$triggerid][$token] = UNRESOLVED_MACRO_STRING;
 
@@ -714,6 +809,7 @@ class CMacrosResolver extends CMacrosResolverGeneral {
 		$macro_values = $this->getHostMacros($macros['host'], $macro_values);
 		$macro_values = $this->getIpMacros($macros['interface'], $macro_values);
 		$macro_values = $this->getItemMacros($macros['item'], $macro_values);
+		$macro_values = $this->getItemLogMacros($macros['log'], $macro_values);
 
 		if ($usermacros) {
 			// Get hosts for triggers.
@@ -944,19 +1040,25 @@ class CMacrosResolver extends CMacrosResolverGeneral {
 									$link = (new CSpan($function['host'].':'.$function['key_']))->addClass($style);
 								}
 								elseif ($function['flags'] == ZBX_FLAG_DISCOVERY_PROTOTYPE) {
-									$link = (new CLink($function['host'].':'.$function['key_'],
-										'disc_prototypes.php?form=update&itemid='.$function['itemid'].
-										'&parent_discoveryid='.$function['parent_itemid']
-									))
-										->addClass(ZBX_STYLE_LINK_ALT)
-										->addClass($style);
+									$link = CWebUser::checkAccess(CRoleHelper::UI_CONFIGURATION_HOSTS)
+										? (new CLink($function['host'].':'.$function['key_'],
+											'disc_prototypes.php?form=update&itemid='.$function['itemid'].
+											'&parent_discoveryid='.$function['parent_itemid']
+										))
+											->addClass(ZBX_STYLE_LINK_ALT)
+											->addClass($style)
+										: (new CSpan($function['host'].':'.$function['key_']))
+											->addClass($style);
 								}
 								else {
-									$link = (new CLink($function['host'].':'.$function['key_'],
+									$link = CWebUser::checkAccess(CRoleHelper::UI_CONFIGURATION_HOSTS)
+									? (new CLink($function['host'].':'.$function['key_'],
 										'items.php?form=update&itemid='.$function['itemid']
 									))
 										->addClass(ZBX_STYLE_LINK_ALT)
 										->setAttribute('data-itemid', $function['itemid'])
+										->addClass($style)
+									: (new CSpan($function['host'].':'.$function['key_']))
 										->addClass($style);
 								}
 
@@ -1594,6 +1696,41 @@ class CMacrosResolver extends CMacrosResolverGeneral {
 		return $items;
 	}
 
+	/**
+	 * Resolve macros in item description.
+	 *
+	 * @param array  $items
+	 * @param string $items[n]['hostid']
+	 * @param string $items[n]['description']
+	 *
+	 * @return array
+	 */
+	public function resolveItemDescriptions(array $items): array {
+		$types = ['usermacros' => true];
+		$macro_values = [];
+		$usermacros = [];
+
+		foreach ($items as $key => $item) {
+			$matched_macros = self::extractMacros([$item['description']], $types);
+
+			if ($matched_macros['usermacros']) {
+				$usermacros[$key] = ['hostids' => [$item['hostid']], 'macros' => $matched_macros['usermacros']];
+			}
+		}
+
+		foreach ($this->getUserMacros($usermacros) as $key => $usermacros_data) {
+			$macro_values[$key] = array_key_exists($key, $macro_values)
+				? array_merge($macro_values[$key], $usermacros_data['macros'])
+				: $usermacros_data['macros'];
+		}
+
+		foreach ($macro_values as $key => $macro_value) {
+			$items[$key]['description'] = strtr($items[$key]['description'], $macro_value);
+		}
+
+		return $items;
+	}
+
 	/**
 	 * Resolve item delay macros, item history and item trend macros.
 	 *
@@ -2302,19 +2439,19 @@ class CMacrosResolver extends CMacrosResolverGeneral {
 
 					case 'HOST.IP':
 					case 'IPADDRESS': // deprecated
-						if ($host) {
+						if ($host && $host['interface']) {
 							$matched_macro['value'] = $host['interface']['ip'];
 						}
 						break;
 
 					case 'HOST.DNS':
-						if ($host) {
+						if ($host && $host['interface']) {
 							$matched_macro['value'] = $host['interface']['dns'];
 						}
 						break;
 
 					case 'HOST.CONN':
-						if ($host) {
+						if ($host && $host['interface']) {
 							$matched_macro['value'] = $host['interface']['conn'];
 						}
 						break;
