@@ -88,6 +88,12 @@ class CMacrosResolverHelper {
 	 * @param string $interfaces[n]['ip']
 	 * @param string $interfaces[n]['dns']
 	 * @param string $interfaces[n]['port']
+	 * @param array  $interfaces[n]['details']                    (optional)
+	 * @param string $interfaces[n]['details']['securityname']    (optional)
+	 * @param string $interfaces[n]['details']['authpassphrase']  (optional)
+	 * @param string $interfaces[n]['details']['privpassphrase']  (optional)
+	 * @param string $interfaces[n]['details']['contextname']     (optional)
+	 * @param string $interfaces[n]['details']['community']       (optional)
 	 *
 	 * @return array
 	 */
@@ -175,6 +181,92 @@ class CMacrosResolverHelper {
 			unset($interface);
 		}
 
+		// Interface details.
+		$data = [
+			'securityname' => [],
+			'authpassphrase' => [],
+			'privpassphrase' => [],
+			'contextname' => [],
+			'community' => [],
+		];
+
+		foreach ($interfaces as $index => $interface) {
+			$hostid = $interface['hostid'];
+
+			if (array_key_exists('details', $interface)) {
+				if (array_key_exists('securityname', $interface['details'])) {
+					$data['securityname'][$hostid][$index] = $interface['details']['securityname'];
+				}
+
+				if (array_key_exists('authpassphrase', $interface['details'])) {
+					$data['authpassphrase'][$hostid][$index] = $interface['details']['authpassphrase'];
+				}
+
+				if (array_key_exists('privpassphrase', $interface['details'])) {
+					$data['privpassphrase'][$hostid][$index] = $interface['details']['privpassphrase'];
+				}
+
+				if (array_key_exists('contextname', $interface['details'])) {
+					$data['contextname'][$hostid][$index] = $interface['details']['contextname'];
+				}
+
+				if (array_key_exists('community', $interface['details'])) {
+					$data['community'][$hostid][$index] = $interface['details']['community'];
+				}
+			}
+		}
+
+		$resolved_securityname = self::$macrosResolver->resolve([
+			'config' => 'hostInterfaceDetailsSecurityname',
+			'data' => $data['securityname']
+		]);
+
+		$resolved_authpassphrase = self::$macrosResolver->resolve([
+			'config' => 'hostInterfaceDetailsAuthPassphrase',
+			'data' => $data['authpassphrase']
+		]);
+
+		$resolved_privpassphrase = self::$macrosResolver->resolve([
+			'config' => 'hostInterfaceDetailsPrivPassphrase',
+			'data' => $data['privpassphrase']
+		]);
+
+		$resolved_contextname = self::$macrosResolver->resolve([
+			'config' => 'hostInterfaceDetailsContextName',
+			'data' => $data['contextname']
+		]);
+
+		$resolved_community = self::$macrosResolver->resolve([
+			'config' => 'hostInterfaceDetailsCommunity',
+			'data' => $data['community']
+		]);
+
+		foreach ($interfaces as $index => $interface) {
+			$hostid = $interface['hostid'];
+
+			if (array_key_exists('details', $interface)) {
+				if (array_key_exists('securityname', $interface['details'])) {
+					$interfaces[$index]['details']['securityname'] = $resolved_securityname[$hostid][$index];
+				}
+
+				if (array_key_exists('authpassphrase', $interface['details'])) {
+					$interfaces[$index]['details']['authpassphrase'] = $resolved_authpassphrase[$hostid][$index];
+				}
+
+				if (array_key_exists('privpassphrase', $interface['details'])) {
+					$interfaces[$index]['details']['privpassphrase'] = $resolved_privpassphrase[$hostid][$index];
+				}
+
+				if (array_key_exists('contextname', $interface['details'])) {
+					$interfaces[$index]['details']['contextname'] = $resolved_contextname[$hostid][$index];
+				}
+
+				if (array_key_exists('community', $interface['details'])) {
+					$interfaces[$index]['details']['community'] = $resolved_community[$hostid][$index];
+				}
+			}
+		}
+
 		return $interfaces;
 	}
 
@@ -316,19 +408,17 @@ class CMacrosResolverHelper {
 	 * @static
 	 *
 	 * @param string $expression
-	 * @param array  $options		see resolveTriggerExpressions() for more details ('sources' is not supported here)
+	 * @param array  $options     See CMacrosResolver::resolveTriggerExpressions() for more details.
+	 *                            'sources' is not supported here.
 	 *
 	 * @return string
 	 */
 	public static function resolveTriggerExpression($expression, array $options = []) {
 		self::init();
 
-		return self::$macrosResolver->resolveTriggerExpressions([['expression' => $expression]], [
-			'html' => array_key_exists('html', $options) && $options['html'],
-			'resolve_usermacros' => array_key_exists('resolve_usermacros', $options) && $options['resolve_usermacros'],
-			'resolve_macros' => array_key_exists('resolve_macros', $options) && $options['resolve_macros'],
-			'sources' => ['expression']
-		])[0]['expression'];
+		return self::$macrosResolver->resolveTriggerExpressions(
+			[['expression' => $expression]], $options
+		)[0]['expression'];
 	}
 
 	/**
@@ -336,45 +426,15 @@ class CMacrosResolverHelper {
 	 *
 	 * @static
 	 *
-	 * @param array  $triggers
-	 * @param string $triggers[]['expression']
-	 * @param array  $options
-	 * @param bool   $options['html']				(optional) returns formatted trigger expression
-	 * @param bool   $options['resolve_usermacros']	(optional) resolve user macros
-	 * @param bool   $options['resolve_macros']		(optional) resolve macros in item keys and functions
-	 * @param array  $options['sources']			(optional) an array of the field names; default ['expression']
+	 * @param array $triggers
+	 * @param array $options   See CMacrosResolver::resolveTriggerExpressions() for more details.
 	 *
 	 * @return array
 	 */
 	public static function resolveTriggerExpressions(array $triggers, array $options = []) {
 		self::init();
 
-		return self::$macrosResolver->resolveTriggerExpressions($triggers, [
-			'html' => array_key_exists('html', $options) && $options['html'],
-			'resolve_usermacros' => array_key_exists('resolve_usermacros', $options) && $options['resolve_usermacros'],
-			'resolve_macros' => array_key_exists('resolve_macros', $options) && $options['resolve_macros'],
-			'sources' => array_key_exists('sources', $options) ? $options['sources'] : ['expression']
-		]);
-	}
-
-	/**
-	 * Resolve user macros in trigger expression.
-	 *
-	 * @static
-	 *
-	 * @param array $trigger
-	 * @param array $trigger['triggerid']
-	 * @param array $trigger['expression']
-	 *
-	 * @return string
-	 */
-	public static function resolveTriggerExpressionUserMacro(array $trigger) {
-		self::init();
-
-		$triggers = self::$macrosResolver->resolveTriggerExpressionUserMacro(zbx_toHash([$trigger], 'triggerid'));
-		$trigger = reset($triggers);
-
-		return $trigger['expression'];
+		return self::$macrosResolver->resolveTriggerExpressions($triggers, $options);
 	}
 
 	/**
@@ -499,6 +559,23 @@ class CMacrosResolverHelper {
 		return self::$macrosResolver->resolveItemKeys($items);
 	}
 
+	/**
+	 * Resolve macros in item description.
+	 *
+	 * @static
+	 *
+	 * @param array	 $items
+	 * @param string $items[n]['hostid']
+	 * @param string $items[n]['description']
+	 *
+	 * @return array
+	 */
+	public static function resolveItemDescriptions(array $items): array {
+		self::init();
+
+		return self::$macrosResolver->resolveItemDescriptions($items);
+	}
+
 	/**
 	 * Resolve function parameter macros to "parameter_expanded" field.
 	 *
@@ -564,7 +641,7 @@ class CMacrosResolverHelper {
 	 * @param array $data
 	 * @param array $field_names
 	 *
-	 * @return string
+	 * @return array
 	 */
 	public static function resolveTimeUnitMacros(array $data, array $field_names) {
 		self::init();
@@ -624,15 +701,19 @@ class CMacrosResolverHelper {
 	 * @param string $data['steps'][]['params']                  Preprocessing step parameters.
 	 * @param string $data['steps'][]['error_handler_params]     Preprocessing steps error handle parameters.
 	 * @param string $data['delay']                              Update interval value.
-	 * @param string $data['hostids']                            Hostid for which tested item belongs to.
-	 * @param bool   $support_lldmacros                          Enable or disable LLD macro selection.
+	 * @param array  $data['supported_macros']                   Supported macros.
+	 * @param bool   $data['support_lldmacros']                  Either LLD macros need to be extracted.
+	 * @param array  $data['texts_support_macros']               List of texts potentially could contain macros.
+	 * @param array  $data['texts_support_user_macros']          List of texts potentially could contain user macros.
+	 * @param array  $data['texts_support_lld_macros']           List of texts potentially could contain LLD macros.
+	 * @param int    $data['hostids']                            Hostid for which tested item belongs to.
 	 *
 	 * @return array
 	 */
-	public static function extractMacrosFromPreprocessingSteps(array $data, $support_lldmacros) {
+	public static function extractItemTestMacros(array $data) {
 		self::init();
 
-		return self::$macrosResolver->extractMacrosFromPreprocessingSteps($data, $support_lldmacros);
+		return self::$macrosResolver->extractItemTestMacros($data);
 	}
 
 	/**
