@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2019 Zabbix SIA
+** Copyright (C) 2001-2020 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -33,19 +33,15 @@ function eventSource($source = null) {
 	$sources = [
 		EVENT_SOURCE_TRIGGERS => _('trigger'),
 		EVENT_SOURCE_DISCOVERY => _('discovery'),
-		EVENT_SOURCE_AUTO_REGISTRATION => _('auto registration'),
+		EVENT_SOURCE_AUTOREGISTRATION => _('autoregistration'),
 		EVENT_SOURCE_INTERNAL => _x('internal', 'event source')
 	];
 
 	if ($source === null) {
 		return $sources;
 	}
-	elseif (isset($sources[$source])) {
-		return $sources[$source];
-	}
-	else {
-		return _('Unknown');
-	}
+
+	return array_key_exists($source, $sources) ?  $sources[$source] : _('Unknown');
 }
 
 /**
@@ -63,7 +59,7 @@ function eventObject($object = null) {
 		EVENT_OBJECT_TRIGGER => _('trigger'),
 		EVENT_OBJECT_DHOST => _('discovered host'),
 		EVENT_OBJECT_DSERVICE => _('discovered service'),
-		EVENT_OBJECT_AUTOREGHOST => _('auto-registered host'),
+		EVENT_OBJECT_AUTOREGHOST => _('autoregistered host'),
 		EVENT_OBJECT_ITEM => _('item'),
 		EVENT_OBJECT_LLDRULE => _('low-level discovery rule')
 	];
@@ -89,7 +85,7 @@ function eventSourceObjects() {
 		['source' => EVENT_SOURCE_TRIGGERS, 'object' => EVENT_OBJECT_TRIGGER],
 		['source' => EVENT_SOURCE_DISCOVERY, 'object' => EVENT_OBJECT_DHOST],
 		['source' => EVENT_SOURCE_DISCOVERY, 'object' => EVENT_OBJECT_DSERVICE],
-		['source' => EVENT_SOURCE_AUTO_REGISTRATION, 'object' => EVENT_OBJECT_AUTOREGHOST],
+		['source' => EVENT_SOURCE_AUTOREGISTRATION, 'object' => EVENT_OBJECT_AUTOREGHOST],
 		['source' => EVENT_SOURCE_INTERNAL, 'object' => EVENT_OBJECT_TRIGGER],
 		['source' => EVENT_SOURCE_INTERNAL, 'object' => EVENT_OBJECT_ITEM],
 		['source' => EVENT_SOURCE_INTERNAL, 'object' => EVENT_OBJECT_LLDRULE]
@@ -104,12 +100,11 @@ function get_events_unacknowledged($db_element, $value_trigger = null, $value_ev
 		return 0;
 	}
 
-	$config = select_config();
 	$options = [
 		'output' => ['triggerid'],
 		'monitored' => 1,
 		'skipDependent' => 1,
-		'limit' => $config['search_limit'] + 1
+		'limit' => CSettingsHelper::get(CSettingsHelper::SEARCH_LIMIT) + 1
 	];
 	if (!is_null($value_trigger)) {
 		$options['filter'] = ['value' => $value_trigger];
@@ -139,27 +134,23 @@ function get_events_unacknowledged($db_element, $value_trigger = null, $value_ev
 
 /**
  *
- * @param array  $event                   An array of event data.
- * @param string $event['eventid']        Event ID.
- * @param string $event['objectid']       Object ID.
- * @param string $event['correlationid']  OK Event correlation ID.
- * @param string $event['userid']         User ID who generated the OK event.
- * @param string $event['name']           Event name.
- * @param string $event['acknowledged']   State of acknowledgement.
- * @param CCOl   $event['opdata']         Operational data with expanded macros.
- * @param string $event['comments']       Trigger description with expanded macros.
- * @param string $backurl                 A link back after acknowledgement has been clicked.
+ * @param array  $event                      An array of event data.
+ * @param string $event['eventid']           Event ID.
+ * @param string $event['objectid']          Object ID.
+ * @param string $event['correlationid']     OK Event correlation ID.
+ * @param string $event['userid']            User ID who generated the OK event.
+ * @param string $event['name']              Event name.
+ * @param string $event['acknowledged']      State of acknowledgement.
+ * @param CCOl   $event['opdata']            Operational data with expanded macros.
+ * @param string $event['comments']          Trigger description with expanded macros.
+ * @param array  $allowed                    An array of user role rules.
+ * @param bool   $allowed['ack']             Whether user is allowed to make acknowledge actions.
+ * @param bool   $allowed['ui_correlation']  Whether user is allowed to visit event correlation page.
  *
  * @return CTableInfo
  */
-function make_event_details($event, $backurl) {
-	$event_update_url = (new CUrl('zabbix.php'))
-		->setArgument('action', 'acknowledge.edit')
-		->setArgument('eventids', [$event['eventid']])
-		->setArgument('backurl', $backurl)
-		->getUrl();
-
-	$config = select_config();
+function make_event_details(array $event, array $allowed) {
+	$is_acknowledged = ($event['acknowledged'] == EVENT_ACKNOWLEDGED);
 
 	$table = (new CTableInfo())
 		->addRow([
@@ -172,7 +163,7 @@ function make_event_details($event, $backurl) {
 		])
 		->addRow([
 			_('Severity'),
-			getSeverityCell($event['severity'], $config)
+			getSeverityCell($event['severity'])
 		])
 		->addRow([
 			_('Time'),
@@ -180,9 +171,14 @@ function make_event_details($event, $backurl) {
 		])
 		->addRow([
 			_('Acknowledged'),
-			(new CLink($event['acknowledged'] == EVENT_ACKNOWLEDGED ? _('Yes') : _('No'), $event_update_url))
-				->addClass($event['acknowledged'] == EVENT_ACKNOWLEDGED ? ZBX_STYLE_GREEN : ZBX_STYLE_RED)
-				->addClass(ZBX_STYLE_LINK_ALT)
+			$allowed['ack']
+				? (new CLink($is_acknowledged ? _('Yes') : _('No')))
+					->addClass($is_acknowledged ? ZBX_STYLE_GREEN : ZBX_STYLE_RED)
+					->addClass(ZBX_STYLE_LINK_ALT)
+					->onClick('acknowledgePopUp('.json_encode(['eventids' => [$event['eventid']]]).', this);')
+				: (new CSpan($is_acknowledged ? _('Yes') : _('No')))->addClass(
+					$is_acknowledged ? ZBX_STYLE_GREEN : ZBX_STYLE_RED
+				)
 		]);
 
 	if ($event['r_eventid'] != 0) {
@@ -193,7 +189,7 @@ function make_event_details($event, $backurl) {
 			]);
 
 			if ($correlations) {
-				if (CWebUser::getType() == USER_TYPE_SUPER_ADMIN) {
+				if ($allowed['ui_correlation']) {
 					$correlation_name = (new CLink($correlations[0]['name'],
 						(new CUrl('correlation.php'))
 							->setArgument('correlationid', $correlations[0]['correlationid'])
@@ -246,9 +242,17 @@ function make_event_details($event, $backurl) {
 	return $table;
 }
 
-function make_small_eventlist($startEvent, $backurl) {
-	$config = select_config();
-
+/**
+ *
+ * @param array  $startEvent              An array of event data.
+ * @param string $startEvent['eventid']   Event ID.
+ * @param string $startEvent['objectid']  Object ID.
+ * @param array  $allowed                 An array of user role rules.
+ * @param bool   $allowed['ack']          Whether user is allowed to make acknowledge actions.
+ *
+ * @return CTableInfo
+ */
+function make_small_eventlist(array $startEvent, array $allowed) {
 	$table = (new CTableInfo())
 		->setHeader([
 			_('Time'),
@@ -310,14 +314,6 @@ function make_small_eventlist($startEvent, $backurl) {
 		])
 		: [];
 
-	$severity_config = [
-		'severity_name_0' => $config['severity_name_0'],
-		'severity_name_1' => $config['severity_name_1'],
-		'severity_name_2' => $config['severity_name_2'],
-		'severity_name_3' => $config['severity_name_3'],
-		'severity_name_4' => $config['severity_name_4'],
-		'severity_name_5' => $config['severity_name_5']
-	];
 	$actions = getEventsActionsIconsData($events, $triggers, $r_events);
 	$users = API::User()->get([
 		'output' => ['alias', 'name', 'surname'],
@@ -355,20 +351,24 @@ function make_small_eventlist($startEvent, $backurl) {
 			$value_clock = $event['r_clock'];
 		}
 
+		$is_acknowledged = ($event['acknowledged'] == EVENT_ACKNOWLEDGED);
 		$cell_status = new CSpan($value_str);
 
 		/*
 		 * Add colors to span depending on configuration and trigger parameters. No blinking added to status,
 		 * since the page is not on autorefresh.
 		 */
-		addTriggerValueStyle($cell_status, $value, $value_clock, $event['acknowledged'] == EVENT_ACKNOWLEDGED);
+		addTriggerValueStyle($cell_status, $value, $value_clock, $is_acknowledged);
 
-		// Create link to Problem update page.
-		$problem_update_url = (new CUrl('zabbix.php'))
-			->setArgument('action', 'acknowledge.edit')
-			->setArgument('eventids', [$event['eventid']])
-			->setArgument('backurl', $backurl)
-			->getUrl();
+		// Create acknowledge link.
+		$problem_update_link = $allowed['ack']
+			? (new CLink($is_acknowledged ? _('Yes') : _('No')))
+				->addClass($is_acknowledged ? ZBX_STYLE_GREEN : ZBX_STYLE_RED)
+				->addClass(ZBX_STYLE_LINK_ALT)
+				->onClick('acknowledgePopUp('.json_encode(['eventids' => [$event['eventid']]]).', this);')
+			: (new CSpan($is_acknowledged ? _('Yes') : _('No')))->addClass(
+				$is_acknowledged ? ZBX_STYLE_GREEN : ZBX_STYLE_RED
+			);
 
 		$table->addRow([
 			(new CLink(zbx_date2str(DATE_TIME_FORMAT_SECONDS, $event['clock']),
@@ -382,10 +382,8 @@ function make_small_eventlist($startEvent, $backurl) {
 			$cell_status,
 			zbx_date2age($event['clock']),
 			$duration,
-			(new CLink($event['acknowledged'] == EVENT_ACKNOWLEDGED ? _('Yes') : _('No'), $problem_update_url))
-				->addClass($event['acknowledged'] == EVENT_ACKNOWLEDGED ? ZBX_STYLE_GREEN : ZBX_STYLE_RED)
-				->addClass(ZBX_STYLE_LINK_ALT),
-			makeEventActionsIcons($event['eventid'], $actions['data'], $mediatypes, $users, $severity_config)
+			$problem_update_link,
+			makeEventActionsIcons($event['eventid'], $actions['data'], $mediatypes, $users)
 		]);
 	}
 
@@ -400,7 +398,9 @@ function make_small_eventlist($startEvent, $backurl) {
  * @param string $trigger['comments']        Trigger description.
  * @param string $trigger['url']             Trigger URL.
  * @param string $eventid_till
- * @param string $backurl                    URL to return to.
+ * @param array  $allowed                    An array of user role rules.
+ * @param bool   $allowed['ui_problems']     Whether user is allowed to visit problems page.
+ * @param bool   $allowed['ack']             Whether user is allowed to make acknowledge actions.
  * @param bool   $show_timeline              Show time line flag.
  * @param int    $show_tags                  Show tags flag. Possible values:
  *                                             - PROBLEMS_SHOW_TAGS_NONE;
@@ -419,9 +419,8 @@ function make_small_eventlist($startEvent, $backurl) {
  *
  * @return CDiv
  */
-function make_popup_eventlist($trigger, $eventid_till, $backurl, $show_timeline = true,
-		$show_tags = PROBLEMS_SHOW_TAGS_3, array $filter_tags = [], $tag_name_format = PROBLEMS_TAG_NAME_FULL,
-		$tag_priority = '') {
+function make_popup_eventlist($trigger, $eventid_till, $allowed, $show_timeline = true, $show_tags = PROBLEMS_SHOW_TAGS_3,
+		array $filter_tags = [], $tag_name_format = PROBLEMS_TAG_NAME_FULL, $tag_priority = '') {
 	// Show trigger description and URL.
 	$div = new CDiv();
 
@@ -516,9 +515,11 @@ function make_popup_eventlist($trigger, $eventid_till, $backurl, $show_timeline
 			$tags = makeTags($problems, true, 'eventid', $show_tags, $filter_tags, $tag_name_format, $tag_priority);
 		}
 
-		$url_details = (new CUrl('tr_events.php'))
-			->setArgument('triggerid', '')
-			->setArgument('eventid', '');
+		$url_details = $allowed['ui_problems']
+			? (new CUrl('tr_events.php'))
+				->setArgument('triggerid', '')
+				->setArgument('eventid', '')
+			: null;
 
 		foreach ($problems as $problem) {
 			if (array_key_exists($problem['r_eventid'], $r_events)) {
@@ -552,19 +553,28 @@ function make_popup_eventlist($trigger, $eventid_till, $backurl, $show_timeline
 				$value_clock = $in_closing ? time() : $problem['clock'];
 			}
 
-			$url_details
-				->setArgument('triggerid', $problem['objectid'])
-				->setArgument('eventid', $problem['eventid']);
-
 			$cell_clock = ($problem['clock'] >= $today)
 				? zbx_date2str(TIME_FORMAT_SECONDS, $problem['clock'])
 				: zbx_date2str(DATE_TIME_FORMAT_SECONDS, $problem['clock']);
-			$cell_clock = new CCol(new CLink($cell_clock, $url_details));
+
+			if ($url_details !== null) {
+				$url_details
+					->setArgument('triggerid', $problem['objectid'])
+					->setArgument('eventid', $problem['eventid']);
+				$cell_clock = new CCol(new CLink($cell_clock, $url_details));
+			}
+			else {
+				$cell_clock = new CCol($cell_clock);
+			}
+
 			if ($problem['r_eventid'] != 0) {
 				$cell_r_clock = ($problem['r_clock'] >= $today)
 					? zbx_date2str(TIME_FORMAT_SECONDS, $problem['r_clock'])
 					: zbx_date2str(DATE_TIME_FORMAT_SECONDS, $problem['r_clock']);
-				$cell_r_clock = (new CCol(new CLink($cell_r_clock, $url_details)))
+				$cell_r_clock = (new CCol(($url_details !== null)
+					? new CLink($cell_r_clock, $url_details)
+					: $cell_r_clock
+				))
 					->addClass(ZBX_STYLE_NOWRAP)
 					->addClass(ZBX_STYLE_RIGHT);
 			}
@@ -572,10 +582,11 @@ function make_popup_eventlist($trigger, $eventid_till, $backurl, $show_timeline
 				$cell_r_clock = '';
 			}
 
+			$is_acknowledged = ($problem['acknowledged'] == EVENT_ACKNOWLEDGED);
 			$cell_status = new CSpan($value_str);
 
 			// add colors and blinking to span depending on configuration and trigger parameters
-			addTriggerValueStyle($cell_status, $value, $value_clock, $problem['acknowledged'] == EVENT_ACKNOWLEDGED);
+			addTriggerValueStyle($cell_status, $value, $value_clock, $is_acknowledged);
 
 			if ($show_timeline) {
 				if ($last_clock != 0) {
@@ -600,16 +611,14 @@ function make_popup_eventlist($trigger, $eventid_till, $backurl, $show_timeline
 			}
 
 			// Create acknowledge link.
-			$problem_update_url = (new CUrl('zabbix.php'))
-				->setArgument('action', 'acknowledge.edit')
-				->setArgument('eventids', [$problem['eventid']])
-				->setArgument('backurl', $backurl)
-				->getUrl();
-
-			$acknowledged = $problem['acknowledged'] == EVENT_ACKNOWLEDGED;
-			$problem_update_link = (new CLink($acknowledged ? _('Yes') : _('No'), $problem_update_url))
-				->addClass($acknowledged ? ZBX_STYLE_GREEN : ZBX_STYLE_RED)
-				->addClass(ZBX_STYLE_LINK_ALT);
+			$problem_update_link = $allowed['ack']
+				? (new CLink($is_acknowledged ? _('Yes') : _('No')))
+					->addClass($is_acknowledged ? ZBX_STYLE_GREEN : ZBX_STYLE_RED)
+					->addClass(ZBX_STYLE_LINK_ALT)
+					->onClick('acknowledgePopUp('.json_encode(['eventids' => [$problem['eventid']]]).', this);')
+				: (new CSpan($is_acknowledged ? _('Yes') : _('No')))->addClass(
+					$is_acknowledged ? ZBX_STYLE_GREEN : ZBX_STYLE_RED
+				);
 
 			$table->addRow(array_merge($row, [
 				$cell_r_clock,
@@ -699,7 +708,7 @@ function orderEventTagsByPriority(array $event_tags, array $priorities) {
  * @param bool   $html
  * @param string $key                        Name of tag source ID. Possible values:
  *                                            - 'eventid' - for events and problems (default);
- *                                            - 'hostid' - for hosts;
+ *                                            - 'hostid' - for hosts and host prototypes;
  *                                            - 'templateid' - for templates;
  *                                            - 'triggerid' - for triggers.
  * @param int    $list_tag_count             Maximum number of tags to display.
@@ -715,8 +724,8 @@ function orderEventTagsByPriority(array $event_tags, array $priorities) {
  *
  * @return array
  */
-function makeTags(array $list, $html = true, $key = 'eventid', $list_tag_count = ZBX_TAG_COUNT_DEFAULT,
-		array $filter_tags = [], $tag_name_format = PROBLEMS_TAG_NAME_FULL, $tag_priority = '') {
+function makeTags(array $list, bool $html = true, string $key = 'eventid', int $list_tag_count = ZBX_TAG_COUNT_DEFAULT,
+		array $filter_tags = [], int $tag_name_format = PROBLEMS_TAG_NAME_FULL, string $tag_priority = ''): array {
 	$tags = [];
 
 	if ($html) {
