@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2017 Zabbix SIA
+** Copyright (C) 2001-2020 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -33,12 +33,11 @@ function screen_resources($resource = null) {
 		SCREEN_RESOURCE_HOST_INFO => _('Host info'),
 		SCREEN_RESOURCE_MAP => _('Map'),
 		SCREEN_RESOURCE_PLAIN_TEXT => _('Plain text'),
-		SCREEN_RESOURCE_SCREEN => _('Screen'),
-		SCREEN_RESOURCE_SERVER_INFO => _('Status of Zabbix'),
+		SCREEN_RESOURCE_SERVER_INFO => _('System information'),
 		SCREEN_RESOURCE_SIMPLE_GRAPH => _('Simple graph'),
 		SCREEN_RESOURCE_HOSTGROUP_TRIGGERS => _('Host group issues'),
 		SCREEN_RESOURCE_HOST_TRIGGERS => _('Host issues'),
-		SCREEN_RESOURCE_SYSTEM_STATUS => _('System status'),
+		SCREEN_RESOURCE_SYSTEM_STATUS => _('Problems by severity'),
 		SCREEN_RESOURCE_TRIGGER_INFO => _('Trigger info'),
 		SCREEN_RESOURCE_TRIGGER_OVERVIEW => _('Trigger overview'),
 		SCREEN_RESOURCE_URL => _('URL'),
@@ -58,26 +57,6 @@ function screen_resources($resource = null) {
 	}
 }
 
-function check_screen_recursion($mother_screenid, $child_screenid) {
-	if (bccomp($mother_screenid , $child_screenid) == 0) {
-		return true;
-	}
-
-	$db_scr_items = DBselect(
-		'SELECT si.resourceid'.
-		' FROM screens_items si'.
-		' WHERE si.screenid='.zbx_dbstr($child_screenid).
-		' AND si.resourcetype='.SCREEN_RESOURCE_SCREEN
-	);
-	while ($scr_item = DBfetch($db_scr_items)) {
-		if (check_screen_recursion($mother_screenid, $scr_item['resourceid'])) {
-			return true;
-		}
-	}
-
-	return false;
-}
-
 /**
  * Add screen row.
  *
@@ -99,9 +78,7 @@ function addScreenRow(array $screen, $row_num) {
 		'screenitems' => $screen['screenitems']
 	];
 
-	$result = ($screen['templateid'] != 0)
-		? API::TemplateScreen()->update($options)
-		: API::Screen()->update($options);
+	$result = API::Screen()->update($options);
 
 	if ($result) {
 		add_audit_details(AUDIT_ACTION_UPDATE, AUDIT_RESOURCE_SCREEN, $screen['screenid'], $screen['name'],
@@ -132,9 +109,7 @@ function addScreenColumn(array $screen, $col_num) {
 		'screenitems' => $screen['screenitems']
 	];
 
-	$result = ($screen['templateid'] != 0)
-		? API::TemplateScreen()->update($options)
-		: API::Screen()->update($options);
+	$result = API::Screen()->update($options);
 
 	if ($result) {
 		add_audit_details(AUDIT_ACTION_UPDATE, AUDIT_RESOURCE_SCREEN, $screen['screenid'], $screen['name'],
@@ -168,9 +143,7 @@ function delScreenRow(array $screen, $row_num) {
 		'screenitems' => $screen['screenitems']
 	];
 
-	$result = ($screen['templateid'] != 0)
-		? API::TemplateScreen()->update($options)
-		: API::Screen()->update($options);
+	$result = API::Screen()->update($options);
 
 	if ($result) {
 		add_audit_details(AUDIT_ACTION_UPDATE, AUDIT_RESOURCE_SCREEN, $screen['screenid'], $screen['name'],
@@ -204,9 +177,7 @@ function delScreenColumn(array $screen, $col_num) {
 		'screenitems' => $screen['screenitems']
 	];
 
-	$result = ($screen['templateid'] != 0)
-		? API::TemplateScreen()->update($options)
-		: API::Screen()->update($options);
+	$result = API::Screen()->update($options);
 
 	if ($result) {
 		add_audit_details(AUDIT_ACTION_UPDATE, AUDIT_RESOURCE_SCREEN, $screen['screenid'], $screen['name'],
@@ -361,7 +332,7 @@ function add_slideshow($data) {
 	));
 
 	if ($db_slideshow) {
-		error(_s('Slide show "%s" already exists.', $data['name']));
+		error(_s('Slide show "%1$s" already exists.', $data['name']));
 
 		return false;
 	}
