@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2018 Zabbix SIA
+** Copyright (C) 2001-2020 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -19,37 +19,60 @@
 **/
 
 
+/**
+ * @var CView $this
+ */
+
+$submenu_source = [
+	EVENT_SOURCE_TRIGGERS => _('Trigger actions'),
+	EVENT_SOURCE_DISCOVERY => _('Discovery actions'),
+	EVENT_SOURCE_AUTOREGISTRATION => _('Autoregistration actions'),
+	EVENT_SOURCE_INTERNAL => _('Internal actions')
+];
+
+$submenu = [];
+foreach ($submenu_source as $value => $label) {
+	$url = (new CUrl('actionconf.php'))
+		->setArgument('eventsource', $value)
+		->getUrl();
+
+	$submenu[$url] = $label;
+}
+
 $widget = (new CWidget())
-	->setTitle(_('Actions'))
-	->setControls((new CForm('get'))
-		->cleanItems()
-		->addItem((new CList())
-			->addItem([
-				new CLabel(_('Event source'), 'eventsource'),
-				(new CDiv())->addClass(ZBX_STYLE_FORM_INPUT_MARGIN),
-				new CComboBox('eventsource', $data['eventsource'], 'submit()', [
-					EVENT_SOURCE_TRIGGERS => _('Triggers'),
-					EVENT_SOURCE_DISCOVERY => _('Discovery'),
-					EVENT_SOURCE_AUTO_REGISTRATION => _('Auto registration'),
-					EVENT_SOURCE_INTERNAL => _x('Internal', 'event source')
-				])
-			])
-			->addItem(new CSubmit('form', _('Create action')))
-		)
+	->setTitle(array_key_exists($data['eventsource'], $submenu_source) ? $submenu_source[$data['eventsource']] : null)
+	->setTitleSubmenu([
+		'main_section' => [
+			'items' => $submenu
+		]
+	])
+	->setControls((new CTag('nav', true,
+		(new CForm('get'))
+			->cleanItems()
+			->addItem(new CInput('hidden', 'eventsource', $data['eventsource']))
+			->addItem((new CList())
+				->addItem(new CSubmit('form', _('Create action')))
+			)
+		))
+			->setAttribute('aria-label', _('Content controls'))
 	)
-	->addItem((new CFilter('web.actionconf.filter.state'))
-		->addColumn((new CFormList())->addRow(_('Name'),
-			(new CTextBox('filter_name', $data['filter']['name']))
-				->setWidth(ZBX_TEXTAREA_FILTER_SMALL_WIDTH)
-				->setAttribute('autofocus', 'autofocus')
-		))
-		->addColumn((new CFormList())->addRow(_('Status'),
-			(new CRadioButtonList('filter_status', (int) $data['filter']['status']))
-				->addValue(_('Any'), -1)
-				->addValue(_('Enabled'), ACTION_STATUS_ENABLED)
-				->addValue(_('Disabled'), ACTION_STATUS_DISABLED)
-				->setModern(true)
-		))
+	->addItem((new CFilter(new CUrl('actionconf.php')))
+		->setProfile($data['profileIdx'])
+		->setActiveTab($data['active_tab'])
+		->addFilterTab(_('Filter'), [
+			(new CFormList())->addRow(_('Name'),
+				(new CTextBox('filter_name', $data['filter']['name']))
+					->setWidth(ZBX_TEXTAREA_FILTER_SMALL_WIDTH)
+					->setAttribute('autofocus', 'autofocus')
+			),
+			(new CFormList())->addRow(_('Status'),
+				(new CRadioButtonList('filter_status', (int) $data['filter']['status']))
+					->addValue(_('Any'), -1)
+					->addValue(_('Enabled'), ACTION_STATUS_ENABLED)
+					->addValue(_('Disabled'), ACTION_STATUS_DISABLED)
+					->setModern(true)
+			)
+		])
 	);
 
 // create form
@@ -69,7 +92,7 @@ $actionTable = (new CTableInfo())
 	]);
 
 if ($this->data['actions']) {
-	$actionConditionStringValues = actionConditionValueToString($this->data['actions'], $this->data['config']);
+	$actionConditionStringValues = actionConditionValueToString($this->data['actions']);
 	$actionOperationDescriptions = getActionOperationDescriptions($this->data['actions'], ACTION_OPERATION);
 
 	foreach ($this->data['actions'] as $aIdx => $action) {
@@ -132,4 +155,4 @@ $actionForm->addItem([
 // append form to widget
 $widget->addItem($actionForm);
 
-return $widget;
+$widget->show();
