@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2019 Zabbix SIA
+** Copyright (C) 2001-2020 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -19,6 +19,10 @@
 **/
 
 
+/**
+ * @var CView $this
+ */
+
 require_once dirname(__FILE__).'/js/configuration.host.edit.js.php';
 require_once dirname(__FILE__).'/js/common.template.edit.js.php';
 
@@ -27,14 +31,16 @@ $widget = (new CWidget())
 	->addItem(get_header_host_table('', $data['hostid']));
 
 $divTabs = new CTabView();
+
 if (!hasRequest('form_refresh')) {
 	$divTabs->setSelected(0);
 }
 
 $frmHost = (new CForm())
-	->setId('hostsForm')
+	->setId('hosts-form')
 	->setName('hostsForm')
 	->setAttribute('aria-labeledby', ZBX_STYLE_PAGE_TITLE)
+	->disablePasswordAutofill()
 	->addVar('form', $data['form'])
 	->addVar('clear_templates', $data['clear_templates'])
 	->addVar('flags', $data['flags'])
@@ -51,7 +57,7 @@ if ($data['clone_hostid'] != 0) {
 $hostList = new CFormList('hostlist');
 
 // LLD rule link
-if ($data['flags'] == ZBX_FLAG_DISCOVERY_CREATED) {
+if ($data['readonly']) {
 	$hostList->addRow(_('Discovered by'), $data['discoveryRule']
 		? new CLink($data['discoveryRule']['name'],
 			(new CUrl('host_prototypes.php'))
@@ -66,20 +72,20 @@ if ($data['flags'] == ZBX_FLAG_DISCOVERY_CREATED) {
 $hostList
 	->addRow(
 		(new CLabel(_('Host name'), 'host'))->setAsteriskMark(),
-		(new CTextBox('host', $data['host'], ($data['flags'] == ZBX_FLAG_DISCOVERY_CREATED), 128))
+		(new CTextBox('host', $data['host'], $data['readonly'], 128))
 			->setWidth(ZBX_TEXTAREA_STANDARD_WIDTH)
 			->setAriaRequired()
 			->setAttribute('autofocus', 'autofocus')
 	)
 	->addRow(_('Visible name'),
-		(new CTextBox('visiblename', $data['visiblename'], ($data['flags'] == ZBX_FLAG_DISCOVERY_CREATED), 128))
+		(new CTextBox('visiblename', $data['visiblename'], $data['readonly'], 128))
 			->setWidth(ZBX_TEXTAREA_STANDARD_WIDTH)
 	)
 	->addRow((new CLabel(_('Groups'), 'groups__ms'))->setAsteriskMark(),
 		(new CMultiSelect([
 			'name' => 'groups[]',
 			'object_name' => 'hostGroup',
-			'disabled' => ($data['flags'] == ZBX_FLAG_DISCOVERY_CREATED),
+			'disabled' => $data['readonly'],
 			'add_new' => (CWebUser::$data['type'] == USER_TYPE_SUPER_ADMIN),
 			'data' => $data['groups_ms'],
 			'popup' => [
@@ -92,194 +98,107 @@ $hostList
 				]
 			]
 		]))
-			->setAriaRequired()
 			->setWidth(ZBX_TEXTAREA_STANDARD_WIDTH)
+			->setAriaRequired()
 	);
 
-// interfaces for normal hosts
+// Interfaces for normal hosts.
 if ($data['flags'] != ZBX_FLAG_DISCOVERY_CREATED) {
-	zbx_add_post_js($data['interfaces']
-		? 'hostInterfacesManager.add('.CJs::encodeJson($data['interfaces']).');'
-		: 'hostInterfacesManager.addNew("agent");');
+	zbx_add_post_js('window.hostInterfaceManager = new HostInterfaceManager('.json_encode($data['interfaces']).');');
+	zbx_add_post_js('hostInterfaceManager.render();');
 
-	$hostList->addRow('',
-		(new CLabel(_('At least one interface must exist.')))->setAsteriskMark()
-	);
-	// Zabbix agent interfaces
-	$ifTab = (new CTable())
+	$interface_header = renderInterfaceHeaders();
+
+	$agent_interfaces = (new CDiv())
 		->setId('agentInterfaces')
-		->setHeader([
-			new CColHeader(),
-			new CColHeader(_('IP address')),
-			new CColHeader(_('DNS name')),
-			new CColHeader(_('Connect to')),
-			new CColHeader(_('Port')),
-			(new CColHeader(_('Default')))->setColSpan(2)
-		])
-		->addRow((new CRow([
-			(new CCol(
-				(new CButton('addAgentInterface', _('Add')))->addClass(ZBX_STYLE_BTN_LINK)
-			))->setColSpan(7)
-		]))->setId('agentInterfacesFooter'));
+		->addClass(ZBX_STYLE_HOST_INTERFACE_CONTAINER)
+		->setAttribute('data-type', 'agent');
 
-	$hostList->addRow(_('Agent interfaces'),
-		(new CDiv($ifTab))
-			->addClass(ZBX_STYLE_TABLE_FORMS_SEPARATOR)
-			->setAttribute('data-type', 'agent')
-			->setWidth(ZBX_HOST_INTERFACE_WIDTH)
-	);
-
-	// SNMP interfaces
-	$ifTab = (new CTable())
+	$snmp_interfaces = (new CDiv())
 		->setId('SNMPInterfaces')
-		->addRow((new CRow([
-			(new CCol(
-				(new CButton('addSNMPInterface', _('Add')))->addClass(ZBX_STYLE_BTN_LINK)
-			))->setColSpan(7)
-		]))->setId('SNMPInterfacesFooter'));
+		->addClass(ZBX_STYLE_HOST_INTERFACE_CONTAINER.' '.ZBX_STYLE_LIST_VERTICAL_ACCORDION)
+		->setAttribute('data-type', 'snmp');
 
-	$hostList->addRow(_('SNMP interfaces'),
-		(new CDiv($ifTab))
-			->addClass(ZBX_STYLE_TABLE_FORMS_SEPARATOR)
-			->setAttribute('data-type', 'snmp')
-			->setWidth(ZBX_HOST_INTERFACE_WIDTH)
-	);
-
-	// JMX interfaces
-	$ifTab = (new CTable())
+	$jmx_interfaces = (new CDiv())
 		->setId('JMXInterfaces')
-		->addRow((new CRow([
-			(new CCol(
-				(new CButton('addJMXInterface', _('Add')))->addClass(ZBX_STYLE_BTN_LINK)
-			))->setColSpan(7)
-		]))->setId('JMXInterfacesFooter'));
+		->addClass(ZBX_STYLE_HOST_INTERFACE_CONTAINER)
+		->setAttribute('data-type', 'jmx');
 
-	$hostList->addRow(_('JMX interfaces'),
-		(new CDiv($ifTab))
-			->addClass(ZBX_STYLE_TABLE_FORMS_SEPARATOR)
-			->setAttribute('data-type', 'jmx')
-			->setWidth(ZBX_HOST_INTERFACE_WIDTH)
-	);
-
-	// IPMI interfaces
-	$ifTab = (new CTable())
+	$ipmi_interfaces = (new CDiv())
 		->setId('IPMIInterfaces')
-		->addRow((new CRow([
-			(new CCol(
-				(new CButton('addIPMIInterface', _('Add')))->addClass(ZBX_STYLE_BTN_LINK)
-			))->setColSpan(7)
-		]))->setId('IPMIInterfacesFooter'));
+		->addClass(ZBX_STYLE_HOST_INTERFACE_CONTAINER)
+		->setAttribute('data-type', 'ipmi');
 
-	$hostList->addRow(_('IPMI interfaces'),
-		(new CDiv($ifTab))
-			->addClass(ZBX_STYLE_TABLE_FORMS_SEPARATOR)
-			->setAttribute('data-type', 'ipmi')
-			->setWidth(ZBX_HOST_INTERFACE_WIDTH)
+	$hostList->addRow(new CLabel(_('Interfaces')),
+		[
+			new CDiv([$interface_header, $agent_interfaces, $snmp_interfaces, $jmx_interfaces, $ipmi_interfaces]),
+			new CDiv(
+				(new CButton('', _('Add')))
+					->addClass(ZBX_STYLE_BTN_LINK)
+					->setMenuPopup([
+						'type' => 'submenu',
+						'data' => [
+							'submenu' => getAddNewInterfaceSubmenu()
+						]
+					])
+					->setAttribute('aria-label', _('Add new interface'))
+			)
+		]
 	);
 }
-// interfaces for discovered hosts
+// Interfaces for discovered hosts.
 else {
-	$existingInterfaceTypes = [];
-	foreach ($data['interfaces'] as $interface) {
-		$existingInterfaceTypes[$interface['type']] = true;
-	}
-	zbx_add_post_js('hostInterfacesManager.add('.CJs::encodeJson($data['interfaces']).');');
-	zbx_add_post_js('hostInterfacesManager.disable();');
+	zbx_add_post_js('window.hostInterfaceManager = new HostInterfaceManager('.json_encode($data['interfaces']).');');
+	zbx_add_post_js('hostInterfaceManager.render();');
+	zbx_add_post_js('HostInterfaceManager.makeReadonly();');
 
 	$hostList->addVar('interfaces', $data['interfaces']);
 
-	// Zabbix agent interfaces
-	$ifTab = (new CTable())
+	$interface_header = renderInterfaceHeaders();
+
+	$agent_interfaces = (new CDiv())
 		->setId('agentInterfaces')
-		->setHeader([
-			new CColHeader(),
-			new CColHeader(_('IP address')),
-			new CColHeader(_('DNS name')),
-			new CColHeader(_('Connect to')),
-			new CColHeader(_('Port')),
-			(new CColHeader(_('Default')))->setColSpan(2)
-		]);
+		->addClass(ZBX_STYLE_HOST_INTERFACE_CONTAINER)
+		->setAttribute('data-type', 'agent');
 
-	$row = (new CRow())->setId('agentInterfacesFooter');
-	if (!array_key_exists(INTERFACE_TYPE_AGENT, $existingInterfaceTypes)) {
-		$row->addItem(new CCol());
-		$row->addItem((new CCol(_('No agent interfaces found.')))->setColSpan(6));
-	}
-	$ifTab->addRow($row);
+	$snmp_interfaces = (new CDiv())
+		->setId('SNMPInterfaces')
+		->addClass(ZBX_STYLE_HOST_INTERFACE_CONTAINER.' '.ZBX_STYLE_LIST_VERTICAL_ACCORDION)
+		->setAttribute('data-type', 'snmp');
 
-	$hostList->addRow(_('Agent interfaces'),
-		(new CDiv($ifTab))
-			->addClass(ZBX_STYLE_TABLE_FORMS_SEPARATOR)
-			->setAttribute('data-type', 'agent')
-			->setWidth(ZBX_HOST_INTERFACE_WIDTH)
-	);
+	$jmx_interfaces = (new CDiv())
+		->setId('JMXInterfaces')
+		->addClass(ZBX_STYLE_HOST_INTERFACE_CONTAINER)
+		->setAttribute('data-type', 'jmx');
 
-	// SNMP interfaces
-	$ifTab = (new CTable())->setId('SNMPInterfaces');
+	$ipmi_interfaces = (new CDiv())
+		->setId('IPMIInterfaces')
+		->addClass(ZBX_STYLE_HOST_INTERFACE_CONTAINER)
+		->setAttribute('data-type', 'ipmi');
 
-	$row = (new CRow())->setId('SNMPInterfacesFooter');
-	if (!array_key_exists(INTERFACE_TYPE_SNMP, $existingInterfaceTypes)) {
-		$row->addItem(new CCol());
-		$row->addItem((new CCol(_('No SNMP interfaces found.')))->setColSpan(6));
-	}
-	$ifTab->addRow($row);
-
-	$hostList->addRow(_('SNMP interfaces'),
-		(new CDiv($ifTab))
-			->addClass(ZBX_STYLE_TABLE_FORMS_SEPARATOR)
-			->setAttribute('data-type', 'snmp')
-			->setWidth(ZBX_HOST_INTERFACE_WIDTH)
-	);
-
-	// JMX interfaces
-	$ifTab = (new CTable())->setId('JMXInterfaces');
-
-	$row = (new CRow())->setId('JMXInterfacesFooter');
-	if (!array_key_exists(INTERFACE_TYPE_JMX, $existingInterfaceTypes)) {
-		$row->addItem(new CCol());
-		$row->addItem((new CCol(_('No JMX interfaces found.')))->setColSpan(6));
-	}
-	$ifTab->addRow($row);
-
-	$hostList->addRow(_('JMX interfaces'),
-		(new CDiv($ifTab))
-			->addClass(ZBX_STYLE_TABLE_FORMS_SEPARATOR)
-			->setAttribute('data-type', 'jmx')
-			->setWidth(ZBX_HOST_INTERFACE_WIDTH)
-	);
-
-	// IPMI interfaces
-	$ifTab = (new CTable())->setId('IPMIInterfaces');
-
-	$row = (new CRow())->setId('IPMIInterfacesFooter');
-	if (!array_key_exists(INTERFACE_TYPE_IPMI, $existingInterfaceTypes)) {
-		$row->addItem(new CCol());
-		$row->addItem((new CCol(_('No IPMI interfaces found.')))->setColSpan(6));
-	}
-	$ifTab->addRow($row);
-
-	$hostList->addRow(_('IPMI interfaces'),
-		(new CDiv($ifTab))
-			->addClass(ZBX_STYLE_TABLE_FORMS_SEPARATOR)
-			->setAttribute('data-type', 'ipmi')
-			->setWidth(ZBX_HOST_INTERFACE_WIDTH)
+	$hostList->addRow(new CLabel(_('Interfaces')),
+		[new CDiv([$interface_header, $agent_interfaces, $snmp_interfaces, $jmx_interfaces, $ipmi_interfaces])]
 	);
 }
 
 $hostList->addRow(_('Description'),
-	(new CTextArea('description', $data['description']))->setWidth(ZBX_TEXTAREA_STANDARD_WIDTH)
+	(new CTextArea('description', $data['description']))
+		->setWidth(ZBX_TEXTAREA_STANDARD_WIDTH)
+		->setMaxlength(DB::getFieldLength('hosts', 'description'))
 );
 
 // Proxy
-if ($data['flags'] != ZBX_FLAG_DISCOVERY_CREATED) {
-	$proxy = new CComboBox('proxy_hostid', $data['proxy_hostid'], null, [0 => _('(no proxy)')] + $data['proxies']);
-	$proxy->setEnabled($data['flags'] != ZBX_FLAG_DISCOVERY_CREATED);
-}
-else {
-	$proxy = (new CTextBox(null, $data['proxy_hostid'] != 0 ? $data['proxies'][$data['proxy_hostid']] : _('(no proxy)'), true))
-		->setWidth(ZBX_TEXTAREA_STANDARD_WIDTH);
+if ($data['readonly']) {
+	$proxy = (new CTextBox(null,
+		($data['proxy_hostid'] == 0) ? _('(no proxy)') : $data['proxies'][$data['proxy_hostid']], true)
+	)->setWidth(ZBX_TEXTAREA_STANDARD_WIDTH);
 	$hostList->addVar('proxy_hostid', $data['proxy_hostid']);
 }
+else {
+	$proxy = new CComboBox('proxy_hostid', $data['proxy_hostid'], null, [0 => _('(no proxy)')] + $data['proxies']);
+	$proxy->setEnabled(true);
+}
+
 $hostList->addRow(_('Monitored by proxy'), $proxy);
 
 $hostList->addRow(_('Enabled'),
@@ -542,19 +461,51 @@ $divTabs->addTab('hostTab', _('Host'), $hostList);
 // templates
 $tmplList = new CFormList();
 
-// templates for normal hosts
-if ($data['flags'] != ZBX_FLAG_DISCOVERY_CREATED) {
+// Templates for discovered hosts.
+if ($data['readonly']) {
+	$linkedTemplateTable = (new CTable())
+		->setId('linked-template')
+		->setHeader([_('Name')])
+		->addStyle('width: 100%;');
+
+	foreach ($data['linked_templates'] as $template) {
+		$tmplList->addVar('templates[]', $template['templateid']);
+
+		if ($data['allowed_ui_conf_templates']
+				&& array_key_exists($template['templateid'], $data['writable_templates'])) {
+			$template_link = (new CLink($template['name'],
+				(new CUrl('templates.php'))
+					->setArgument('form','update')
+					->setArgument('templateid', $template['templateid'])
+			))->setTarget('_blank');
+		}
+		else {
+			$template_link = new CSpan($template['name']);
+		}
+
+		$linkedTemplateTable->addRow($template_link, null, 'conditions_'.$template['templateid']);
+	}
+
+	$tmplList->addRow(_('Linked templates'),
+		(new CDiv($linkedTemplateTable))
+			->addClass(ZBX_STYLE_TABLE_FORMS_SEPARATOR)
+			->addStyle('min-width: '.ZBX_TEXTAREA_BIG_WIDTH.'px;')
+	);
+}
+// Templates for normal hosts.
+else {
 	$disableids = [];
 
 	$linkedTemplateTable = (new CTable())
+		->setId('linked-template')
 		->setAttribute('style', 'width: 100%;')
 		->setHeader([_('Name'), _('Action')]);
 
-	$n = 0;
 	foreach ($data['linked_templates'] as $template) {
-		$tmplList->addVar('templates['.$n++.']', $template['templateid']);
+		$tmplList->addItem((new CVar('templates['.$template['templateid'].']', $template['templateid']))->removeId());
 
-		if (array_key_exists($template['templateid'], $data['writable_templates'])) {
+		if ($data['allowed_ui_conf_templates']
+				&& array_key_exists($template['templateid'], $data['writable_templates'])) {
 			$template_link = (new CLink($template['name'],
 				'templates.php?form=update&templateid='.$template['templateid']
 			))->setTarget('_blank');
@@ -586,67 +537,43 @@ if ($data['flags'] != ZBX_FLAG_DISCOVERY_CREATED) {
 		$disableids[] = $template['templateid'];
 	}
 
-	$linkedTemplateTable->addRow([
-		(new CSimpleButton(_('Add')))
-			->onClick('return PopUp("popup.generic",'.CJs::encodeJson([
-				'dstfrm' => $frmHost->getName(),
-				'dstfld1' => $frmHost->getName(),
+	$add_templates_ms = (new CMultiSelect([
+		'name' => 'add_templates[]',
+		'object_name' => 'templates',
+		'data' => $data['add_templates'],
+		'popup' => [
+			'parameters' => [
 				'srctbl' => 'templates',
 				'srcfld1' => 'hostid',
-				'templated_hosts' => '1',
-				'popup_type' => 'templates',
-				'multiselect' => 1,
+				'srcfld2' => 'host',
+				'dstfrm' => $frmHost->getName(),
+				'dstfld1' => 'add_templates_',
 				'disableids' => $disableids
-			]).', null, this);')
-			->addClass(ZBX_STYLE_BTN_LINK)
-	]);
+			]
+		]
+	]))->setWidth(ZBX_TEXTAREA_STANDARD_WIDTH);
 
-	$tmplList->addRow(_('Linked templates'),
-		(new CDiv($linkedTemplateTable))
-			->addClass(ZBX_STYLE_TABLE_FORMS_SEPARATOR)
-			->setAttribute('style', 'min-width: '.ZBX_TEXTAREA_BIG_WIDTH.'px;')
-	);
-}
-// templates for discovered hosts
-else {
-	$linkedTemplateTable = (new CTable())
-		->setHeader([_('Name')])
-		->addStyle('width: 100%;');
-
-	foreach ($data['linked_templates'] as $template) {
-		$tmplList->addVar('templates[]', $template['templateid']);
-
-		if (array_key_exists($template['templateid'], $data['writable_templates'])) {
-			$template_link = (new CLink($template['name'],
-				(new CUrl('templates.php'))
-					->setArgument('form','update')
-					->setArgument('templateid', $template['templateid'])
-			))->setTarget('_blank');
-		}
-		else {
-			$template_link = new CSpan($template['name']);
-		}
-
-		$linkedTemplateTable->addRow($template_link, null, 'conditions_'.$template['templateid']);
-	}
-
-	$tmplList->addRow(_('Linked templates'),
-		(new CDiv($linkedTemplateTable))
-			->addClass(ZBX_STYLE_TABLE_FORMS_SEPARATOR)
-			->addStyle('min-width: '.ZBX_TEXTAREA_BIG_WIDTH.'px;')
-	);
+	$tmplList
+		->addRow(_('Linked templates'),
+			(new CDiv($linkedTemplateTable))
+				->addClass(ZBX_STYLE_TABLE_FORMS_SEPARATOR)
+				->setAttribute('style', 'min-width: '.ZBX_TEXTAREA_BIG_WIDTH.'px;')
+		)
+		->addRow((new CLabel(_('Link new templates'), 'add_templates__ms')),
+			(new CDiv(
+				(new CTable())->addRow([$add_templates_ms])
+			))
+				->addClass(ZBX_STYLE_TABLE_FORMS_SEPARATOR)
+				->setAttribute('style', 'min-width: '.ZBX_TEXTAREA_BIG_WIDTH.'px;')
+		);
 }
 
-$divTabs->addTab('templateTab', _('Templates'), $tmplList);
+$divTabs->addTab('templateTab', _('Templates'), $tmplList, TAB_INDICATOR_LINKED_TEMPLATE);
 
 /*
  * IPMI
  */
-if ($data['flags'] != ZBX_FLAG_DISCOVERY_CREATED) {
-	$cmbIPMIAuthtype = new CListBox('ipmi_authtype', $data['ipmi_authtype'], 7, null, ipmiAuthTypes());
-	$cmbIPMIPrivilege = new CListBox('ipmi_privilege', $data['ipmi_privilege'], 5, null, ipmiPrivileges());
-}
-else {
+if ($data['readonly']) {
 	$cmbIPMIAuthtype = [
 		(new CTextBox('ipmi_authtype_name', ipmiAuthTypes($data['ipmi_authtype']), true))
 			->setWidth(ZBX_TEXTAREA_SMALL_WIDTH),
@@ -658,43 +585,48 @@ else {
 		new CVar('ipmi_privilege', $data['ipmi_privilege'])
 	];
 }
+else {
+	$cmbIPMIAuthtype = new CListBox('ipmi_authtype', $data['ipmi_authtype'], 7, null, ipmiAuthTypes());
+	$cmbIPMIPrivilege = new CListBox('ipmi_privilege', $data['ipmi_privilege'], 5, null, ipmiPrivileges());
+}
 
 $divTabs->addTab('ipmiTab', _('IPMI'),
 	(new CFormList())
 		->addRow(_('Authentication algorithm'), $cmbIPMIAuthtype)
 		->addRow(_('Privilege level'), $cmbIPMIPrivilege)
 		->addRow(_('Username'),
-			(new CTextBox('ipmi_username', $data['ipmi_username'], ($data['flags'] == ZBX_FLAG_DISCOVERY_CREATED)))
+			(new CTextBox('ipmi_username', $data['ipmi_username'], $data['readonly']))
 				->setWidth(ZBX_TEXTAREA_SMALL_WIDTH)
 		)
 		->addRow(_('Password'),
-			(new CTextBox('ipmi_password', $data['ipmi_password'], ($data['flags'] == ZBX_FLAG_DISCOVERY_CREATED)))
+			(new CTextBox('ipmi_password', $data['ipmi_password'], $data['readonly']))
 				->setWidth(ZBX_TEXTAREA_SMALL_WIDTH)
 		)
 );
 
-/*
- * Tags
- */
-if ($data['flags'] == ZBX_FLAG_DISCOVERY_NORMAL) {
-	$tags_view = new CView('configuration.tags.tab', [
+// tags
+$divTabs->addTab('tags-tab', _('Tags'), new CPartial('configuration.tags.tab', [
 		'source' => 'host',
 		'tags' => $data['tags'],
 		'readonly' => false
-	]);
-	$divTabs->addTab('tags-tab', _('Tags'), $tags_view->render());
-}
+	]), TAB_INDICATOR_TAGS
+);
 
-/*
- * Macros
- */
-$macrosView = new CView('hostmacros', [
-	'macros' => $data['macros'],
-	'show_inherited_macros' => $data['show_inherited_macros'],
-	'is_template' => false,
-	'readonly' => ($data['flags'] == ZBX_FLAG_DISCOVERY_CREATED)
-]);
-$divTabs->addTab('macroTab', _('Macros'), $macrosView->render());
+// macros
+$tmpl = $data['show_inherited_macros'] ? 'hostmacros.inherited.list.html' : 'hostmacros.list.html';
+$divTabs->addTab('macroTab', _('Macros'),
+	(new CFormList('macrosFormList'))
+		->addRow(null, (new CRadioButtonList('show_inherited_macros', (int) $data['show_inherited_macros']))
+			->addValue(_('Host macros'), 0)
+			->addValue(_('Inherited and host macros'), 1)
+			->setModern(true)
+		)
+		->addRow(null, new CPartial($tmpl, [
+			'macros' => $data['macros'],
+			'readonly' => $data['readonly']
+		]), 'macros_container'),
+	TAB_INDICATOR_MACROS
+);
 
 $inventoryFormList = new CFormList('inventorylist');
 
@@ -703,10 +635,10 @@ $inventoryFormList->addRow(null,
 		->addValue(_('Disabled'), HOST_INVENTORY_DISABLED)
 		->addValue(_('Manual'), HOST_INVENTORY_MANUAL)
 		->addValue(_('Automatic'), HOST_INVENTORY_AUTOMATIC)
-		->setEnabled($data['flags'] != ZBX_FLAG_DISCOVERY_CREATED)
+		->setEnabled(!$data['readonly'])
 		->setModern(true)
 );
-if ($data['flags'] == ZBX_FLAG_DISCOVERY_CREATED) {
+if ($data['readonly']) {
 	$inventoryFormList->addVar('inventory_mode', $data['inventory_mode']);
 }
 
@@ -739,7 +671,7 @@ foreach ($hostInventoryFields as $inventoryNo => $inventoryInfo) {
 		$name = $data['inventory_items'][$inventoryNo]['name_expanded'];
 
 		$link = (new CLink($name, 'items.php?form=update&itemid='.$data['inventory_items'][$inventoryNo]['itemid']))
-			->setTitle(_s('This field is automatically populated by item "%s".', $name));
+			->setTitle(_s('This field is automatically populated by item "%1$s".', $name));
 
 		$inventory_item = (new CSpan([' &larr; ', $link]))->addClass('populating_item');
 		if ($data['inventory_mode'] != HOST_INVENTORY_AUTOMATIC) {
@@ -760,7 +692,7 @@ foreach ($hostInventoryFields as $inventoryNo => $inventoryInfo) {
 	$inventoryFormList->addRow($inventoryInfo['title'], [$input, $inventory_item]);
 }
 
-$divTabs->addTab('inventoryTab', _('Inventory'), $inventoryFormList);
+$divTabs->addTab('inventoryTab', _('Inventory'), $inventoryFormList, TAB_INDICATOR_INVENTORY);
 
 // Encryption form list.
 $encryption_form_list = (new CFormList('encryption'))
@@ -770,46 +702,46 @@ $encryption_form_list = (new CFormList('encryption'))
 			->addValue(_('PSK'), HOST_ENCRYPTION_PSK)
 			->addValue(_('Certificate'), HOST_ENCRYPTION_CERTIFICATE)
 			->setModern(true)
-			->setEnabled($data['flags'] != ZBX_FLAG_DISCOVERY_CREATED)
+			->setEnabled(!$data['readonly'])
 	)
 	->addRow(_('Connections from host'),
 		(new CList())
 			->addClass(ZBX_STYLE_LIST_CHECK_RADIO)
 			->addItem((new CCheckBox('tls_in_none'))
 				->setLabel(_('No encryption'))
-				->setEnabled($data['flags'] != ZBX_FLAG_DISCOVERY_CREATED)
+				->setEnabled(!$data['readonly'])
 			)
 			->addItem((new CCheckBox('tls_in_psk'))
 				->setLabel(_('PSK'))
-				->setEnabled($data['flags'] != ZBX_FLAG_DISCOVERY_CREATED)
+				->setEnabled(!$data['readonly'])
 			)
 			->addItem((new CCheckBox('tls_in_cert'))
 				->setLabel(_('Certificate'))
-				->setEnabled($data['flags'] != ZBX_FLAG_DISCOVERY_CREATED)
+				->setEnabled(!$data['readonly'])
 			)
 	)
 	->addRow(
 		(new CLabel(_('PSK identity'), 'tls_psk_identity'))->setAsteriskMark(),
-		(new CTextBox('tls_psk_identity', $data['tls_psk_identity'], $data['flags'] == ZBX_FLAG_DISCOVERY_CREATED, 128))
+		(new CTextBox('tls_psk_identity', $data['tls_psk_identity'], $data['readonly'], 128))
 			->setWidth(ZBX_TEXTAREA_BIG_WIDTH)
 			->setAriaRequired()
 	)
 	->addRow(
 		(new CLabel(_('PSK'), 'tls_psk'))->setAsteriskMark(),
-		(new CTextBox('tls_psk', $data['tls_psk'], $data['flags'] == ZBX_FLAG_DISCOVERY_CREATED, 512))
+		(new CTextBox('tls_psk', $data['tls_psk'], $data['readonly'], 512))
 			->setWidth(ZBX_TEXTAREA_BIG_WIDTH)
 			->setAriaRequired()
 	)
 	->addRow(_('Issuer'),
-		(new CTextBox('tls_issuer', $data['tls_issuer'], $data['flags'] == ZBX_FLAG_DISCOVERY_CREATED, 1024))
+		(new CTextBox('tls_issuer', $data['tls_issuer'], $data['readonly'], 1024))
 			->setWidth(ZBX_TEXTAREA_BIG_WIDTH)
 	)
 	->addRow(_x('Subject', 'encryption certificate'),
-		(new CTextBox('tls_subject', $data['tls_subject'], $data['flags'] == ZBX_FLAG_DISCOVERY_CREATED, 1024))
+		(new CTextBox('tls_subject', $data['tls_subject'], $data['readonly'], 1024))
 			->setWidth(ZBX_TEXTAREA_BIG_WIDTH)
 	);
 
-$divTabs->addTab('encryptionTab', _('Encryption'), $encryption_form_list);
+$divTabs->addTab('encryptionTab', _('Encryption'), $encryption_form_list, TAB_INDICATOR_ENCRYPTION);
 
 /*
  * footer
@@ -821,20 +753,19 @@ if ($data['hostid'] != 0) {
 		[
 			new CSubmit('clone', _('Clone')),
 			new CSubmit('full_clone', _('Full clone')),
-			new CButtonDelete(_('Delete selected host?'), url_param('form').url_param('hostid').url_param('groupid')),
-			new CButtonCancel(url_param('groupid'))
+			new CButtonDelete(_('Delete selected host?'), url_param('form').url_param('hostid')),
+			new CButtonCancel()
 		]
 	));
 }
 else {
 	$divTabs->setFooter(makeFormFooter(
 		new CSubmit('add', _('Add')),
-		[new CButtonCancel(url_param('groupid'))]
+		[new CButtonCancel()]
 	));
 }
 
 $frmHost->addItem($divTabs);
-
 $widget->addItem($frmHost);
 
-return $widget;
+$widget->show();
