@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2019 Zabbix SIA
+** Copyright (C) 2001-2020 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -19,6 +19,10 @@
 **/
 
 
+/**
+ * @var CView $this
+ */
+
 $widget = (new CWidget())->setTitle(_('Items'));
 
 if ($data['hostid'] != 0) {
@@ -45,46 +49,22 @@ $item_form_list = (new CFormList('item-form-list'))
 	);
 
 // Append hosts to item form list.
-if ($data['displayInterfaces']) {
-	$interfaces_combo_box = new CComboBox('interfaceid', $data['interfaceid']);
-	$interfaces_combo_box->addItem(new CComboItem(0, '', false, false));
-
-	// Set up interface groups sorted by priority.
-	$interface_types = zbx_objectValues($data['hosts']['interfaces'], 'type');
-	$interface_groups = [];
-	foreach ([INTERFACE_TYPE_AGENT, INTERFACE_TYPE_SNMP, INTERFACE_TYPE_JMX, INTERFACE_TYPE_IPMI] as $interface_type) {
-		if (in_array($interface_type, $interface_types)) {
-			$interface_groups[$interface_type] = new COptGroup(interfaceType2str($interface_type));
-		}
-	}
-
-	// add interfaces to groups
-	foreach ($data['hosts']['interfaces'] as $interface) {
-		$option = new CComboItem(
-			$interface['interfaceid'],
-			$interface['useip']
-				? $interface['ip'].' : '.$interface['port']
-				: $interface['dns'].' : '.$interface['port'],
-			($interface['interfaceid'] == $data['interfaceid'])
-		);
-		$option->setAttribute('data-interfacetype', $interface['type']);
-		$interface_groups[$interface['type']]->addItem($option);
-	}
-	foreach ($interface_groups as $interface_group) {
-		$interfaces_combo_box->addItem($interface_group);
-	}
-
-	$span = (new CSpan(_('No interface found')))
-		->addClass(ZBX_STYLE_RED)
-		->setId('interface_not_defined')
-		->setAttribute('style', 'display: none;');
-
+if ($data['display_interfaces']) {
 	$item_form_list->addRow(
 		(new CVisibilityBox('visible[interfaceid]', 'interfaceDiv', _('Original')))
 			->setLabel(_('Host interface'))
 			->setChecked(isset($data['visible']['interfaceid']))
 			->setAttribute('data-multiple-interface-types', $data['multiple_interface_types']),
-		(new CDiv([$interfaces_combo_box, $span]))->setId('interfaceDiv'),
+		(new CDiv([
+			getInterfaceSelect($data['hosts']['interfaces'])
+				->setId('interface-select')
+				->setValue($data['interfaceid'])
+				->addClass(ZBX_STYLE_ZSELECT_HOST_INTERFACE),
+			(new CSpan(_('No interface found')))
+				->addClass(ZBX_STYLE_RED)
+				->setId('interface_not_defined')
+				->setAttribute('style', 'display: none;')
+		]))->setId('interfaceDiv'),
 		'interface_row'
 	);
 	$form->addVar('selectedInterfaceId', $data['interfaceid']);
@@ -119,6 +99,12 @@ $item_form_list
 				->setModern(true)
 		))->setId('post_type_container')
 	)
+	->addRow(
+		(new CVisibilityBox('visible[timeout]', 'timeout', _('Original')))
+			->setLabel(_('Timeout'))
+			->setChecked(array_key_exists('timeout', $data['visible'])),
+		(new CTextBox('timeout', $data['timeout']))->setWidth(ZBX_TEXTAREA_SMALL_WIDTH)
+	)
 	// Append ITEM_TYPE_HTTPAGENT Request body.
 	->addRow(
 		(new CVisibilityBox('visible[posts]', 'posts', _('Original')))
@@ -139,7 +125,7 @@ else {
 	$headers_data[] = ['name' => '', 'value' => ''];
 }
 $headers = (new CTag('script', true))->setAttribute('type', 'text/json');
-$headers->items = [CJs::encodeJson($headers_data)];
+$headers->items = [json_encode($headers_data)];
 
 $item_form_list
 	->addRow(
@@ -177,86 +163,6 @@ $item_form_list
 			->setAttribute('data-sortable-pairs-table', '1')
 			->setAttribute('style', 'min-width: '.ZBX_TEXTAREA_BIG_WIDTH.'px;')
 	)
-	// Append SNMP community to item form list.
-	->addRow(
-		(new CVisibilityBox('visible[snmp_community]', 'snmp_community', _('Original')))
-			->setLabel(_('SNMP community'))
-			->setChecked(isset($data['visible']['snmp_community'])),
-		(new CTextBox('snmp_community', $data['snmp_community']))->setWidth(ZBX_TEXTAREA_STANDARD_WIDTH)
-	)
-	// Append SNMPv3 contextname to item form list.
-	->addRow(
-		(new CVisibilityBox('visible[snmpv3_contextname]', 'snmpv3_contextname', _('Original')))
-			->setLabel(_('Context name'))
-			->setChecked(isset($data['visible']['snmpv3_contextname'])),
-		(new CTextBox('snmpv3_contextname', $data['snmpv3_contextname']))->setWidth(ZBX_TEXTAREA_STANDARD_WIDTH)
-	)
-	// Append SNMPv3 securityname to item form list.
-	->addRow(
-		(new CVisibilityBox('visible[snmpv3_securityname]', 'snmpv3_securityname', _('Original')))
-			->setLabel(_('Security name'))
-			->setChecked(isset($data['visible']['snmpv3_securityname'])),
-		(new CTextBox('snmpv3_securityname', $data['snmpv3_securityname']))->setWidth(ZBX_TEXTAREA_STANDARD_WIDTH)
-	)
-	// Append SNMPv3 securitylevel to item form list.
-	->addRow(
-		(new CVisibilityBox('visible[snmpv3_securitylevel]', 'snmpv3_securitylevel', _('Original')))
-			->setLabel(_('Security level'))
-			->setChecked(isset($data['visible']['snmpv3_securitylevel'])),
-		new CComboBox('snmpv3_securitylevel', $data['snmpv3_securitylevel'], null, [
-			ITEM_SNMPV3_SECURITYLEVEL_NOAUTHNOPRIV => 'noAuthNoPriv',
-			ITEM_SNMPV3_SECURITYLEVEL_AUTHNOPRIV => 'authNoPriv',
-			ITEM_SNMPV3_SECURITYLEVEL_AUTHPRIV => 'authPriv'
-		])
-	)
-	// Append SNMPv3 authprotocol to item form list.
-	->addRow(
-		(new CVisibilityBox('visible[snmpv3_authprotocol]', 'authprotocol_div', _('Original')))
-			->setLabel(_('Authentication protocol'))
-			->setChecked(isset($data['visible']['snmpv3_authprotocol'])),
-		(new CDiv(
-			(new CRadioButtonList('snmpv3_authprotocol', (int) $data['snmpv3_authprotocol']))
-				->addValue(_('MD5'), ITEM_AUTHPROTOCOL_MD5)
-				->addValue(_('SHA'), ITEM_AUTHPROTOCOL_SHA)
-				->setModern(true)
-		))->setId('authprotocol_div')
-	)
-	// Append SNMPv3 authpassphrase to item form list.
-	->addRow(
-		(new CVisibilityBox('visible[snmpv3_authpassphrase]', 'snmpv3_authpassphrase', _('Original')))
-			->setLabel(_('Authentication passphrase'))
-			->setChecked(isset($data['visible']['snmpv3_authpassphrase'])),
-		(new CTextBox('snmpv3_authpassphrase', $data['snmpv3_authpassphrase']))
-			->setWidth(ZBX_TEXTAREA_STANDARD_WIDTH)
-	)
-	// Append SNMPv3 privprotocol to item form list.
-	->addRow(
-		(new CVisibilityBox('visible[snmpv3_privprotocol]', 'privprotocol_div', _('Original')))
-			->setLabel(_('Privacy protocol'))
-			->setChecked(isset($data['visible']['snmpv3_privprotocol'])),
-		(new CDiv(
-			(new CRadioButtonList('snmpv3_privprotocol', (int) $data['snmpv3_privprotocol']))
-				->addValue(_('DES'), ITEM_PRIVPROTOCOL_DES)
-				->addValue(_('AES'), ITEM_PRIVPROTOCOL_AES)
-				->setModern(true)
-		))
-			->setId('privprotocol_div')
-	)
-	// Append SNMPv3 privpassphrase to item form list.
-	->addRow(
-		(new CVisibilityBox('visible[snmpv3_privpassphrase]', 'snmpv3_privpassphrase', _('Original')))
-			->setLabel(_('Privacy passphrase'))
-			->setChecked(isset($data['visible']['snmpv3_privpassphrase'])),
-		(new CTextBox('snmpv3_privpassphrase', $data['snmpv3_privpassphrase']))
-			->setWidth(ZBX_TEXTAREA_STANDARD_WIDTH)
-	)
-	// Append port to item form list.
-	->addRow(
-		(new CVisibilityBox('visible[port]', 'port', _('Original')))
-			->setLabel(_('Port'))
-			->setChecked(isset($data['visible']['port'])),
-		(new CTextBox('port', $data['port']))->setWidth(ZBX_TEXTAREA_SMALL_WIDTH)
-	)
 	// Append value type to item form list.
 	->addRow(
 		(new CVisibilityBox('visible[value_type]', 'value_type', _('Original')))
@@ -467,7 +373,10 @@ $item_form_list
 			->setLabel(_('Show value'))
 			->setChecked(isset($data['visible']['valuemapid'])),
 		(new CDiv([$value_maps_combo_box, SPACE,
-			(new CLink(_('show value mappings'), 'adm.valuemapping.php'))->setAttribute('target', '_blank')
+			(new CLink(_('show value mappings'), (new CUrl('zabbix.php'))
+				->setArgument('action', 'valuemap.list')
+				->getUrl()
+			))->setAttribute('target', '_blank')
 		]))->setId('valuemap')
 	)
 	->addRow(
@@ -613,6 +522,13 @@ $form->addItem($tabs);
 
 $widget->addItem($form);
 
+$interface_ids_by_types = [];
+if ($data['display_interfaces']) {
+	foreach ($data['hosts']['interfaces'] as $interface) {
+		$interface_ids_by_types[$interface['type']][] = $interface['interfaceid'];
+	}
+}
+
 require_once dirname(__FILE__).'/js/configuration.item.massupdate.js.php';
 
-return $widget;
+$widget->show();
