@@ -30,6 +30,7 @@ if (!empty($data['hostid'])) {
 }
 
 $form = (new CForm())
+	->setId('item-prototype-form')
 	->setName('itemForm')
 	->setAttribute('aria-labeledby', ZBX_STYLE_PAGE_TITLE)
 	->addVar('form', $data['form'])
@@ -110,7 +111,7 @@ $form_list
 		'url_row'
 	);
 
-// Append ITEM_TYPE_HTTPAGENT Query fields to form list.
+// Prepare ITEM_TYPE_HTTPAGENT query fields.
 $query_fields_data = [];
 
 if (is_array($data['query_fields']) && $data['query_fields']) {
@@ -124,7 +125,56 @@ elseif (!$readonly) {
 $query_fields = (new CTag('script', true))->setAttribute('type', 'text/json');
 $query_fields->items = [json_encode($query_fields_data)];
 
+// Prepare ITEM_TYPE_SCRIPT parameters.
+$parameters_data = [];
+if ($data['parameters']) {
+	$parameters_data = $data['parameters'];
+}
+elseif (!$readonly) {
+	$parameters_data[] = ['name' => '', 'value' => ''];
+}
+
+$parameters_table = (new CTable())
+	->setId('parameters_table')
+	->setHeader([
+		(new CColHeader(_('Name')))->setWidth('50%'),
+		(new CColHeader(_('Value')))->setWidth('50%'),
+		_('Action')
+	])
+	->setAttribute('style', 'width: 100%;');
+
+if ($parameters_data) {
+	foreach ($parameters_data as $parameter) {
+		$parameters_table->addRow([
+			(new CTextBox('parameters[name][]', $parameter['name'], $readonly,
+				DB::getFieldLength('item_parameter', 'name'))
+			)
+				->setAttribute('style', 'width: 100%;')
+				->removeId(),
+			(new CTextBox('parameters[value][]', $parameter['value'], $readonly,
+				DB::getFieldLength('item_parameter', 'value'))
+			)
+				->setAttribute('style', 'width: 100%;')
+				->removeId(),
+			(new CButton('', _('Remove')))
+				->removeId()
+				->onClick('jQuery(this).closest("tr").remove()')
+				->addClass(ZBX_STYLE_BTN_LINK)
+				->addClass('element-table-remove')
+				->setEnabled(!$readonly)
+		]);
+	}
+}
+
+$parameters_table->addRow([
+	(new CButton('parameter_add', _('Add')))
+		->addClass(ZBX_STYLE_BTN_LINK)
+		->addClass('element-table-add')
+		->setEnabled(!$readonly)
+]);
+
 $form_list
+	// Append ITEM_TYPE_HTTPAGENT Query fields to form list.
 	->addRow(
 		new CLabel(_('Query fields'), 'query_fields_pairs'),
 		(new CDiv([
@@ -164,6 +214,48 @@ $form_list
 			->setAttribute('style', 'min-width: '.ZBX_TEXTAREA_BIG_WIDTH . 'px;'),
 		'query_fields_row'
 	)
+	// Append ITEM_TYPE_SCRIPT parameters to form list.
+	->addItem(
+		(new CTag('script', true))
+			->setId('parameters_table_row')
+			->setAttribute('type', 'text/x-jquery-tmpl')
+			->addItem(
+				(new CRow([
+					(new CTextBox('parameters[name][]', '', false, DB::getFieldLength('item_parameter', 'name')))
+						->setAttribute('style', 'width: 100%;')
+						->removeId(),
+					(new CTextBox('parameters[value][]', '', false, DB::getFieldLength('item_parameter', 'value')))
+						->setAttribute('style', 'width: 100%;')
+						->removeId(),
+					(new CButton('', _('Remove')))
+						->removeId()
+						->onClick('jQuery(this).closest("tr").remove()')
+						->addClass(ZBX_STYLE_BTN_LINK)
+						->addClass('element-table-remove')
+				]))
+			)
+	)
+	->addRow(
+		new CLabel(_('Parameters'), $parameters_table->getId()),
+		(new CDiv($parameters_table))
+			->addClass(ZBX_STYLE_TABLE_FORMS_SEPARATOR)
+			->setAttribute('style', 'min-width: '.ZBX_TEXTAREA_STANDARD_WIDTH.'px;'),
+		'parameters_row'
+	)
+	->addRow((new CLabel(_('Script'), 'script'))->setAsteriskMark(),
+		(new CMultilineInput('script', $data['params'], [
+			'title' => _('JavaScript'),
+			'placeholder' => _('script'),
+			'placeholder_textarea' => 'return value',
+			'grow' => 'auto',
+			'rows' => 0,
+			'maxlength' => DB::getFieldLength('items', 'params'),
+			'readonly' => $readonly
+		]))
+			->setWidth(ZBX_TEXTAREA_STANDARD_WIDTH)
+			->setAriaRequired(),
+		'script_row'
+	)
 	// Append ITEM_TYPE_HTTPAGENT Request type to form list.
 	->addRow(
 		new CLabel(_('Request type'), 'request_method'),
@@ -178,10 +270,12 @@ $form_list
 		],
 		'request_method_row'
 	)
-	// Append ITEM_TYPE_HTTPAGENT Timeout field to form list.
+	// Append ITEM_TYPE_HTTPAGENT and ITEM_TYPE_SCRIPT timeout field to form list.
 	->addRow(
-		new CLabel(_('Timeout'), 'timeout'),
-		(new CTextBox('timeout', $data['timeout'], $readonly))->setWidth(ZBX_TEXTAREA_SMALL_WIDTH),
+		(new CLabel(_('Timeout'), 'timeout'))->setAsteriskMark(),
+		(new CTextBox('timeout', $data['timeout'], $readonly))
+			->setWidth(ZBX_TEXTAREA_SMALL_WIDTH)
+			->setAriaRequired(),
 		'timeout_row'
 	)
 	// Append ITEM_TYPE_HTTPAGENT Request body type to form list.
@@ -303,12 +397,8 @@ $form_list
 		new CLabel(_('HTTP authentication'), 'http_authtype'),
 		[
 			$readonly ? new CVar('http_authtype', $data['http_authtype']) : null,
-			(new CComboBox($readonly ? '' : 'http_authtype', $data['http_authtype'], null, [
-				HTTPTEST_AUTH_NONE => _('None'),
-				HTTPTEST_AUTH_BASIC => _('Basic'),
-				HTTPTEST_AUTH_NTLM => _('NTLM'),
-				HTTPTEST_AUTH_KERBEROS => _('Kerberos')
-			]))->setEnabled(!$readonly)
+			(new CComboBox($readonly ? '' : 'http_authtype', $data['http_authtype'], null, httptest_authentications()))
+				->setEnabled(!$readonly)
 		],
 		'http_authtype_row'
 	)
@@ -419,41 +509,23 @@ $form_list->addRow(
 );
 
 // append interfaces to form list
-if (!empty($data['interfaces'])) {
-	$interfacesComboBox = (new CComboBox('interfaceid', $data['interfaceid']))->setAriaRequired();
+$select_interface = getInterfaceSelect($data['interfaces'])
+	->setId('interface-select')
+	->setValue($data['interfaceid'])
+	->addClass(ZBX_STYLE_ZSELECT_HOST_INTERFACE)
+	->setFocusableElementId('interfaceid')
+	->setAriaRequired();
 
-	// Set up interface groups sorted by priority.
-	$interface_types = zbx_objectValues($data['interfaces'], 'type');
-	$interface_groups = [];
-	foreach ([INTERFACE_TYPE_AGENT, INTERFACE_TYPE_SNMP, INTERFACE_TYPE_JMX, INTERFACE_TYPE_IPMI] as $interface_type) {
-		if (in_array($interface_type, $interface_types)) {
-			$interface_groups[$interface_type] = new COptGroup(interfaceType2str($interface_type));
-		}
-	}
-
-	// add interfaces to groups
-	foreach ($data['interfaces'] as $interface) {
-		$option = new CComboItem($interface['interfaceid'],
-			$interface['useip']
-				? $interface['ip'].' : '.$interface['port']
-				: $interface['dns'].' : '.$interface['port'],
-			($interface['interfaceid'] == $data['interfaceid'])
-		);
-		$option->setAttribute('data-interfacetype', $interface['type']);
-		$interface_groups[$interface['type']]->addItem($option);
-	}
-	foreach ($interface_groups as $interface_group) {
-		$interfacesComboBox->addItem($interface_group);
-	}
-
-	$span = (new CSpan(_('No interface found')))
-		->addClass(ZBX_STYLE_RED)
-		->setId('interface_not_defined')
-		->setAttribute('style', 'display: none;');
-
-	$form_list->addRow((new CLabel(_('Host interface'), 'interfaceid'))->setAsteriskMark(),
-		[$interfacesComboBox, $span], 'interface_row'
-	);
+if ($data['display_interfaces']) {
+	$form_list->addRow(
+		(new CLabel(_('Host interface'), $select_interface->getFocusableElementId()))->setAsteriskMark(),
+		[
+			$select_interface,
+			(new CSpan(_('No interface found')))
+				->addClass(ZBX_STYLE_RED)
+				->setId('interface_not_defined')
+				->setAttribute('style', 'display: none;')
+		], 'interface_row');
 	$form->addVar('selectedInterfaceId', $data['interfaceid']);
 }
 
@@ -727,7 +799,8 @@ $tab = (new CTabView())
 		(new CFormList('item_preproc_list'))
 			->addRow(_('Preprocessing steps'),
 				getItemPreprocessing($form, $data['preprocessing'], $readonly, $data['preprocessing_types'])
-			)
+			),
+		TAB_INDICATOR_PREPROCESSING
 	);
 
 if (!hasRequest('form_refresh')) {
