@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2019 Zabbix SIA
+** Copyright (C) 2001-2020 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -19,6 +19,10 @@
 **/
 
 
+/**
+ * @var CView $this
+ */
+
 $widget = (new CWidget())
 	->setTitle(_('Item prototypes'))
 	->addItem(get_header_host_table('items', $data['hostid'], $data['parent_discoveryid']));
@@ -42,45 +46,21 @@ $item_form_list = (new CFormList('item-form-list'))
 
 // Add interfaces if mass updating item prototypes on host level.
 if ($data['display_interfaces']) {
-	$interfaces_combo_box = new CComboBox('interfaceid', $data['interfaceid']);
-	$interfaces_combo_box->addItem(new CComboItem(0, '', false, false));
-
-	// Set up interface groups sorted by priority.
-	$interface_types = zbx_objectValues($data['hosts']['interfaces'], 'type');
-	$interface_groups = [];
-
-	foreach ([INTERFACE_TYPE_AGENT, INTERFACE_TYPE_SNMP, INTERFACE_TYPE_JMX, INTERFACE_TYPE_IPMI] as $interface_type) {
-		if (in_array($interface_type, $interface_types)) {
-			$interface_groups[$interface_type] = new COptGroup(interfaceType2str($interface_type));
-		}
-	}
-
-	// Add interfaces to groups.
-	foreach ($data['hosts']['interfaces'] as $interface) {
-		$option = new CComboItem(
-			$interface['interfaceid'],
-			$interface['useip']
-				? $interface['ip'].' : '.$interface['port']
-				: $interface['dns'].' : '.$interface['port'],
-			($interface['interfaceid'] == $data['interfaceid'])
-		);
-		$option->setAttribute('data-interfacetype', $interface['type']);
-		$interface_groups[$interface['type']]->addItem($option);
-	}
-	foreach ($interface_groups as $interface_group) {
-		$interfaces_combo_box->addItem($interface_group);
-	}
-
-	$span = (new CSpan(_('No interface found')))
-		->addClass(ZBX_STYLE_RED)
-		->setId('interface_not_defined')
-		->addStyle('display: none;');
 	$item_form_list->addRow(
 		(new CVisibilityBox('visible[interfaceid]', 'interfaceDiv', _('Original')))
 			->setLabel(_('Host interface'))
 			->setChecked(array_key_exists('interfaceid', $data['visible']))
 			->setAttribute('data-multiple-interface-types', $data['multiple_interface_types']),
-		(new CDiv([$interfaces_combo_box, $span]))->setId('interfaceDiv'),
+		(new CDiv([
+			getInterfaceSelect($data['hosts']['interfaces'])
+				->setId('interface-select')
+				->setValue($data['interfaceid'])
+				->addClass(ZBX_STYLE_ZSELECT_HOST_INTERFACE),
+			(new CSpan(_('No interface found')))
+				->addClass(ZBX_STYLE_RED)
+				->setId('interface_not_defined')
+				->addStyle('display: none;')
+		]))->setId('interfaceDiv'),
 		'interface_row'
 	);
 	$form->addVar('selectedInterfaceId', $data['interfaceid']);
@@ -115,6 +95,12 @@ $item_form_list
 				->setModern(true)
 		))->setId('post_type_container')
 	)
+	->addRow(
+		(new CVisibilityBox('visible[timeout]', 'timeout', _('Original')))
+			->setLabel(_('Timeout'))
+			->setChecked(array_key_exists('timeout', $data['visible'])),
+		(new CTextBox('timeout', $data['timeout']))->setWidth(ZBX_TEXTAREA_SMALL_WIDTH)
+	)
 	// Append "Request body" field (optional) for item prototype type "ITEM_TYPE_HTTPAGENT".
 	->addRow(
 		(new CVisibilityBox('visible[posts]', 'posts', _('Original')))
@@ -135,7 +121,7 @@ else {
 	$headers_data[] = ['name' => '', 'value' => ''];
 }
 $headers = (new CTag('script', true))->setAttribute('type', 'text/json');
-$headers->items = [CJs::encodeJson($headers_data)];
+$headers->items = [json_encode($headers_data)];
 
 $item_form_list
 	->addRow(
@@ -173,85 +159,6 @@ $item_form_list
 			->setAttribute('data-sortable-pairs-table', '1')
 			->addStyle('min-width: '.ZBX_TEXTAREA_BIG_WIDTH.'px;')
 	)
-	// Append SNMP community to form list.
-	->addRow(
-		(new CVisibilityBox('visible[snmp_community]', 'snmp_community', _('Original')))
-			->setLabel(_('SNMP community'))
-			->setChecked(array_key_exists('snmp_community', $data['visible'])),
-		(new CTextBox('snmp_community', $data['snmp_community']))->setWidth(ZBX_TEXTAREA_STANDARD_WIDTH)
-	)
-	// Append SNMPv3 contextname to form list.
-	->addRow(
-		(new CVisibilityBox('visible[snmpv3_contextname]', 'snmpv3_contextname', _('Original')))
-			->setLabel(_('Context name'))
-			->setChecked(array_key_exists('snmpv3_contextname', $data['visible'])),
-		(new CTextBox('snmpv3_contextname', $data['snmpv3_contextname']))->setWidth(ZBX_TEXTAREA_STANDARD_WIDTH)
-	)
-	// Append SNMPv3 securityname to form list.
-	->addRow(
-		(new CVisibilityBox('visible[snmpv3_securityname]', 'snmpv3_securityname', _('Original')))
-			->setLabel(_('Security name'))
-			->setChecked(array_key_exists('snmpv3_securityname', $data['visible'])),
-		(new CTextBox('snmpv3_securityname', $data['snmpv3_securityname']))->setWidth(ZBX_TEXTAREA_STANDARD_WIDTH)
-	)
-	// Append SNMPv3 securitylevel to form list.
-	->addRow(
-		(new CVisibilityBox('visible[snmpv3_securitylevel]', 'snmpv3_securitylevel', _('Original')))
-			->setLabel(_('Security level'))
-			->setChecked(array_key_exists('snmpv3_securitylevel', $data['visible'])),
-		new CComboBox('snmpv3_securitylevel', $data['snmpv3_securitylevel'], null, [
-			ITEM_SNMPV3_SECURITYLEVEL_NOAUTHNOPRIV => 'noAuthNoPriv',
-			ITEM_SNMPV3_SECURITYLEVEL_AUTHNOPRIV => 'authNoPriv',
-			ITEM_SNMPV3_SECURITYLEVEL_AUTHPRIV => 'authPriv'
-		])
-	)
-	// Append SNMPv3 authprotocol to form list.
-	->addRow(
-		(new CVisibilityBox('visible[snmpv3_authprotocol]', 'authprotocol_div', _('Original')))
-			->setLabel(_('Authentication protocol'))
-			->setChecked(array_key_exists('snmpv3_authprotocol', $data['visible'])),
-		(new CDiv(
-			(new CRadioButtonList('snmpv3_authprotocol', (int) $data['snmpv3_authprotocol']))
-				->addValue(_('MD5'), ITEM_AUTHPROTOCOL_MD5)
-				->addValue(_('SHA'), ITEM_AUTHPROTOCOL_SHA)
-				->setModern(true)
-		))->setId('authprotocol_div')
-	)
-	// Append SNMPv3 authpassphrase to form list.
-	->addRow(
-		(new CVisibilityBox('visible[snmpv3_authpassphrase]', 'snmpv3_authpassphrase', _('Original')))
-			->setLabel(_('Authentication passphrase'))
-			->setChecked(array_key_exists('snmpv3_authpassphrase', $data['visible'])),
-		(new CTextBox('snmpv3_authpassphrase', $data['snmpv3_authpassphrase']))
-			->setWidth(ZBX_TEXTAREA_STANDARD_WIDTH)
-	)
-	// Append SNMPv3 privprotocol to form list.
-	->addRow(
-		(new CVisibilityBox('visible[snmpv3_privprotocol]', 'privprotocol_div', _('Original')))
-			->setLabel(_('Privacy protocol'))
-			->setChecked(array_key_exists('snmpv3_privprotocol', $data['visible'])),
-		(new CDiv(
-			(new CRadioButtonList('snmpv3_privprotocol', (int) $data['snmpv3_privprotocol']))
-				->addValue(_('DES'), ITEM_PRIVPROTOCOL_DES)
-				->addValue(_('AES'), ITEM_PRIVPROTOCOL_AES)
-				->setModern(true)
-		))->setId('privprotocol_div')
-	)
-	// Append SNMPv3 privpassphrase to form list.
-	->addRow(
-		(new CVisibilityBox('visible[snmpv3_privpassphrase]', 'snmpv3_privpassphrase', _('Original')))
-			->setLabel(_('Privacy passphrase'))
-			->setChecked(array_key_exists('snmpv3_privpassphrase', $data['visible'])),
-		(new CTextBox('snmpv3_privpassphrase', $data['snmpv3_privpassphrase']))
-			->setWidth(ZBX_TEXTAREA_STANDARD_WIDTH)
-	)
-	// Append port to form list.
-	->addRow(
-		(new CVisibilityBox('visible[port]', 'port', _('Original')))
-			->setLabel(_('Port'))
-			->setChecked(array_key_exists('port', $data['visible'])),
-		(new CTextBox('port', $data['port']))->setWidth(ZBX_TEXTAREA_SMALL_WIDTH)
-	)
 	// Append value type to form list.
 	->addRow(
 		(new CVisibilityBox('visible[value_type]', 'value_type', _('Original')))
@@ -445,6 +352,15 @@ $item_form_list
 			->setChecked(array_key_exists('status', $data['visible'])),
 		$status_combo_box
 	)
+	->addRow(
+		(new CVisibilityBox('visible[discover]', 'discover', _('Original')))
+			->setLabel(_('Discover'))
+			->setChecked(array_key_exists('discover', $data['visible'])),
+		(new CRadioButtonList('discover', (int) $data['discover']))
+			->addValue(_('Yes'), ZBX_PROTOTYPE_DISCOVER)
+			->addValue(_('No'), ZBX_PROTOTYPE_NO_DISCOVER)
+			->setModern(true)
+	)
 	// Append logtime to form list.
 	->addRow(
 		(new CVisibilityBox('visible[logtimefmt]', 'logtimefmt', _('Original')))
@@ -467,7 +383,10 @@ $item_form_list
 			->setLabel(_('Show value'))
 			->setChecked(array_key_exists('valuemapid', $data['visible'])),
 		(new CDiv([$valuemaps_combo_box, ' ',
-			(new CLink(_('show value mappings'), 'adm.valuemapping.php'))->setAttribute('target', '_blank')
+			(new CLink(_('show value mappings'), (new CUrl('zabbix.php'))
+				->setArgument('action', 'valuemap.list')
+				->getUrl()
+			))->setAttribute('target', '_blank')
 		]))->setId('valuemap')
 	)
 	->addRow(
@@ -530,7 +449,7 @@ $master_item[] = (new CDiv())->addClass(ZBX_STYLE_FORM_INPUT_MARGIN);
 $master_item[] = (new CButton('button', _('Select')))
 	->addClass(ZBX_STYLE_BTN_GREY)
 	->onClick('return PopUp("popup.generic",'.
-		CJs::encodeJson([
+		json_encode([
 			'srctbl' => 'items',
 			'srcfld1' => 'itemid',
 			'srcfld2' => 'name',
@@ -546,7 +465,7 @@ $master_item[] = (new CDiv())->addClass(ZBX_STYLE_FORM_INPUT_MARGIN);
 $master_item[] = (new CButton('button', _('Select prototype')))
 	->addClass(ZBX_STYLE_BTN_GREY)
 	->onClick('return PopUp("popup.generic",'.
-		CJs::encodeJson([
+		json_encode([
 			'srctbl' => 'item_prototypes',
 			'srcfld1' => 'itemid',
 			'srcfld2' => 'name',
@@ -626,6 +545,13 @@ $form->addItem($tabs);
 
 $widget->addItem($form);
 
+$interface_ids_by_types = [];
+if ($data['display_interfaces']) {
+	foreach ($data['hosts']['interfaces'] as $interface) {
+		$interface_ids_by_types[$interface['type']][] = $interface['interfaceid'];
+	}
+}
+
 require_once dirname(__FILE__).'/js/configuration.item.massupdate.js.php';
 
-return $widget;
+$widget->show();
