@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2019 Zabbix SIA
+** Copyright (C) 2001-2020 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -19,6 +19,10 @@
 **/
 
 
+/**
+ * @var CView $this
+ */
+
 // Overview tab.
 $overviewFormList = new CFormList();
 
@@ -78,17 +82,24 @@ foreach ([INTERFACE_TYPE_AGENT, INTERFACE_TYPE_SNMP, INTERFACE_TYPE_JMX, INTERFA
 
 		foreach ($interfaces[$type] as $interface) {
 			$ifTab->addRow([
-				(new CTextBox('ip', $interface['ip'], true, 64))->setWidth(ZBX_TEXTAREA_INTERFACE_IP_WIDTH),
-				(new CTextBox('dns', $interface['dns'], true, 64))->setWidth(ZBX_TEXTAREA_INTERFACE_DNS_WIDTH),
+				(new CTextBox('ip', $interface['ip'], true, 64))
+					->setWidth(ZBX_TEXTAREA_INTERFACE_IP_WIDTH)
+					->removeId(),
+				(new CTextBox('dns', $interface['dns'], true, 64))
+					->setWidth(ZBX_TEXTAREA_INTERFACE_DNS_WIDTH)
+					->removeId(),
 				(new CRadioButtonList('useip['.$interface['interfaceid'].']', (int) $interface['useip']))
-					->addValue(_('IP'), INTERFACE_USE_IP)
-					->addValue(_('DNS'), INTERFACE_USE_DNS)
+					->addValue('IP', INTERFACE_USE_IP)
+					->addValue('DNS', INTERFACE_USE_DNS)
 					->setModern(true)
 					->setEnabled(false),
-				(new CTextBox('port', $interface['port'], true, 64))->setWidth(ZBX_TEXTAREA_INTERFACE_PORT_WIDTH),
-				(new CRadioButtonList('main['.$interface['type'].']', (int) $interface['main']))
+				(new CTextBox('port', $interface['port'], true, 64))
+					->setWidth(ZBX_TEXTAREA_INTERFACE_PORT_WIDTH)
+					->removeId(),
+				(new CRadioButtonList('main['.$interface['interfaceid'].']', (int) $interface['main']))
 					->addValue(null, INTERFACE_PRIMARY)
 					->setEnabled(false)
+					->removeId()
 			]);
 		}
 
@@ -121,35 +132,58 @@ if ($data['host']['description'] !== '') {
 // latest data
 $overviewFormList->addRow(_('Monitoring'),
 	new CHorList([
-		new CLink(_('Web'),
-			'zabbix.php?action=web.view&hostid='.$data['host']['hostid'].url_param('groupid')
-		),
-		new CLink(_('Latest data'),
-			(new CUrl('latest.php'))
-				->setArgument('form', '1')
-				->setArgument('select', '')
-				->setArgument('show_details', '1')
-				->setArgument('filter_set', 'Filter')
-				->setArgument('hostids[]', $data['host']['hostid'])
-		),
-		new CLink(_('Problems'),
-			(new CUrl('zabbix.php'))
-				->setArgument('action', 'problem.view')
+		$data['allowed_ui_hosts']
+			? new CLink(_('Web'), (new CUrl('zabbix.php'))
+				->setArgument('action', 'web.view')
 				->setArgument('filter_hostids[]', $data['host']['hostid'])
 				->setArgument('filter_set', '1')
-		),
-		new CLink(_('Graphs'), 'charts.php?hostid='.$data['host']['hostid'].url_param('groupid')),
-		new CLink(_('Screens'),
-			'host_screen.php?hostid='.$data['host']['hostid'].url_param('groupid')
-		)
+			)
+			: _('Web'),
+		$data['allowed_ui_latest_data']
+			? new CLink(_('Latest data'), (new CUrl('zabbix.php'))
+				->setArgument('action', 'latest.view')
+				->setArgument('filter_hostids[]', $data['host']['hostid'])
+				->setArgument('filter_show_details', '1')
+				->setArgument('filter_set', '1')
+			)
+			: _('Latest data'),
+		$data['allowed_ui_problems']
+			? new CLink(_('Problems'), (new CUrl('zabbix.php'))
+				->setArgument('action', 'problem.view')
+				->setArgument('filter_name', '')
+				->setArgument('hostids', [$data['host']['hostid']])
+			)
+			: _('Problems'),
+		$data['allowed_ui_hosts']
+			? new CLink(_('Graphs'),
+				(new CUrl('zabbix.php'))
+					->setArgument('action', 'charts.view')
+					->setArgument('filter_set', '1')
+					->setArgument('view_as', HISTORY_GRAPH)
+					->setArgument('filter_search_type', ZBX_SEARCH_TYPE_STRICT)
+					->setArgument('filter_hostids', [$data['host']['hostid']])
+			)
+			: _('Graphs'),
+		$data['allowed_ui_hosts']
+			? new CLink(_('Dashboards'), (new CUrl('zabbix.php'))
+				->setArgument('action', 'host.dashboard.view')
+				->setArgument('hostid', $data['host']['hostid'])
+			)
+			: _('Dashboards')
 	])
 );
 
 // configuration
-if ($data['rwHost']) {
-	$hostLink = new CLink(_('Host'), 'hosts.php?form=update&hostid='.$data['host']['hostid'].url_param('groupid'));
+if ($data['allowed_ui_conf_hosts'] && $data['rwHost']) {
+	$hostLink = new CLink(_('Host'), (new CUrl('hosts.php'))
+		->setArgument('form', 'update')
+		->setArgument('hostid', $data['host']['hostid'])
+	);
 	$applicationsLink = new CLink(_('Applications'),
-		'applications.php?hostid='.$data['host']['hostid'].url_param('groupid')
+		(new CUrl('zabbix.php'))
+			->setArgument('action', 'application.list')
+			->setArgument('filter_set', '1')
+			->setArgument('filter_hostids', [$data['host']['hostid']])
 	);
 
 	$itemsLink = new CLink(_('Items'),
@@ -162,11 +196,20 @@ if ($data['rwHost']) {
 			->setArgument('filter_set', '1')
 			->setArgument('filter_hostids', [$data['host']['hostid']])
 	);
-	$graphsLink = new CLink(_('Graphs'), 'graphs.php?hostid='.$data['host']['hostid'].url_param('groupid'));
+	$graphsLink = new CLink(_('Graphs'),
+		(new CUrl('graphs.php'))
+			->setArgument('filter_set', '1')
+			->setArgument('filter_hostids', [$data['host']['hostid']])
+	);
 	$discoveryLink = new CLink(_('Discovery'),
-		'host_discovery.php?hostid='.$data['host']['hostid'].url_param('groupid')
+		(new CUrl('host_discovery.php'))
+			->setArgument('filter_set', '1')
+			->setArgument('filter_hostids', [$data['host']['hostid']])
+	);
+	$webLink = new CLink(_('Web'), (new CUrl('httpconf.php'))
+			->setArgument('filter_set', '1')
+			->setArgument('filter_hostids', [$data['host']['hostid']])
 	);
-	$webLink = new CLink(_('Web'), 'httpconf.php?hostid='.$data['host']['hostid'].url_param('groupid'));
 }
 else {
 	$hostLink = _('Host');
@@ -217,13 +260,16 @@ if (!$inventoryValues) {
 $hostInventoriesTab->addTab('detailsTab', _('Details'), $detailsFormList);
 
 // append tabs and form
-$hostInventoriesTab->setFooter(makeFormFooter(null, [new CButtonCancel(url_param('groupid'))]));
+$hostInventoriesTab->setFooter(makeFormFooter(null, [new CButtonCancel()]));
 
-return (new CWidget())
+$web_layout_mode = CViewHelper::loadLayoutMode();
+
+(new CWidget())
 	->setTitle(_('Host inventory'))
-	->setWebLayoutMode(CView::getLayoutMode())
-	->setControls((new CList())->addItem(get_icon('fullscreen')))
+	->setWebLayoutMode($web_layout_mode)
+	->setControls((new CList())->addItem(get_icon('kioskmode', ['mode' => $web_layout_mode])))
 	->addItem((new CForm())
 		->setAttribute('aria-labeledby', ZBX_STYLE_PAGE_TITLE)
 		->addItem($hostInventoriesTab)
-	);
+	)
+	->show();
