@@ -19,6 +19,10 @@
 **/
 
 
+/**
+ * @var CView $this
+ */
+
 $action = 'screenedit.php?form=update&screenid='.getRequest('screenid');
 if (isset($_REQUEST['screenitemid'])) {
 	$action .= '&screenitemid='.getRequest('screenitemid');
@@ -29,10 +33,6 @@ $form = (new CForm('post', $action))
 	->setAttribute('aria-labeledby', ZBX_STYLE_PAGE_TITLE)
 	->addVar('screenid', getRequest('screenid'));
 
-if (array_key_exists('templateid', $data['screen'])) {
-	$form->addVar('templateid', $data['screen']['templateid']);
-}
-
 if (hasRequest('screenitemid')) {
 	$form->addVar('screenitemid', getRequest('screenitemid'));
 	$screenItems = zbx_toHash($this->data['screen']['screenitems'], 'screenitemid');
@@ -81,16 +81,6 @@ else {
 
 // append resource types to form list
 $screenResources = screen_resources();
-if (array_key_exists('templateid', $data['screen'])) {
-	unset(
-		$screenResources[SCREEN_RESOURCE_DATA_OVERVIEW], $screenResources[SCREEN_RESOURCE_ACTIONS],
-		$screenResources[SCREEN_RESOURCE_EVENTS], $screenResources[SCREEN_RESOURCE_HOST_INFO],
-		$screenResources[SCREEN_RESOURCE_MAP], $screenResources[SCREEN_RESOURCE_SERVER_INFO],
-		$screenResources[SCREEN_RESOURCE_HOSTGROUP_TRIGGERS], $screenResources[SCREEN_RESOURCE_HOST_TRIGGERS],
-		$screenResources[SCREEN_RESOURCE_SYSTEM_STATUS], $screenResources[SCREEN_RESOURCE_TRIGGER_INFO],
-		$screenResources[SCREEN_RESOURCE_TRIGGER_OVERVIEW]
-	);
-}
 
 $screenFormList = (new CFormList())
 	->addRow((new CLabel(_('Resource'), 'resourcetype')),
@@ -134,19 +124,15 @@ if ($resourceType == SCREEN_RESOURCE_GRAPH) {
 				]
 				: [],
 			'popup' => [
-				'parameters' => array_merge([
+				'parameters' => [
 					'srctbl' => 'graphs',
 					'srcfld1' => 'graphid',
 					'srcfld2' => 'name',
 					'dstfrm' => $form->getName(),
-					'dstfld1' => 'resourceid'
-				], array_key_exists('templateid', $data['screen']) ? [
-					'templated_hosts' => '1',
-					'hostid' => $data['screen']['templateid']
-				] : [
+					'dstfld1' => 'resourceid',
 					'real_hosts' => '1',
 					'with_graphs' => '1'
-				]),
+				],
 			]
 		]))
 			->setWidth(ZBX_TEXTAREA_STANDARD_WIDTH)
@@ -192,18 +178,14 @@ elseif ($resourceType == SCREEN_RESOURCE_LLD_GRAPH) {
 					]
 					: [],
 				'popup' => [
-					'parameters' => array_merge([
+					'parameters' => [
 						'srctbl' => 'graph_prototypes',
 						'srcfld1' => 'graphid',
 						'srcfld2' => 'name',
 						'dstfrm' => $form->getName(),
-						'dstfld1' => 'resourceid'
-					], array_key_exists('templateid', $data['screen']) ? [
-						'templated_hosts' => '1',
-						'hostid' => $data['screen']['templateid']
-					] : [
+						'dstfld1' => 'resourceid',
 						'real_hosts' => '1'
-					]),
+					]
 				]
 			]))
 				->setWidth(ZBX_TEXTAREA_STANDARD_WIDTH)
@@ -260,9 +242,9 @@ elseif ($resourceType == SCREEN_RESOURCE_SIMPLE_GRAPH) {
 					'dstfld1' => 'resourceid',
 					'webitems' => true,
 					'numeric' => true,
-					'hostid' => array_key_exists('templateid', $data['screen']) ? $data['screen']['templateid'] : 0,
-					'real_hosts' => !array_key_exists('templateid', $data['screen']),
-					'with_simple_graph_items' => !array_key_exists('templateid', $data['screen'])
+					'hostid' => 0,
+					'real_hosts' => true,
+					'with_simple_graph_items' => true
 				]
 			]
 		]))
@@ -294,41 +276,22 @@ elseif ($resourceType == SCREEN_RESOURCE_LLD_SIMPLE_GRAPH) {
 		$caption = $item['host']['name'].NAME_DELIMITER.$item['name_expanded'];
 	}
 
-	if (array_key_exists('templateid', $data['screen'])) {
-		$selectButton = (new CButton('select', _('Select')))
-			->addClass(ZBX_STYLE_BTN_GREY)
-			->onClick('return PopUp("popup.generic",'.
-				CJs::encodeJson([
-					'srctbl' => 'item_prototypes',
-					'srcfld1' => 'itemid',
-					'srcfld2' => 'name',
-					'dstfrm' => $form->getName(),
-					'dstfld1' => 'resourceid',
-					'dstfld2' => 'caption',
-					'templated_hosts' => '1',
-					'only_hostid' => $data['screen']['templateid'],
-					'numeric' => '1'
-				]).', null, this);'
-			);
-	}
-	else {
-		$selectButton = (new CButton('select', _('Select')))
-			->addClass(ZBX_STYLE_BTN_GREY)
-			->onClick('return PopUp("popup.generic",'.
-				CJs::encodeJson([
-					'srctbl' => 'item_prototypes',
-					'srcfld1' => 'itemid',
-					'srcfld2' => 'name',
-					'dstfrm' => $form->getName(),
-					'dstfld1' => 'resourceid',
-					'dstfld2' => 'caption',
-					'real_hosts' => '1',
-					'with_discovery_rule' => '1',
-					'items' => '1',
-					'numeric' => '1'
-				]).', null, this);'
-			);
-	}
+	$selectButton = (new CButton('select', _('Select')))
+		->addClass(ZBX_STYLE_BTN_GREY)
+		->onClick('return PopUp("popup.generic",'.
+			json_encode([
+				'srctbl' => 'item_prototypes',
+				'srcfld1' => 'itemid',
+				'srcfld2' => 'name',
+				'dstfrm' => $form->getName(),
+				'dstfld1' => 'resourceid',
+				'dstfld2' => 'caption',
+				'real_hosts' => '1',
+				'with_discovery_rule' => '1',
+				'items' => '1',
+				'numeric' => '1'
+			]).', null, this);'
+		);
 
 	$form->addVar('resourceid', $id);
 	$screenFormList->addRow((new CLabel(_('Item prototype'), 'caption'))->setAsteriskMark(), [
@@ -373,7 +336,7 @@ elseif ($resourceType == SCREEN_RESOURCE_MAP) {
 		(new CButton('select', _('Select')))
 			->addClass(ZBX_STYLE_BTN_GREY)
 			->onClick('return PopUp("popup.generic",'.
-				CJs::encodeJson([
+				json_encode([
 					'srctbl' => 'sysmaps',
 					'srcfld1' => 'sysmapid',
 					'srcfld2' => 'name',
@@ -427,8 +390,8 @@ elseif ($resourceType == SCREEN_RESOURCE_PLAIN_TEXT) {
 						'srcfld1' => 'itemid',
 						'dstfrm' => $form->getName(),
 						'dstfld1' => 'resourceid',
-						'hostid' => array_key_exists('templateid', $data['screen']) ? $data['screen']['templateid'] : 0,
-						'real_hosts' => !array_key_exists('templateid', $data['screen']),
+						'hostid' => 0,
+						'real_hosts' => true,
 						'webitems' => true
 					]
 				],
@@ -634,7 +597,7 @@ elseif (in_array($resourceType, [SCREEN_RESOURCE_TRIGGER_OVERVIEW, SCREEN_RESOUR
 		(new CButton('application_select', _('Select')))
 			->addClass(ZBX_STYLE_BTN_GREY)
 			->onClick('return PopUp("popup.generic", '.
-				CJs::encodeJson([
+				json_encode([
 					'srctbl' => 'applications',
 					'srcfld1' => 'name',
 					'dstfrm' => $form->getName(),
@@ -738,8 +701,8 @@ elseif ($resourceType == SCREEN_RESOURCE_CLOCK) {
 						'srcfld1' => 'itemid',
 						'dstfrm' => $form->getName(),
 						'dstfld1' => 'resourceid',
-						'hostid' => array_key_exists('templateid', $data['screen']) ? $data['screen']['templateid'] : 0,
-						'real_hosts' => !array_key_exists('templateid', $data['screen']),
+						'hostid' => 0,
+						'real_hosts' => true,
 						'webitems' => true
 					]
 				]
@@ -855,7 +818,7 @@ $resourcesWithDynamic = [
 	SCREEN_RESOURCE_LLD_GRAPH,
 	SCREEN_RESOURCE_LLD_SIMPLE_GRAPH
 ];
-if (!array_key_exists('templateid', $this->data['screen']) && in_array($resourceType, $resourcesWithDynamic)) {
+if (in_array($resourceType, $resourcesWithDynamic)) {
 	$screenFormList->addRow(_('Dynamic item'), (new CCheckBox('dynamic'))->setChecked($dynamic == 1));
 }
 
@@ -867,16 +830,16 @@ if (isset($_REQUEST['screenitemid'])) {
 	$form->addItem(makeFormFooter(
 		new CSubmit('update', _('Update')),
 		[
-			new CButtonDelete(null, url_params(['form', 'screenid', 'templateid', 'screenitemid'])),
-			new CButtonCancel(url_params(['screenid', 'templateid']))
+			new CButtonDelete(null, url_params(['form', 'screenid', 'screenitemid'])),
+			new CButtonCancel(url_param('screenid'))
 		]
 	));
 }
 else {
 	$form->addItem(makeFormFooter(
 		new CSubmit('add', _('Add')),
-		[new CButtonCancel(url_params(['screenid', 'templateid']))]
+		[new CButtonCancel(url_param('screenid'))]
 	));
 }
 
-return $form;
+$form->show();
