@@ -287,6 +287,11 @@ Overlay.prototype.makeButton = function(obj) {
 		});
 
 	$button.on('click', function(e) {
+		if (('confirmation' in obj) && !confirm(obj.confirmation)) {
+			e.preventDefault();
+			return;
+		}
+
 		if (obj.isSubmit && this.$btn_submit) {
 			this.$btn_submit.blur().addClass('is-loading');
 		}
@@ -428,7 +433,9 @@ Overlay.prototype.setProperties = function(obj) {
 
 			case 'script_inline':
 				this.unsetProperty(key);
-				this.$dialogue.$script.html(obj[key]);
+				// See: jQuery.html() rnoInnerhtml = /<script|<style|<link/i
+				// If content matches this regex it will be parsed in jQuery.buildFragment as HTML, but here we have JS.
+				this.$dialogue.$script.get(0).innerHTML = obj[key];
 				this.$dialogue.$footer.prepend(this.$dialogue.$script);
 				break;
 
