@@ -26,7 +26,7 @@ require_once dirname(__FILE__).'/include/forms.inc.php';
 
 $page['title'] = _('Configuration of maintenance periods');
 $page['file'] = 'maintenance.php';
-$page['scripts'] = ['class.calendar.js', 'multiselect.js'];
+$page['scripts'] = ['class.cviewswitcher.js', 'class.calendar.js', 'multiselect.js', 'class.tab-indicators.js'];
 
 require_once dirname(__FILE__).'/include/page_header.php';
 
@@ -34,7 +34,6 @@ require_once dirname(__FILE__).'/include/page_header.php';
 $fields = [
 	'hostids' =>							[T_ZBX_INT, O_OPT, P_SYS,	DB_ID,		null],
 	'groupids' =>							[T_ZBX_INT, O_OPT, P_SYS,	DB_ID,		null],
-	'groupid' =>							[T_ZBX_INT, O_OPT, P_SYS,	DB_ID,		null],
 	// maintenance
 	'maintenanceid' =>						[T_ZBX_INT, O_OPT, P_SYS,	DB_ID,		'isset({form}) && {form} == "update"'],
 	'maintenanceids' =>						[T_ZBX_INT, O_OPT, P_SYS,	DB_ID, 		null],
@@ -47,17 +46,11 @@ $fields = [
 	'active_till' =>						[T_ZBX_ABS_TIME, O_OPT, null, NOT_EMPTY,
 												'isset({add}) || isset({update})', _('Active till')
 											],
-	'new_timeperiod_start_date' =>			[T_ZBX_ABS_TIME, O_OPT, null, 	NOT_EMPTY,	null, _('Date')],
-	'new_timeperiod' =>						[T_ZBX_STR, O_OPT, null,	null,		'isset({add_timeperiod})'],
 	'timeperiods' =>						[T_ZBX_STR, O_OPT, null,	null,		null],
-	'del_timeperiodid' =>					[T_ZBX_STR, O_OPT, P_ACT,	null,		null],
-	'edit_timeperiodid' =>					[T_ZBX_STR, O_OPT, P_ACT,	null,		null],
 	'tags_evaltype' =>						[T_ZBX_INT, O_OPT, null,	null,		null],
 	'tags' =>								[T_ZBX_STR, O_OPT, null,	null,		null],
 	// actions
 	'action' =>								[T_ZBX_STR, O_OPT, P_SYS|P_ACT, IN('"maintenance.massdelete"'), null],
-	'add_timeperiod' =>						[T_ZBX_STR, O_OPT, P_SYS|P_ACT, null,	null],
-	'cancel_new_timeperiod' =>				[T_ZBX_STR, O_OPT, P_SYS, null,	null],
 	'add' =>								[T_ZBX_STR, O_OPT, P_SYS|P_ACT, null,	null],
 	'update' =>								[T_ZBX_STR, O_OPT, P_SYS|P_ACT, null,	null],
 	'clone' =>								[T_ZBX_STR, O_OPT, P_SYS|P_ACT, null,	null],
@@ -71,6 +64,7 @@ $fields = [
 	'filter_rst' =>							[T_ZBX_STR, O_OPT, P_SYS,	null,		null],
 	'filter_name' =>						[T_ZBX_STR, O_OPT, null,	null,		null],
 	'filter_status' =>						[T_ZBX_INT, O_OPT, null,	IN([-1, MAINTENANCE_STATUS_ACTIVE, MAINTENANCE_STATUS_APPROACH, MAINTENANCE_STATUS_EXPIRED]), null],
+	'filter_groups' =>						[T_ZBX_INT, O_OPT, null,	DB_ID,		null],
 	// sort and sortorder
 	'sort' =>								[T_ZBX_STR, O_OPT, P_SYS,
 												IN('"active_since","active_till","maintenance_type","name"'),
@@ -86,9 +80,6 @@ check_fields($fields);
 /*
  * Permissions
  */
-if (getRequest('groupid') && !isWritableHostGroups([getRequest('groupid')])) {
-	access_deny();
-}
 if (isset($_REQUEST['maintenanceid'])) {
 	$dbMaintenance = API::Maintenance()->get([
 		'output' => API_OUTPUT_EXTEND,
@@ -105,6 +96,12 @@ if (hasRequest('action') && (!hasRequest('maintenanceids') || !is_array(getReque
 	access_deny();
 }
 
+$allowed_edit = CWebUser::checkAccess(CRoleHelper::ACTIONS_EDIT_MAINTENANCE);
+
+if (!$allowed_edit && hasRequest('form') && getRequest('form') !== 'update') {
+	access_deny(ACCESS_DENY_PAGE);
+}
+
 /*
  * Actions
  */
@@ -112,10 +109,11 @@ if (isset($_REQUEST['clone']) && isset($_REQUEST['maintenanceid'])) {
 	unset($_REQUEST['maintenanceid']);
 	$_REQUEST['form'] = 'clone';
 }
-elseif (isset($_REQUEST['cancel_new_timeperiod'])) {
-	unset($_REQUEST['new_timeperiod']);
-}
 elseif (hasRequest('add') || hasRequest('update')) {
+	if (!$allowed_edit) {
+		access_deny(ACCESS_DENY_PAGE);
+	}
+
 	if (hasRequest('update')) {
 		$messageSuccess = _('Maintenance updated');
 		$messageFailed = _('Cannot update maintenance');
@@ -136,13 +134,13 @@ elseif (hasRequest('add') || hasRequest('update')) {
 
 	if (!validateDateInterval($active_since_date->format('Y'), $active_since_date->format('m'),
 			$active_since_date->format('d'))) {
-		info(_s('"%s" must be between 1970.01.01 and 2038.01.18.', _('Active since')));
+		info(_s('"%1$s" must be between 1970.01.01 and 2038.01.18.', _('Active since')));
 		$result = false;
 	}
 
 	if (!validateDateInterval($active_till_date->format('Y'), $active_till_date->format('m'),
 			$active_till_date->format('d'))) {
-		info(_s('"%s" must be between 1970.01.01 and 2038.01.18.', _('Active till')));
+		info(_s('"%1$s" must be between 1970.01.01 and 2038.01.18.', _('Active till')));
 		$result = false;
 	}
 
@@ -199,7 +197,11 @@ elseif (hasRequest('add') || hasRequest('update')) {
 
 	show_messages($result, $messageSuccess, $messageFailed);
 }
-elseif (hasRequest('delete') || (hasRequest('action') && getRequest('action') == 'maintenance.massdelete')) {
+elseif (hasRequest('delete') || getRequest('action', '') == 'maintenance.massdelete') {
+	if (!$allowed_edit) {
+		access_deny(ACCESS_DENY_PAGE);
+	}
+
 	$maintenanceids = getRequest('maintenanceid', []);
 	if (hasRequest('maintenanceids')) {
 		$maintenanceids = getRequest('maintenanceids');
@@ -223,159 +225,13 @@ elseif (hasRequest('delete') || (hasRequest('action') && getRequest('action') ==
 
 	show_messages($result, _('Maintenance deleted'), _('Cannot delete maintenance'));
 }
-elseif (hasRequest('add_timeperiod') && hasRequest('new_timeperiod')) {
-	$new_timeperiod = getRequest('new_timeperiod');
-
-	if ($new_timeperiod['timeperiod_type'] == TIMEPERIOD_TYPE_ONETIME) {
-		$new_timeperiod['start_date'] = getRequest('new_timeperiod_start_date');
-	}
-
-	// start time
-	$new_timeperiod['start_time'] = ($new_timeperiod['hour'] * SEC_PER_HOUR) + ($new_timeperiod['minute'] * SEC_PER_MIN);
-
-	// period
-	$new_timeperiod['period'] = ($new_timeperiod['period_days'] * SEC_PER_DAY) + ($new_timeperiod['period_hours'] * SEC_PER_HOUR) + ($new_timeperiod['period_minutes'] * SEC_PER_MIN);
-
-	// days of week
-	if (!isset($new_timeperiod['dayofweek'])) {
-		$dayofweek =  (!isset($new_timeperiod['dayofweek_su'])) ? '0' : '1';
-		$dayofweek .= (!isset($new_timeperiod['dayofweek_sa'])) ? '0' : '1';
-		$dayofweek .= (!isset($new_timeperiod['dayofweek_fr'])) ? '0' : '1';
-		$dayofweek .= (!isset($new_timeperiod['dayofweek_th'])) ? '0' : '1';
-		$dayofweek .= (!isset($new_timeperiod['dayofweek_we'])) ? '0' : '1';
-		$dayofweek .= (!isset($new_timeperiod['dayofweek_tu'])) ? '0' : '1';
-		$dayofweek .= (!isset($new_timeperiod['dayofweek_mo'])) ? '0' : '1';
-		$new_timeperiod['dayofweek'] = bindec($dayofweek);
-	}
-
-	// months
-	if (!isset($new_timeperiod['month'])) {
-		$month =  (!isset($new_timeperiod['month_dec'])) ? '0' : '1';
-		$month .= (!isset($new_timeperiod['month_nov'])) ? '0' : '1';
-		$month .= (!isset($new_timeperiod['month_oct'])) ? '0' : '1';
-		$month .= (!isset($new_timeperiod['month_sep'])) ? '0' : '1';
-		$month .= (!isset($new_timeperiod['month_aug'])) ? '0' : '1';
-		$month .= (!isset($new_timeperiod['month_jul'])) ? '0' : '1';
-		$month .= (!isset($new_timeperiod['month_jun'])) ? '0' : '1';
-		$month .= (!isset($new_timeperiod['month_may'])) ? '0' : '1';
-		$month .= (!isset($new_timeperiod['month_apr'])) ? '0' : '1';
-		$month .= (!isset($new_timeperiod['month_mar'])) ? '0' : '1';
-		$month .= (!isset($new_timeperiod['month_feb'])) ? '0' : '1';
-		$month .= (!isset($new_timeperiod['month_jan'])) ? '0' : '1';
-		$new_timeperiod['month'] = bindec($month);
-	}
-
-	if ($new_timeperiod['timeperiod_type'] == TIMEPERIOD_TYPE_MONTHLY) {
-		if ($new_timeperiod['month_date_type'] > 0) {
-			$new_timeperiod['day'] = 0;
-		}
-		else {
-			$new_timeperiod['every'] = 1;
-			$new_timeperiod['dayofweek'] = 0;
-		}
-	}
-
-	$_REQUEST['timeperiods'] = getRequest('timeperiods', []);
-
-	$result = true;
-
-	if ($new_timeperiod['period'] < 300) {
-		info(_('Incorrect maintenance period (minimum 5 minutes)'));
-		$result = false;
-	}
-	elseif ($new_timeperiod['hour'] > 23 || $new_timeperiod['minute'] > 59) {
-		info(_('Incorrect maintenance period'));
-		$result = false;
-	}
-	elseif ($new_timeperiod['timeperiod_type'] == TIMEPERIOD_TYPE_ONETIME) {
-		$absolute_time_parser = new CAbsoluteTimeParser();
-		$absolute_time_parser->parse($new_timeperiod['start_date']);
-		$start_date = $absolute_time_parser->getDateTime(true);
-		$new_timeperiod['start_date'] = $start_date->format(ZBX_DATE_TIME);
-
-		if (!validateDateInterval($start_date->format('Y'), $start_date->format('m'), $start_date->format('d'))) {
-			error(_('Incorrect maintenance - date must be between 1970.01.01 and 2038.01.18'));
-			$result = false;
-		}
-	}
-	elseif ($new_timeperiod['timeperiod_type'] == TIMEPERIOD_TYPE_DAILY && $new_timeperiod['every'] < 1) {
-		info(_('Incorrect maintenance day period'));
-		$result = false;
-	}
-	elseif ($new_timeperiod['timeperiod_type'] == TIMEPERIOD_TYPE_WEEKLY) {
-		if ($new_timeperiod['every'] < 1) {
-			info(_('Incorrect maintenance week period'));
-			$result = false;
-		}
-		elseif ($new_timeperiod['dayofweek'] < 1) {
-			info(_('Incorrect maintenance days of week'));
-			$result = false;
-		}
-	}
-	elseif ($new_timeperiod['timeperiod_type'] == TIMEPERIOD_TYPE_MONTHLY) {
-		if ($new_timeperiod['month'] < 1) {
-			info(_('Incorrect maintenance month period'));
-			$result = false;
-		}
-		elseif ($new_timeperiod['day'] == 0 && $new_timeperiod['dayofweek'] < 1) {
-			info(_('Incorrect maintenance days of week'));
-			$result = false;
-		}
-		elseif (($new_timeperiod['day'] < 1 || $new_timeperiod['day'] > 31) && $new_timeperiod['dayofweek'] == 0) {
-			info(_('Incorrect maintenance date'));
-			$result = false;
-		}
-	}
-
-	show_messages();
-
-	if ($result) {
-		if (!isset($new_timeperiod['id'])) {
-			if (!str_in_array($new_timeperiod, $_REQUEST['timeperiods'])) {
-				array_push($_REQUEST['timeperiods'], $new_timeperiod);
-			}
-		}
-		else {
-			$id = $new_timeperiod['id'];
-			unset($new_timeperiod['id']);
-			$_REQUEST['timeperiods'][$id] = $new_timeperiod;
-		}
-		unset($_REQUEST['new_timeperiod']);
-	}
-}
-elseif (isset($_REQUEST['del_timeperiodid'])) {
-	$_REQUEST['timeperiods'] = getRequest('timeperiods', []);
-	$delTimeperiodId = array_keys($_REQUEST['del_timeperiodid']);
-	$delTimeperiodId = reset($delTimeperiodId);
-	unset($_REQUEST['timeperiods'][$delTimeperiodId]);
-}
-elseif (isset($_REQUEST['edit_timeperiodid'])) {
-	$_REQUEST['edit_timeperiodid'] = array_keys($_REQUEST['edit_timeperiodid']);
-	$edit_timeperiodid = $_REQUEST['edit_timeperiodid'] = array_pop($_REQUEST['edit_timeperiodid']);
-	$_REQUEST['timeperiods'] = getRequest('timeperiods', []);
-
-	if (isset($_REQUEST['timeperiods'][$edit_timeperiodid])) {
-		$_REQUEST['new_timeperiod'] = $_REQUEST['timeperiods'][$edit_timeperiodid];
-		$_REQUEST['new_timeperiod']['id'] = $edit_timeperiodid;
-		$_REQUEST['new_timeperiod_start_date'] = $_REQUEST['new_timeperiod']['start_date'];
-	}
-}
-
-$options = [
-	'groups' => ['editable' => 1],
-	'groupid' => getRequest('groupid')
-];
-$pageFilter = new CPageFilter($options);
-$_REQUEST['groupid'] = $pageFilter->groupid;
 
 /*
  * Display
  */
 $data = [
 	'form' => getRequest('form'),
-	'new_timeperiod' => getRequest('new_timeperiod', []),
-	'add_timeperiod' => getRequest('add_timeperiod', 0),
-	'new_timeperiod_start_date' => getRequest('new_timeperiod_start_date')
+	'allowed_edit' => $allowed_edit
 ];
 
 if (!empty($data['form'])) {
@@ -419,12 +275,6 @@ if (!empty($data['form'])) {
 		CArrayHelper::sort($data['tags'], ['tag', 'value']);
 	}
 	else {
-
-		if ($data['new_timeperiod'] != 1 && $data['new_timeperiod'] !== []
-				&& hasRequest('new_timeperiod_start_date')) {
-			$data['new_timeperiod']['start_date'] = getRequest('new_timeperiod_start_date');
-		}
-
 		$data += [
 			'mname' => getRequest('mname', ''),
 			'maintenance_type' => getRequest('maintenance_type', 0),
@@ -463,9 +313,7 @@ if (!empty($data['form'])) {
 	CArrayHelper::sort($data['groups_ms'], ['name']);
 
 	// render view
-	$maintenanceView = new CView('configuration.maintenance.edit', $data);
-	$maintenanceView->render();
-	$maintenanceView->show();
+	echo (new CView('configuration.maintenance.edit', $data))->getOutput();
 }
 else {
 	// get maintenances
@@ -479,25 +327,42 @@ else {
 	if (hasRequest('filter_set')) {
 		CProfile::update('web.maintenance.filter_name', getRequest('filter_name', ''), PROFILE_TYPE_STR);
 		CProfile::update('web.maintenance.filter_status', getRequest('filter_status', -1), PROFILE_TYPE_INT);
+		CProfile::updateArray('web.maintenance.filter_groups', getRequest('filter_groups', []), PROFILE_TYPE_ID);
 	}
 	elseif (hasRequest('filter_rst')) {
 		CProfile::delete('web.maintenance.filter_name');
 		CProfile::delete('web.maintenance.filter_status');
+		CProfile::deleteIdx('web.maintenance.filter_groups');
 	}
 
 	$filter = [
 		'name' => CProfile::get('web.maintenance.filter_name', ''),
-		'status' => CProfile::get('web.maintenance.filter_status', -1)
+		'status' => CProfile::get('web.maintenance.filter_status', -1),
+		'groups' => CProfile::getArray('web.maintenance.filter_groups', [])
 	];
 
-	$config = select_config();
+	// Get host groups.
+	$filter['groups'] = $filter['groups']
+		? CArrayHelper::renameObjectsKeys(API::HostGroup()->get([
+			'output' => ['groupid', 'name'],
+			'groupids' => $filter['groups'],
+			'editable' => true,
+			'preservekeys' => true
+		]), ['groupid' => 'id'])
+		: [];
+
+	$filter_groupids = $filter['groups'] ? array_keys($filter['groups']) : null;
+	if ($filter_groupids) {
+		$filter_groupids = getSubGroups($filter_groupids);
+	}
 
 	$data = [
 		'sort' => $sortField,
 		'sortorder' => $sortOrder,
 		'filter' => $filter,
 		'profileIdx' => 'web.maintenance.filter',
-		'active_tab' => CProfile::get('web.maintenance.filter.active', 1)
+		'active_tab' => CProfile::get('web.maintenance.filter.active', 1),
+		'allowed_edit' => $allowed_edit
 	];
 
 	// Get list of maintenances.
@@ -506,19 +371,13 @@ else {
 		'search' => [
 			'name' => ($filter['name'] === '') ? null : $filter['name']
 		],
+		'groupids' => $filter_groupids,
 		'editable' => true,
 		'sortfield' => $sortField,
 		'sortorder' => $sortOrder,
-		'limit' => $config['search_limit'] + 1
+		'limit' => CSettingsHelper::get(CSettingsHelper::SEARCH_LIMIT) + 1
 	];
 
-	if ($pageFilter->groupsSelected && $pageFilter->groupid > 0) {
-		$options['groupids'] = $pageFilter->groupids;
-	}
-	else {
-		$options['groupids'] = $config['dropdown_first_entry'] ? null : [];
-	}
-
 	$data['maintenances'] = API::Maintenance()->get($options);
 
 	foreach ($data['maintenances'] as $key => $maintenance) {
@@ -544,17 +403,23 @@ else {
 
 	order_result($data['maintenances'], $sortField, $sortOrder);
 
-	$url = (new CUrl('maintenance.php'))
-		->setArgument('groupid', $pageFilter->groupid);
+	// pager
+	if (hasRequest('page')) {
+		$page_num = getRequest('page');
+	}
+	elseif (isRequestMethod('get') && !hasRequest('cancel')) {
+		$page_num = 1;
+	}
+	else {
+		$page_num = CPagerHelper::loadPage($page['file']);
+	}
 
-	$data['paging'] = getPagingLine($data['maintenances'], $sortOrder, $url);
+	CPagerHelper::savePage($page['file'], $page_num);
 
-	$data['pageFilter'] = $pageFilter;
+	$data['paging'] = CPagerHelper::paginate($page_num, $data['maintenances'], $sortOrder, new CUrl('maintenance.php'));
 
 	// render view
-	$maintenanceView = new CView('configuration.maintenance.list', $data);
-	$maintenanceView->render();
-	$maintenanceView->show();
+	echo (new CView('configuration.maintenance.list', $data))->getOutput();
 }
 
 require_once dirname(__FILE__).'/include/page_footer.php';
