@@ -57,7 +57,6 @@ $fields = [
 ];
 check_fields($fields);
 
-$config = select_config();
 if (hasRequest('filter_set')) {
 	CProfile::update('web.overview.filter.show_triggers', getRequest('show_triggers', TRIGGERS_OPTION_RECENT_PROBLEM),
 		PROFILE_TYPE_INT
@@ -128,7 +127,6 @@ if (hasRequest('view_style')) {
 $data = [
 	'type' => $type,
 	'view_style' => CProfile::get('web.overview.view_style', STYLE_TOP),
-	'config' => $config,
 	'profileIdx' => 'web.overview.filter',
 	'active_tab' => CProfile::get('web.overview.filter.active', 1),
 	'db_hosts' => [],
@@ -185,7 +183,6 @@ if ($type == SHOW_TRIGGERS) {
 		? CArrayHelper::renameObjectsKeys(API::HostGroup()->get([
 			'output' => ['groupid', 'name'],
 			'groupids' => $filter['groupids'],
-			'with_monitored_triggers' => true,
 			'preservekeys' => true
 		]), ['groupid' => 'id'])
 		: [];
@@ -221,6 +218,10 @@ if ($type == SHOW_TRIGGERS) {
 
 	$data['filter'] = $filter;
 
+	$data['config'] = [
+		'blink_period' => CSettingsHelper::get(CSettingsHelper::BLINK_PERIOD)
+	];
+
 	// Render view.
 	echo (new CView('monitoring.overview.triggers', $data))->getOutput();
 }
