@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2017 Zabbix SIA
+** Copyright (C) 2001-2020 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -26,12 +26,16 @@ require_once dirname(__FILE__).'/include/forms.inc.php';
 $page['title'] = _('Configuration of network maps');
 $page['file'] = 'sysmap.php';
 $page['scripts'] = ['class.svg.canvas.js', 'class.svg.map.js', 'class.cmap.js', 'class.cviewswitcher.js',
-	'multiselect.js'
+	'multiselect.js', 'colorpicker.js'
 ];
 $page['type'] = detect_page_type();
 
 require_once dirname(__FILE__).'/include/page_header.php';
 
+if (!CWebUser::checkAccess(CRoleHelper::ACTIONS_EDIT_MAPS)) {
+	access_deny(ACCESS_DENY_PAGE);
+}
+
 // VAR	TYPE	OPTIONAL	FLAGS	VALIDATION	EXCEPTION
 $fields = [
 	'sysmapid' =>	[T_ZBX_INT, O_MAND, P_SYS,	DB_ID,		null],
@@ -56,8 +60,6 @@ check_fields($fields);
  * Ajax
  */
 if (isset($_REQUEST['favobj'])) {
-	$json = new CJson();
-
 	if (getRequest('favobj') === 'sysmap' && hasRequest('action')) {
 		if (getRequest('action') === 'update') {
 			$sysmapid = getRequest('sysmapid', 0);
@@ -78,7 +80,7 @@ if (isset($_REQUEST['favobj'])) {
 					throw new Exception(_('Access denied!'));
 				}
 
-				$sysmapUpdate = $json->decode($_REQUEST['sysmap'], true);
+				$sysmapUpdate = json_decode($_REQUEST['sysmap'], true);
 				$sysmapUpdate['sysmapid'] = $sysmapid;
 				$sysmapUpdate['lines'] = [];
 
@@ -99,7 +101,14 @@ if (isset($_REQUEST['favobj'])) {
 				$result = API::Map()->update($sysmapUpdate);
 
 				if ($result !== false) {
-					echo 'if (confirm('.CJs::encodeJson(_('Map is updated! Return?')).')) { location.href = "sysmaps.php"; }';
+					$url = (new CUrl('sysmaps.php'))
+						->setArgument('page', CPagerHelper::loadPage('sysmaps.php', null))
+						->getUrl();
+
+					echo
+						'if (confirm('.json_encode(_('Map is updated! Return to map list?')).')) {'.
+							'location.href = "'.$url.'";'.
+						'}';
 				}
 				else {
 					throw new Exception(_('Map update failed.'));
@@ -111,7 +120,7 @@ if (isset($_REQUEST['favobj'])) {
 				DBend(false);
 				$msg = [$e->getMessage()];
 
-				foreach (clear_messages() as $errMsg) {
+				foreach (get_and_clear_messages() as $errMsg) {
 					$msg[] = $errMsg['type'].': '.$errMsg['message'];
 				}
 
@@ -121,13 +130,15 @@ if (isset($_REQUEST['favobj'])) {
 			}
 
 			@ob_flush();
-			exit;
+			session_write_close();
+			exit();
 		}
 		elseif (getRequest('action') === 'expand') {
 			$values = [];
+			$return = [];
 			$sources = json_decode(getRequest('source'), true);
 
-			foreach ($sources as $source) {
+			foreach ($sources as $num => $source) {
 				if (is_array($source) && (array_key_exists('label', $source) || array_key_exists('text', $source))) {
 					if (array_key_exists('inherited_label', $source) && $source['inherited_label'] !== null) {
 						$source['label'] = $source['inherited_label'];
@@ -135,28 +146,38 @@ if (isset($_REQUEST['favobj'])) {
 
 					if (array_key_exists('elementtype', $source) && array_key_exists('elements', $source)
 							&& is_array($source['elements']) && CMapHelper::checkSelementPermissions([$source])) {
-
-						$values[] = CMacrosResolverHelper::resolveMapLabelMacrosAll($source);
+						$values[$num] = $source;
 					}
 					else {
 						$label = array_key_exists('label', $source) ? $source['label'] : $source['text'];
-						$values[] = CMacrosResolverHelper::resolveMapLabelMacros($label);
+						$return[$num] = CMacrosResolverHelper::resolveMapLabelMacros($label);
 					}
 				}
 				else {
-					$values[] = null;
+					$return[$num] = null;
 				}
 			}
 
-			echo CJs::encodeJson($values);
-			exit;
+			if ($values) {
+				// Resolve macros in map element labels.
+				$values = CMacrosResolverHelper::resolveMacrosInMapElements($values, ['resolve_element_label' => true]);
+
+				foreach ($values as $num => $value) {
+					$return[$num] = $value['label'];
+				}
+			}
+
+			ksort($return);
+
+			echo json_encode($return);
+			session_write_close();
+			exit();
 		}
 	}
 }
 
-if (PAGE_TYPE_HTML != $page['type']) {
+if ($page['type'] != PAGE_TYPE_HTML) {
 	require_once dirname(__FILE__).'/include/page_footer.php';
-	exit;
 }
 
 /*
@@ -199,8 +220,11 @@ $data = [
 	'defaultIconName' => null
 ];
 
+// Apply inherited element label properties.
+$data['sysmap'] = CMapHelper::applyMapElementLabelProperties($data['sysmap']);
+
 // get selements
-add_elementNames($data['sysmap']['selements']);
+addElementNames($data['sysmap']['selements']);
 
 foreach ($data['sysmap']['shapes'] as &$shape) {
 	$shape['expanded'] = CMacrosResolverHelper::resolveMapLabelMacros($shape['text']);
@@ -216,18 +240,13 @@ $data['sysmap']['selements'] = zbx_toHash($data['sysmap']['selements'], 'selemen
 $data['sysmap']['shapes'] = zbx_toHash($data['sysmap']['shapes'], 'sysmap_shapeid');
 $data['sysmap']['links'] = zbx_toHash($data['sysmap']['links'], 'linkid');
 
-$data['sysmap'] = CMapHelper::setElementInheritedLabels($data['sysmap']);
-foreach ($data['sysmap']['selements'] as &$selement) {
-	if ($selement['inherited_label'] !== null) {
-		$label = $selement['label'];
-		$selement['label'] = $selement['inherited_label'];
-	}
+// Extend $selement adding resolved label as property named 'expanded'.
+$resolve_opt = ['resolve_element_label' => true];
+$selements_resolved = CMacrosResolverHelper::resolveMacrosInMapElements($data['sysmap']['selements'], $resolve_opt);
 
-	$selement['expanded'] = CMacrosResolverHelper::resolveMapLabelMacrosAll($selement);
-
-	if ($selement['inherited_label'] !== null) {
-		$selement['label'] = $label;
-	}
+// Set extended and restore original labels.
+foreach ($data['sysmap']['selements'] as $selementid => &$selement) {
+	$selement['expanded'] = $selements_resolved[$selementid]['label'];
 }
 unset($selement);
 
@@ -268,6 +287,7 @@ $images = API::Image()->get([
 	'filter' => ['imagetype' => IMAGE_TYPE_ICON],
 	'select_image' => true
 ]);
+
 foreach ($images as $image) {
 	$image['image'] = base64_decode($image['image']);
 	$ico = imagecreatefromstring($image['image']);
@@ -284,6 +304,7 @@ foreach ($images as $image) {
 		$data['defaultIconName'] = $image['name'];
 	}
 }
+
 if ($data['iconList']) {
 	CArrayHelper::sort($data['iconList'], ['name']);
 	$data['iconList'] = array_values($data['iconList']);
@@ -292,8 +313,6 @@ if ($data['iconList']) {
 $data['theme'] = getUserGraphTheme();
 
 // render view
-$sysmapView = new CView('monitoring.sysmap.constructor', $data);
-$sysmapView->render();
-$sysmapView->show();
+echo (new CView('monitoring.sysmap.constructor', $data))->getOutput();
 
 require_once dirname(__FILE__).'/include/page_footer.php';
