@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2019 Zabbix SIA
+** Copyright (C) 2001-2020 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -30,9 +30,7 @@ $page['title'] = _('Event details');
 $page['file'] = 'tr_events.php';
 $page['type'] = detect_page_type();
 $page['scripts'] = ['layout.mode.js'];
-
-CView::$has_web_layout_mode = true;
-$page['web_layout_mode'] = CView::getLayoutMode();
+$page['web_layout_mode'] = CViewHelper::loadLayoutMode();
 
 require_once dirname(__FILE__).'/include/page_header.php';
 
@@ -143,15 +141,6 @@ else {
 	$event['opdata'] = (new CCol(CScreenProblem::getLatestValues($db_items)))->addClass('latest-values');
 }
 
-$config = select_config();
-$severity_config = [
-	'severity_name_0' => $config['severity_name_0'],
-	'severity_name_1' => $config['severity_name_1'],
-	'severity_name_2' => $config['severity_name_2'],
-	'severity_name_3' => $config['severity_name_3'],
-	'severity_name_4' => $config['severity_name_4'],
-	'severity_name_5' => $config['severity_name_5']
-];
 $actions = getEventDetailsActions($event);
 $users = API::User()->get([
 	'output' => ['alias', 'name', 'surname'],
@@ -164,29 +153,32 @@ $mediatypes = API::Mediatype()->get([
 	'preservekeys' => true
 ]);
 
+$allowed = [
+	'ack' => CWebUser::checkAccess(CRoleHelper::ACTIONS_ACKNOWLEDGE_PROBLEMS)
+			|| CWebUser::checkAccess(CRoleHelper::ACTIONS_CLOSE_PROBLEMS)
+			|| CWebUser::checkAccess(CRoleHelper::ACTIONS_CHANGE_SEVERITY)
+			|| CWebUser::checkAccess(CRoleHelper::ACTIONS_ADD_PROBLEM_COMMENTS),
+	'ui_correlation' => CWebUser::checkAccess(CRoleHelper::UI_CONFIGURATION_EVENT_CORRELATION)
+];
+
 /*
  * Display
  */
-$event_details_url = (new CUrl($page['file']))
-	->setArgument('triggerid', getRequest('triggerid'))
-	->setArgument('eventid', getRequest('eventid'))
-	->getUrl();
-
 $event_tab = (new CDiv([
 	new CDiv([
 		(new CUiWidget(WIDGET_HAT_TRIGGERDETAILS, make_trigger_details($trigger, $event['eventid'])))
 			->setHeader(_('Trigger details')),
-		(new CUiWidget(WIDGET_HAT_EVENTDETAILS, make_event_details($event, $event_details_url)))
+		(new CUiWidget(WIDGET_HAT_EVENTDETAILS, make_event_details($event, $allowed)))
 			->setHeader(_('Event details'))
 	]),
 	new CDiv([
 		(new CCollapsibleUiWidget(WIDGET_HAT_EVENTACTIONS,
-			makeEventDetailsActionsTable($actions, $users, $mediatypes, $severity_config)
+			makeEventDetailsActionsTable($actions, $users, $mediatypes)
 		))
 			->setExpanded((bool) CProfile::get('web.tr_events.hats.'.WIDGET_HAT_EVENTACTIONS.'.state', true))
 			->setHeader(_('Actions'), [], 'web.tr_events.hats.'.WIDGET_HAT_EVENTACTIONS.'.state')
 			->addClass(ZBX_STYLE_DASHBRD_WIDGET_FLUID),
-		(new CCollapsibleUiWidget(WIDGET_HAT_EVENTLIST, make_small_eventlist($event, $event_details_url)))
+		(new CCollapsibleUiWidget(WIDGET_HAT_EVENTLIST, make_small_eventlist($event, $allowed)))
 			->setExpanded((bool) CProfile::get('web.tr_events.hats.'.WIDGET_HAT_EVENTLIST.'.state', true))
 			->setHeader(_('Event list [previous 20]'), [], 'web.tr_events.hats.'.WIDGET_HAT_EVENTLIST.'.state')
 			->addClass(ZBX_STYLE_DASHBRD_WIDGET_FLUID)
@@ -195,16 +187,24 @@ $event_tab = (new CDiv([
 	->addClass(ZBX_STYLE_COLUMNS)
 	->addClass(ZBX_STYLE_COLUMNS_2);
 
+$script = (new CScriptTag(
+	'$.subscribe("acknowledge.create", function(event, response, overlay) {'.
+		'postMessageOk(response.message);'.
+		'location.href = location.href;'.
+	'});'
+))->setOnDocumentReady();
+
 (new CWidget())
 	->setTitle(_('Event details'))
 	->setWebLayoutMode($page['web_layout_mode'])
 	->setControls((new CTag('nav', true,
 		(new CList())
-			->addItem(get_icon('fullscreen'))
+			->addItem(get_icon('kioskmode', ['mode' => $page['web_layout_mode']]))
 		))
 		->setAttribute('aria-label', _('Content controls'))
 	)
 	->addItem($event_tab)
+	->addItem($script)
 	->show();
 
 require_once dirname(__FILE__).'/include/page_footer.php';
