@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2019 Zabbix SIA
+** Copyright (C) 2001-2020 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -26,7 +26,7 @@ require_once dirname(__FILE__).'/include/forms.inc.php';
 
 $page['title'] = _('Configuration of triggers');
 $page['file'] = 'triggers.php';
-$page['scripts'] = ['multiselect.js', 'textareaflexible.js'];
+$page['scripts'] = ['multiselect.js', 'textareaflexible.js', 'class.tab-indicators.js'];
 
 require_once dirname(__FILE__).'/include/page_header.php';
 
@@ -43,6 +43,7 @@ $fields = [
 	'copy_mode' =>								[T_ZBX_INT, O_OPT, P_SYS,	IN('0'),		null],
 	'type' =>									[T_ZBX_INT, O_OPT, null,	IN('0,1'),		null],
 	'description' =>							[T_ZBX_STR, O_OPT, null,	NOT_EMPTY,		'isset({add}) || isset({update})', _('Name')],
+	'event_name' =>								[T_ZBX_STR, O_OPT, null,	null,			'isset({add}) || isset({update})'],
 	'opdata' =>									[T_ZBX_STR, O_OPT, null,	null,			'isset({add}) || isset({update})'],
 	'expression' =>								[T_ZBX_STR, O_OPT, null,	NOT_EMPTY,		'isset({add}) || isset({update})', _('Expression')],
 	'recovery_expression' =>					[T_ZBX_STR, O_OPT, null,	NOT_EMPTY,		'(isset({add}) || isset({update})) && isset({recovery_mode}) && {recovery_mode} == '.ZBX_RECOVERY_MODE_RECOVERY_EXPRESSION.'', _('Recovery expression')],
@@ -251,6 +252,7 @@ if (hasRequest('clone') && hasRequest('triggerid')) {
 elseif (hasRequest('add') || hasRequest('update')) {
 	$dependencies = zbx_toObject(getRequest('dependencies', []), 'triggerid');
 	$description = getRequest('description', '');
+	$event_name = getRequest('event_name', '');
 	$opdata = getRequest('opdata', '');
 	$expression = getRequest('expression', '');
 	$recovery_mode = getRequest('recovery_mode', ZBX_RECOVERY_MODE_EXPRESSION);
@@ -267,6 +269,7 @@ elseif (hasRequest('add') || hasRequest('update')) {
 	if (hasRequest('add')) {
 		$trigger = [
 			'description' => $description,
+			'event_name' => $event_name,
 			'opdata' => $opdata,
 			'expression' => $expression,
 			'recovery_mode' => $recovery_mode,
@@ -302,7 +305,7 @@ elseif (hasRequest('add') || hasRequest('update')) {
 		$db_triggers = API::Trigger()->get([
 			'output' => ['expression', 'description', 'url', 'status', 'priority', 'comments', 'templateid', 'type',
 				'flags', 'recovery_mode', 'recovery_expression', 'correlation_mode', 'correlation_tag', 'manual_close',
-				'opdata'
+				'opdata', 'event_name'
 			],
 			'selectDependencies' => ['triggerid'],
 			'selectTags' => ['tag', 'value'],
@@ -322,6 +325,9 @@ elseif (hasRequest('add') || hasRequest('update')) {
 				if ($db_trigger['description'] !== $description) {
 					$trigger['description'] = $description;
 				}
+				if ($db_trigger['event_name'] !== $event_name) {
+					$trigger['event_name'] = $event_name;
+				}
 				if ($db_trigger['opdata'] !== $opdata) {
 					$trigger['opdata'] = $opdata;
 				}
@@ -615,21 +621,18 @@ elseif (hasRequest('action') && getRequest('action') === 'trigger.massdelete' &&
 	show_messages($result, _('Triggers deleted'), _('Cannot delete triggers'));
 }
 
-$config = select_config();
-
 /*
  * Display
  */
 if ((getRequest('action') === 'trigger.massupdateform' || hasRequest('massupdate')) && hasRequest('g_triggerid')) {
 	$data = getTriggerMassupdateFormData();
 	$data['action'] = 'trigger.massupdate';
-	$triggersView = new CView('configuration.triggers.massupdate', $data);
-	$triggersView->render();
-	$triggersView->show();
+
+	// render view
+	echo (new CView('configuration.triggers.massupdate', $data))->getOutput();
 }
 elseif (isset($_REQUEST['form'])) {
 	$data = [
-		'config' => $config,
 		'form' => getRequest('form'),
 		'form_refresh' => getRequest('form_refresh'),
 		'parent_discoveryid' => null,
@@ -642,6 +645,7 @@ elseif (isset($_REQUEST['form'])) {
 		'recovery_expr_temp' => getRequest('recovery_expr_temp', ''),
 		'recovery_mode' => getRequest('recovery_mode', 0),
 		'description' => getRequest('description', ''),
+		'event_name' => getRequest('event_name', ''),
 		'opdata' => getRequest('opdata', ''),
 		'type' => getRequest('type', 0),
 		'priority' => getRequest('priority', TRIGGER_SEVERITY_NOT_CLASSIFIED),
@@ -656,25 +660,22 @@ elseif (isset($_REQUEST['form'])) {
 		'hostid' => getRequest('hostid', 0),
 		'expression_action' => $expression_action,
 		'recovery_expression_action' => $recovery_expression_action,
-		'tags' => $tags,
+		'tags' => array_values($tags),
 		'show_inherited_tags' => getRequest('show_inherited_tags', 0),
 		'correlation_mode' => getRequest('correlation_mode', ZBX_TRIGGER_CORRELATION_NONE),
 		'correlation_tag' => getRequest('correlation_tag', ''),
 		'manual_close' => getRequest('manual_close', ZBX_TRIGGER_MANUAL_CLOSE_NOT_ALLOWED)
 	];
 
-	$triggersView = new CView('configuration.triggers.edit', getTriggerFormData($data));
-	$triggersView->render();
-	$triggersView->show();
+	// render view
+	echo (new CView('configuration.triggers.edit', getTriggerFormData($data)))->getOutput();
 }
 elseif (hasRequest('action') && getRequest('action') === 'trigger.masscopyto' && hasRequest('g_triggerid')) {
 	$data = getCopyElementsFormData('g_triggerid', _('Triggers'));
 	$data['action'] = 'trigger.masscopyto';
 
 	// render view
-	$triggersView = new CView('configuration.copy.elements', $data);
-	$triggersView->render();
-	$triggersView->show();
+	echo (new CView('configuration.copy.elements', $data))->getOutput();
 }
 else {
 	$filter_groupids_ms = [];
@@ -780,7 +781,7 @@ else {
 		'inherited' => ($filter_inherited != -1) ? $filter_inherited : null,
 		'preservekeys' => true,
 		'sortfield' => $sort,
-		'limit' => $config['search_limit'] + 1
+		'limit' => CSettingsHelper::get(CSettingsHelper::SEARCH_LIMIT) + 1
 	];
 
 	if ($sort === 'status') {
@@ -834,11 +835,22 @@ else {
 		order_result($prefetched_triggers, $sort, $sortorder);
 	}
 
-	$url = (new CUrl('triggers.php'))
-		->setArgument('filter_groupids', $filter_groupids)
-		->setArgument('filter_hostids', $filter_hostids);
+	// pager
+	if (hasRequest('page')) {
+		$page_num = getRequest('page');
+	}
+	elseif (isRequestMethod('get') && !hasRequest('cancel')) {
+		$page_num = 1;
+	}
+	else {
+		$page_num = CPagerHelper::loadPage($page['file']);
+	}
 
-	$paging = getPagingLine($prefetched_triggers, $sortorder, $url);
+	CPagerHelper::savePage($page['file'], $page_num);
+
+	$paging = CPagerHelper::paginate($page_num, $prefetched_triggers, $sortorder, new CUrl('triggers.php'));
+
+	// fetch triggers
 	$triggers = [];
 	if ($prefetched_triggers) {
 		$triggers = API::Trigger()->get([
@@ -848,12 +860,45 @@ else {
 			'selectHosts' => ['hostid', 'host', 'name', 'status'],
 			'selectDependencies' => ['triggerid', 'description'],
 			'selectDiscoveryRule' => ['itemid', 'name'],
+			'selectTriggerDiscovery' => ['ts_delete'],
 			'selectTags' => ['tag', 'value'],
 			'triggerids' => array_keys($prefetched_triggers),
 			'preservekeys' => true,
 			'nopermissions' => true
 		]);
 
+		$items = API::Item()->get([
+			'output' => ['itemid'],
+			'selectTriggers' => ['triggerid'],
+			'selectItemDiscovery' => ['ts_delete'],
+			'triggerids' => array_keys($triggers),
+			'filter' => ['flags' => ZBX_FLAG_DISCOVERY_CREATED]
+		]);
+
+		foreach ($items as $item) {
+			$ts_delete = $item['itemDiscovery']['ts_delete'];
+
+			if ($ts_delete == 0) {
+				continue;
+			}
+
+			foreach (array_column($item['triggers'], 'triggerid') as $triggerid) {
+				if (!array_key_exists($triggerid, $triggers)) {
+					continue;
+				}
+
+				if (!array_key_exists('ts_delete', $triggers[$triggerid]['triggerDiscovery'])) {
+					$triggers[$triggerid]['triggerDiscovery']['ts_delete'] = $ts_delete;
+				}
+				else {
+					$trigger_ts_delete = $triggers[$triggerid]['triggerDiscovery']['ts_delete'];
+					$triggers[$triggerid]['triggerDiscovery']['ts_delete'] = ($trigger_ts_delete > 0)
+						? min($ts_delete, $trigger_ts_delete)
+						: $ts_delete;
+				}
+			}
+		}
+
 		// We must maintain sort order that is applied on prefetched_triggers array.
 		foreach ($triggers as $triggerid => $trigger) {
 			$prefetched_triggers[$triggerid] = $trigger;
@@ -960,14 +1005,6 @@ else {
 		}
 	}
 
-	$config_priorities = [];
-	foreach (range(TRIGGER_SEVERITY_NOT_CLASSIFIED, TRIGGER_SEVERITY_COUNT - 1) as $severity) {
-		$config_priorities[] = [
-			'name' => getSeverityName($severity, $config),
-			'value' => $severity
-		];
-	}
-
 	$single_selected_hostid = 0;
 	if (count($filter_hostids) == 1) {
 		$single_selected_hostid = reset($filter_hostids);
@@ -977,8 +1014,6 @@ else {
 	$checkbox_hash = crc32(implode('', $filter_hostids));
 
 	$data = [
-		'config' => $config,
-		'config_priorities' => $config_priorities,
 		'triggers' => $triggers,
 		'profileIdx' => 'web.triggers.filter',
 		'active_tab' => $active_tab,
@@ -1003,12 +1038,12 @@ else {
 		'parent_templates' => getTriggerParentTemplates($triggers, ZBX_FLAG_DISCOVERY_NORMAL),
 		'paging' => $paging,
 		'dep_triggers' => $dep_triggers,
-		'tags' => makeTags($triggers, true, 'triggerid', ZBX_TAG_COUNT_DEFAULT, $filter_tags)
+		'tags' => makeTags($triggers, true, 'triggerid', ZBX_TAG_COUNT_DEFAULT, $filter_tags),
+		'allowed_ui_conf_templates' => CWebUser::checkAccess(CRoleHelper::UI_CONFIGURATION_TEMPLATES)
 	];
-	$triggersView = new CView('configuration.triggers.list', $data);
 
-	$triggersView->render();
-	$triggersView->show();
+	// render view
+	echo (new CView('configuration.triggers.list', $data))->getOutput();
 }
 
 require_once dirname(__FILE__).'/include/page_footer.php';
